/* CreateDocumentXPath MPEG-7 JRS sample program.
 * 
 * (c) 2003-2009 by JOANNEUM RESEARCH, Graz, Austria.
 */


#include <iostream>
#include <cstdio>
#include <xercesc/util/PlatformUtils.hpp>

#ifdef __APPLE__

#include "Mp7Jrs/Mp7JrsArchive.h"

#include "Mp7Jrs/Mp7JrsException.h"
#include "Mp7Jrs/Mp7JrsFactory.h"
// FTT not used anymore #include "Mp7Jrs/Mp7JrsClass.h"
#include "Mp7Jrs/Mp7JrsMpeg7_LocalType.h"
#include "Mp7Jrs/Mp7JrsMpeg7_Description_CollectionType.h"
#include "Mp7Jrs/Mp7JrsMediaLocatorType.h"
#include "Mp7Jrs/Mp7JrsStillRegionType.h"
#include "Mp7Jrs/Mp7JrsTitleType.h"
#include "Mp7Jrs/Mp7JrsTextualType.h"
#include "Mp7Jrs/Mp7JrsScalableColorType.h"
#include "Mp7Jrs/Mp7JrsintegerVector.h"
#include "Mp7Jrs/Mp7JrsInlineTermDefinitionType_Name_LocalType.h"

#else

#include "Mp7JrsArchive.h"

#include "Mp7JrsException.h"
#include "Mp7JrsFactory.h"
// FTT not used anymore #include "Mp7JrsClass.h"
#include "Mp7JrsMpeg7_LocalType.h"
#include "Mp7JrsMpeg7_Description_CollectionType.h"
#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsStillRegionType.h"
#include "Mp7JrsTitleType.h"
#include "Mp7JrsTextualType.h"
#include "Mp7JrsScalableColorType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsInlineTermDefinitionType_Name_LocalType.h"

#endif

using namespace std;

void createMediaLocator(Mp7JrsNodePtr segment, const char* loc) {
	Mp7JrsNodeEnum mlEnum = segment->GetOrCreateFromXPath(X("MediaLocator"));
	Mp7JrsMediaLocatorPtr mediaLocator = mlEnum.GetNext();
	mediaLocator->SetMediaUri(XMLString::transcode(loc));
}

void setMainTitle(Mp7JrsNodePtr segment, const char* title) {
	Mp7JrsNodeEnum mlEnum = segment->GetOrCreateFromXPath(X("CreationInformation/Creation/Title"));
	Mp7JrsTitlePtr mainTitle = mlEnum.GetNext();
	mainTitle->SetContent(XMLString::transcode(title));
}

void addFreeTextAnnotation(Mp7JrsNodePtr segment, const char* text) {
	Mp7JrsNodeEnum mlEnum = segment->GetOrCreateFromXPath(X("TextAnnotation/FreeTextAnnotation"));
	Mp7JrsTextualPtr freeTextAnnotation = mlEnum.GetNext();
	freeTextAnnotation->SetContent(XMLString::transcode(text));
}

void addSemantic_Label(Mp7JrsNodePtr segment, const char* name) {
	Mp7JrsNodeEnum mlEnum = segment->GetOrCreateFromXPath(X("Semantic/Label/Name"));
	Mp7JrsInlineTermDefinitionType_Name_LocalPtr label = mlEnum.GetNext();
	label->SetContent(XMLString::transcode(name));
}

void addSemantic_Event_Label(Mp7JrsNodePtr segment, const char* name) {
	Mp7JrsNodeEnum mlEnum = segment->GetOrCreateFromXPath(X("Semantic/SemanticBase[@xsi:type=\"EventType\"]/Label/Name"));
	Mp7JrsInlineTermDefinitionType_Name_LocalPtr label = mlEnum.GetNext();
	label->SetContent(XMLString::transcode(name));
}

void addScalableColorD(Mp7JrsNodePtr segment, int* values, int nValues) {
	Mp7JrsNodeEnum mlEnum = segment->GetOrCreateFromXPath(X("VisualDescriptor[@xsi:type=\"ScalableColorType\"]"));
	Mp7JrsScalableColorPtr scd = mlEnum.GetNext();
	scd->SetnumOfCoeff(Mp7JrsScalableColorType_numOfCoeff_LocalType::_16);
	scd->SetnumOfBitplanesDiscarded(Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::_0);

	Mp7JrsintegerVectorPtr intV = CreateintegerVector;
	int i = 0;
	for (int i=0;i<nValues;i++) intV->addElement(values[i]);
	scd->SetCoeff(intV);
}

void rebuildTestTree(Mp7JrsMpeg7_LocalPtr root) {
	// create the image element 
	Mp7JrsNodeEnum imageEnum = root->GetOrCreateFromXPath(X("Description[@xsi:type=\"ContentEntityType\"]/MultimediaContent[@xsi:type=\"ImageType\"]/Image"));

	if (!imageEnum.HasNext()) return;
	Mp7JrsNodePtr image = imageEnum.GetNext();

	// set annotation on image
	createMediaLocator(image,"image.jpg");
	setMainTitle(image, "World Series Game 3");
	addFreeTextAnnotation(image,"Game winning homerun");
	addSemantic_Label(image,"Last inning");
	addSemantic_Event_Label(image,"Game winning homerun");

	int scdValues[16];
	for (int i=0; i<16; i++) scdValues[i] = (i+1)%10;

	addScalableColorD(image, scdValues, 16);
}

int main(int argc, char** argv) {

	XMLPlatformUtils::Initialize();
	Mp7JrsFactory::Initialize();

	// block for smart pointers
	try {
		Mp7JrsArchive archive;
		archive.SetEncoding("UTF-8");
		archive.SetDoValidation(false);
		Mp7JrsNodePtr root = archive.FromFile("test.xml");
		// access via description collection
		Mp7JrsMpeg7_Description_CollectionPtr dcp = ((Mp7JrsMpeg7_LocalPtr)root)->GetDescription();
		// delete all description elements (only root will be left)
		dcp->removeAllElements();
		// example: rebuild the tree using XPath statements
		rebuildTestTree(root);
		archive.ToFile("out.xml", root);
	}
	catch (Mp7JrsException e) {
		printf("Exception: %s\n",e.GetMessage());
	}

	Mp7JrsFactory::Terminate();
	XMLPlatformUtils::Terminate();

	printf("done.");
	getchar();
	return 0;
}
