 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioDType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsAudioDType::IMp7JrsAudioDType()
{

// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtPropInit.h

}

IMp7JrsAudioDType::~IMp7JrsAudioDType()
{
// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtPropCleanup.h

}

Mp7JrsAudioDType::Mp7JrsAudioDType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioDType::~Mp7JrsAudioDType()
{
	Cleanup();
}

void Mp7JrsAudioDType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_channels = Mp7JrsintegerVectorPtr(); // Collection
	m_channels_Exist = false;



// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtMyPropInit.h

}

void Mp7JrsAudioDType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_channels);
}

void Mp7JrsAudioDType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioDTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioDType
    const Mp7JrsPtr< Mp7JrsAudioDType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioDType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_channels);
	if (tmp->Existchannels())
	{
		this->Setchannels(Mp7JrsFactory::CloneObject(tmp->Getchannels()));
	}
	else
	{
		Invalidatechannels();
	}
	}
}

Mp7JrsNodePtr Mp7JrsAudioDType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioDType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsAudioDType::GetBase() const
{
	return m_Base;
}

Mp7JrsintegerVectorPtr Mp7JrsAudioDType::Getchannels() const
{
	return m_channels;
}

bool Mp7JrsAudioDType::Existchannels() const
{
	return m_channels_Exist;
}
void Mp7JrsAudioDType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioDType::Setchannels().");
	}
    m_channels = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_channels) m_channels->SetParent(m_myself.getPointer());
	m_channels_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioDType::Invalidatechannels()
{
	m_channels_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsAudioDType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for AudioDType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "AudioDType");
	}
	
	return result;
}

XMLCh * Mp7JrsAudioDType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioDType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsAudioDType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioDType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_channels_Exist)
	{
	// Collection
	if(m_channels != Mp7JrsintegerVectorPtr())
	{
		XMLCh * tmp = m_channels->ToText();
		element->setAttributeNS(X(""), X("channels"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsAudioDType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("channels")))
	{
        // Deserialize collection type
		Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
		tmp->Parse(parent->getAttributeNS(X(""), X("channels")));
        this->Setchannels(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("channels")))
	{
     
		// Deserialize collection type
        Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
        tmp->Parse(parent->getAttribute(X("channels")));
        this->Setchannels(tmp);
        * current = parent;
    }

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioDType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioDType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioDType_ExtMethodImpl.h


