 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtImplInclude.h



#include "Mp7JrsAudioDType.h"
#include "Mp7JrsAudioDSType.h"
#include "Mp7JrsAudioSegmentType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsAudioSegmentType_LocalType::IMp7JrsAudioSegmentType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtPropInit.h

}

IMp7JrsAudioSegmentType_LocalType::~IMp7JrsAudioSegmentType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtPropCleanup.h

}

Mp7JrsAudioSegmentType_LocalType::Mp7JrsAudioSegmentType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioSegmentType_LocalType::~Mp7JrsAudioSegmentType_LocalType()
{
	Cleanup();
}

void Mp7JrsAudioSegmentType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_AudioDescriptor = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_AudioDescriptionScheme = Mp7JrsPtr< Mp7JrsNode >(); // Class


// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtMyPropInit.h

}

void Mp7JrsAudioSegmentType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_AudioDescriptor);
	// Mp7JrsFactory::DeleteObject(m_AudioDescriptionScheme);
}

void Mp7JrsAudioSegmentType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioSegmentType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioSegmentType_LocalType
    const Mp7JrsPtr< Mp7JrsAudioSegmentType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_AudioDescriptor);
		this->SetAudioDescriptor(Mp7JrsFactory::CloneObject(tmp->GetAudioDescriptor()));
		// Mp7JrsFactory::DeleteObject(m_AudioDescriptionScheme);
		this->SetAudioDescriptionScheme(Mp7JrsFactory::CloneObject(tmp->GetAudioDescriptionScheme()));
}

Mp7JrsNodePtr Mp7JrsAudioSegmentType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsAudioSegmentType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsPtr< Mp7JrsNode > Mp7JrsAudioSegmentType_LocalType::GetAudioDescriptor() const
{
	return m_AudioDescriptor;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsAudioSegmentType_LocalType::GetAudioDescriptionScheme() const
{
	return m_AudioDescriptionScheme;
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioSegmentType_LocalType::SetAudioDescriptor(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSegmentType_LocalType::SetAudioDescriptor().");
	}
	if (m_AudioDescriptor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioDescriptor);
		m_AudioDescriptor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioDescriptor) m_AudioDescriptor->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_AudioDescriptor) m_AudioDescriptor->UseTypeAttribute = true;
		if(m_AudioDescriptor != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioDescriptor->SetContentName(XMLString::transcode("AudioDescriptor"));
		}
	// Mp7JrsFactory::DeleteObject(m_AudioDescriptionScheme);
	m_AudioDescriptionScheme = Mp7JrsPtr< Mp7JrsNode >();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioSegmentType_LocalType::SetAudioDescriptionScheme(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSegmentType_LocalType::SetAudioDescriptionScheme().");
	}
	if (m_AudioDescriptionScheme != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioDescriptionScheme);
		m_AudioDescriptionScheme = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioDescriptionScheme) m_AudioDescriptionScheme->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_AudioDescriptionScheme) m_AudioDescriptionScheme->UseTypeAttribute = true;
		if(m_AudioDescriptionScheme != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioDescriptionScheme->SetContentName(XMLString::transcode("AudioDescriptionScheme"));
		}
	// Mp7JrsFactory::DeleteObject(m_AudioDescriptor);
	m_AudioDescriptor = Mp7JrsPtr< Mp7JrsNode >();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: AudioSegmentType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsAudioSegmentType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsAudioSegmentType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioSegmentType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsAudioSegmentType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_AudioDescriptor != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioDescriptor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioDescriptor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_AudioDescriptor->UseTypeAttribute = true;
		}
		m_AudioDescriptor->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_AudioDescriptionScheme != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioDescriptionScheme->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioDescriptionScheme"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_AudioDescriptionScheme->UseTypeAttribute = true;
		}
		m_AudioDescriptionScheme->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsAudioSegmentType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioDescriptor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioDescriptor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateAudioDType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioDescriptor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioDescriptionScheme"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioDescriptionScheme")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateAudioDSType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioDescriptionScheme(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioSegmentType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("AudioDescriptor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateAudioDType;
    }
    this->SetAudioDescriptor(child);
  }
  if (XMLString::compareString(elementname, X("AudioDescriptionScheme")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateAudioDSType;
    }
    this->SetAudioDescriptionScheme(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioSegmentType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetAudioDescriptor() != Mp7JrsNodePtr())
		result.Insert(GetAudioDescriptor());
	if (GetAudioDescriptionScheme() != Mp7JrsNodePtr())
		result.Insert(GetAudioDescriptionScheme());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioSegmentType_LocalType_ExtMethodImpl.h


