 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtImplInclude.h



#include "Mp7JrsAudioLLDVectorType.h"
#include "Mp7JrsfloatVector.h"
#include "Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioSpectrumEnvelopeType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsAudioLLDVectorType_SeriesOfVector_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SeriesOfVector

#include <assert.h>
IMp7JrsAudioSpectrumEnvelopeType::IMp7JrsAudioSpectrumEnvelopeType()
{

// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtPropInit.h

}

IMp7JrsAudioSpectrumEnvelopeType::~IMp7JrsAudioSpectrumEnvelopeType()
{
// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtPropCleanup.h

}

Mp7JrsAudioSpectrumEnvelopeType::Mp7JrsAudioSpectrumEnvelopeType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioSpectrumEnvelopeType::~Mp7JrsAudioSpectrumEnvelopeType()
{
	Cleanup();
}

void Mp7JrsAudioSpectrumEnvelopeType::Init()
{
	// Init base
	m_Base = CreateAudioLLDVectorType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_loEdge = 0.0; // Value
	m_loEdge_Default = 62.5;
	m_loEdge_Exist = false;
	m_hiEdge = 0.0; // Value
	m_hiEdge_Default = 16000;
	m_hiEdge_Exist = false;
	m_octaveResolution = Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::UninitializedEnumeration;
	m_octaveResolution_Exist = false;



// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtMyPropInit.h

}

void Mp7JrsAudioSpectrumEnvelopeType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsAudioSpectrumEnvelopeType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioSpectrumEnvelopeTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioSpectrumEnvelopeType
    const Mp7JrsPtr< Mp7JrsAudioSpectrumEnvelopeType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioLLDVectorPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioSpectrumEnvelopeType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistloEdge())
	{
		this->SetloEdge(tmp->GetloEdge());
	}
	else
	{
		InvalidateloEdge();
	}
	}
	{
	if (tmp->ExisthiEdge())
	{
		this->SethiEdge(tmp->GethiEdge());
	}
	else
	{
		InvalidatehiEdge();
	}
	}
	{
	if (tmp->ExistoctaveResolution())
	{
		this->SetoctaveResolution(tmp->GetoctaveResolution());
	}
	else
	{
		InvalidateoctaveResolution();
	}
	}
}

Mp7JrsNodePtr Mp7JrsAudioSpectrumEnvelopeType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioSpectrumEnvelopeType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioLLDVectorType > Mp7JrsAudioSpectrumEnvelopeType::GetBase() const
{
	return m_Base;
}

float Mp7JrsAudioSpectrumEnvelopeType::GetloEdge() const
{
	if (this->ExistloEdge()) {
		return m_loEdge;
	} else {
		return m_loEdge_Default;
	}
}

bool Mp7JrsAudioSpectrumEnvelopeType::ExistloEdge() const
{
	return m_loEdge_Exist;
}
void Mp7JrsAudioSpectrumEnvelopeType::SetloEdge(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSpectrumEnvelopeType::SetloEdge().");
	}
    m_loEdge = item;
	m_loEdge_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioSpectrumEnvelopeType::InvalidateloEdge()
{
	m_loEdge_Exist = false;
}
float Mp7JrsAudioSpectrumEnvelopeType::GethiEdge() const
{
	if (this->ExisthiEdge()) {
		return m_hiEdge;
	} else {
		return m_hiEdge_Default;
	}
}

bool Mp7JrsAudioSpectrumEnvelopeType::ExisthiEdge() const
{
	return m_hiEdge_Exist;
}
void Mp7JrsAudioSpectrumEnvelopeType::SethiEdge(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSpectrumEnvelopeType::SethiEdge().");
	}
    m_hiEdge = item;
	m_hiEdge_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioSpectrumEnvelopeType::InvalidatehiEdge()
{
	m_hiEdge_Exist = false;
}
Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::Enumeration Mp7JrsAudioSpectrumEnvelopeType::GetoctaveResolution() const
{
	return m_octaveResolution;
}

bool Mp7JrsAudioSpectrumEnvelopeType::ExistoctaveResolution() const
{
	return m_octaveResolution_Exist;
}
void Mp7JrsAudioSpectrumEnvelopeType::SetoctaveResolution(Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSpectrumEnvelopeType::SetoctaveResolution().");
	}
    m_octaveResolution = item;
	m_octaveResolution_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioSpectrumEnvelopeType::InvalidateoctaveResolution()
{
	m_octaveResolution_Exist = false;
}
Mp7JrsfloatVectorPtr Mp7JrsAudioSpectrumEnvelopeType::GetVector() const
{
	return GetBase()->GetVector();
}

Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr Mp7JrsAudioSpectrumEnvelopeType::GetSeriesOfVector() const
{
	return GetBase()->GetSeriesOfVector();
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioSpectrumEnvelopeType::SetVector(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSpectrumEnvelopeType::SetVector().");
	}
	GetBase()->SetVector(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioSpectrumEnvelopeType::SetSeriesOfVector(const Mp7JrsAudioLLDVectorType_SeriesOfVector_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSpectrumEnvelopeType::SetSeriesOfVector().");
	}
	GetBase()->SetSeriesOfVector(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsAudioSpectrumEnvelopeType::Getchannels() const
{
	return GetBase()->GetBase()->Getchannels();
}

bool Mp7JrsAudioSpectrumEnvelopeType::Existchannels() const
{
	return GetBase()->GetBase()->Existchannels();
}
void Mp7JrsAudioSpectrumEnvelopeType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioSpectrumEnvelopeType::Setchannels().");
	}
	GetBase()->GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioSpectrumEnvelopeType::Invalidatechannels()
{
	GetBase()->GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsAudioSpectrumEnvelopeType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for AudioSpectrumEnvelopeType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "AudioSpectrumEnvelopeType");
	}		
	
	return result;
}

XMLCh * Mp7JrsAudioSpectrumEnvelopeType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioSpectrumEnvelopeType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsAudioSpectrumEnvelopeType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioSpectrumEnvelopeType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_loEdge_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_loEdge);
		element->setAttributeNS(X(""), X("loEdge"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_hiEdge_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_hiEdge);
		element->setAttributeNS(X(""), X("hiEdge"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_octaveResolution_Exist)
	{
	// Enumeration
	if(m_octaveResolution != Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::ToText(m_octaveResolution);
		element->setAttributeNS(X(""), X("octaveResolution"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsAudioSpectrumEnvelopeType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("loEdge")))
	{
        // deserialize value type
        this->SetloEdge(Mp7JrsConvert::TextToFloat(parent->getAttributeNS(X(""), X("loEdge"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("loEdge")))
	{
        // deserialize value type
        this->SetloEdge(Mp7JrsConvert::TextToFloat(parent->getAttribute(X("loEdge"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("hiEdge")))
	{
        // deserialize value type
        this->SethiEdge(Mp7JrsConvert::TextToFloat(parent->getAttributeNS(X(""), X("hiEdge"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("hiEdge")))
	{
        // deserialize value type
        this->SethiEdge(Mp7JrsConvert::TextToFloat(parent->getAttribute(X("hiEdge"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("octaveResolution")))
	{
        this->SetoctaveResolution(Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::Parse(parent->getAttributeNS(X(""), X("octaveResolution"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("octaveResolution")))
	{
        this->SetoctaveResolution(Mp7JrsaudioSpectrumAttributeGrp_octaveResolution_LocalType::Parse(parent->getAttribute(X("octaveResolution"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsAudioLLDVectorType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioSpectrumEnvelopeType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioSpectrumEnvelopeType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVector() != Mp7JrsNodePtr())
		result.Insert(GetVector());
	if (GetSeriesOfVector() != Mp7JrsNodePtr())
		result.Insert(GetSeriesOfVector());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioSpectrumEnvelopeType_ExtMethodImpl.h


