 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtImplInclude.h



#include "Mp7JrsMediaTimeType.h"
#include "Mp7JrsFocusOfExpansionType.h"
#include "Mp7JrsCameraMotionSegmentType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsCameraMotionSegmentType::IMp7JrsCameraMotionSegmentType()
{

// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtPropInit.h

}

IMp7JrsCameraMotionSegmentType::~IMp7JrsCameraMotionSegmentType()
{
// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtPropCleanup.h

}

Mp7JrsCameraMotionSegmentType::Mp7JrsCameraMotionSegmentType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCameraMotionSegmentType::~Mp7JrsCameraMotionSegmentType()
{
	Cleanup();
}

void Mp7JrsCameraMotionSegmentType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_MediaTime = Mp7JrsMediaTimePtr(); // Class
	m_FocusOfExpansion = Mp7JrsFocusOfExpansionPtr(); // Class
	m_FocusOfExpansion_Exist = false;


// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtMyPropInit.h

}

void Mp7JrsCameraMotionSegmentType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_MediaTime);
	// Mp7JrsFactory::DeleteObject(m_FocusOfExpansion);
}

void Mp7JrsCameraMotionSegmentType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CameraMotionSegmentTypePtr, since we
	// might need GetBase(), which isn't defined in ICameraMotionSegmentType
    const Mp7JrsPtr< Mp7JrsCameraMotionSegmentType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_MediaTime);
		this->SetMediaTime(Mp7JrsFactory::CloneObject(tmp->GetMediaTime()));
	if (tmp->IsValidFocusOfExpansion())
	{
		// Mp7JrsFactory::DeleteObject(m_FocusOfExpansion);
		this->SetFocusOfExpansion(Mp7JrsFactory::CloneObject(tmp->GetFocusOfExpansion()));
	}
	else
	{
		InvalidateFocusOfExpansion();
	}
}

Mp7JrsNodePtr Mp7JrsCameraMotionSegmentType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsCameraMotionSegmentType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsMediaTimePtr Mp7JrsCameraMotionSegmentType::GetMediaTime() const
{
	return m_MediaTime;
}

Mp7JrsFocusOfExpansionPtr Mp7JrsCameraMotionSegmentType::GetFocusOfExpansion() const
{
	return m_FocusOfExpansion;
}

// Element is optional
bool Mp7JrsCameraMotionSegmentType::IsValidFocusOfExpansion() const
{
	return m_FocusOfExpansion_Exist;
}

void Mp7JrsCameraMotionSegmentType::SetMediaTime(const Mp7JrsMediaTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCameraMotionSegmentType::SetMediaTime().");
	}
	if (m_MediaTime != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaTime);
		m_MediaTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaTime) m_MediaTime->SetParent(m_myself.getPointer());
		if (m_MediaTime && m_MediaTime->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaTime->UseTypeAttribute = true;
		}
		if(m_MediaTime != Mp7JrsMediaTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaTime->SetContentName(XMLString::transcode("MediaTime"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCameraMotionSegmentType::SetFocusOfExpansion(const Mp7JrsFocusOfExpansionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCameraMotionSegmentType::SetFocusOfExpansion().");
	}
	if (m_FocusOfExpansion != item || m_FocusOfExpansion_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FocusOfExpansion);
		m_FocusOfExpansion = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FocusOfExpansion) m_FocusOfExpansion->SetParent(m_myself.getPointer());
		if (m_FocusOfExpansion && m_FocusOfExpansion->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FocusOfExpansionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FocusOfExpansion->UseTypeAttribute = true;
		}
		m_FocusOfExpansion_Exist = true;
		if(m_FocusOfExpansion != Mp7JrsFocusOfExpansionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FocusOfExpansion->SetContentName(XMLString::transcode("FocusOfExpansion"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCameraMotionSegmentType::InvalidateFocusOfExpansion()
{
	m_FocusOfExpansion_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsCameraMotionSegmentType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("MediaTime")) == 0)
	{
		// MediaTime is simple element MediaTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaTime(p, client);
					if((p = GetMediaTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FocusOfExpansion")) == 0)
	{
		// FocusOfExpansion is simple element FocusOfExpansionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFocusOfExpansion()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FocusOfExpansionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFocusOfExpansionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFocusOfExpansion(p, client);
					if((p = GetFocusOfExpansion()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CameraMotionSegmentType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CameraMotionSegmentType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCameraMotionSegmentType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsCameraMotionSegmentType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsCameraMotionSegmentType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CameraMotionSegmentType"));
	// Element serialization:
	// Class
	
	if (m_MediaTime != Mp7JrsMediaTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaTime->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FocusOfExpansion != Mp7JrsFocusOfExpansionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FocusOfExpansion->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FocusOfExpansion"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FocusOfExpansion->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsCameraMotionSegmentType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaTime"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaTime")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaTimeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaTime(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FocusOfExpansion"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FocusOfExpansion")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateFocusOfExpansionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFocusOfExpansion(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCameraMotionSegmentType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("MediaTime")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaTimeType;
    }
    this->SetMediaTime(child);
  }
  if (XMLString::compareString(elementname, X("FocusOfExpansion")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateFocusOfExpansionType;
    }
    this->SetFocusOfExpansion(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCameraMotionSegmentType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMediaTime() != Mp7JrsNodePtr())
		result.Insert(GetMediaTime());
	if (GetFocusOfExpansion() != Mp7JrsNodePtr())
		result.Insert(GetFocusOfExpansion());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCameraMotionSegmentType_ExtMethodImpl.h


