 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsCameraMotionType_Segment_CollectionType.h"
#include "Mp7JrsCameraMotionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsCameraMotionSegmentType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Segment

#include <assert.h>
IMp7JrsCameraMotionType::IMp7JrsCameraMotionType()
{

// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtPropInit.h

}

IMp7JrsCameraMotionType::~IMp7JrsCameraMotionType()
{
// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtPropCleanup.h

}

Mp7JrsCameraMotionType::Mp7JrsCameraMotionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCameraMotionType::~Mp7JrsCameraMotionType()
{
	Cleanup();
}

void Mp7JrsCameraMotionType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Segment = Mp7JrsCameraMotionType_Segment_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtMyPropInit.h

}

void Mp7JrsCameraMotionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Segment);
}

void Mp7JrsCameraMotionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CameraMotionTypePtr, since we
	// might need GetBase(), which isn't defined in ICameraMotionType
    const Mp7JrsPtr< Mp7JrsCameraMotionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsCameraMotionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Segment);
		this->SetSegment(Mp7JrsFactory::CloneObject(tmp->GetSegment()));
}

Mp7JrsNodePtr Mp7JrsCameraMotionType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsCameraMotionType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsCameraMotionType::GetBase() const
{
	return m_Base;
}

Mp7JrsCameraMotionType_Segment_CollectionPtr Mp7JrsCameraMotionType::GetSegment() const
{
	return m_Segment;
}

void Mp7JrsCameraMotionType::SetSegment(const Mp7JrsCameraMotionType_Segment_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCameraMotionType::SetSegment().");
	}
	if (m_Segment != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Segment);
		m_Segment = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Segment) m_Segment->SetParent(m_myself.getPointer());
		if(m_Segment != Mp7JrsCameraMotionType_Segment_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Segment->SetContentName(XMLString::transcode("Segment"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsCameraMotionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Segment")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Segment is item of abstract type CameraMotionSegmentType
		// in element collection CameraMotionType_Segment_CollectionType
		
		context->Found = true;
		Mp7JrsCameraMotionType_Segment_CollectionPtr coll = GetSegment();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCameraMotionType_Segment_CollectionType;
				SetSegment(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsCameraMotionSegmentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CameraMotionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CameraMotionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCameraMotionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsCameraMotionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsCameraMotionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CameraMotionType"));
	// Element serialization:
	if (m_Segment != Mp7JrsCameraMotionType_Segment_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Segment->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsCameraMotionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Segment"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Segment")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCameraMotionType_Segment_CollectionPtr tmp = CreateCameraMotionType_Segment_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSegment(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCameraMotionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Segment")) == 0))
  {
    Mp7JrsCameraMotionType_Segment_CollectionPtr tmp = CreateCameraMotionType_Segment_CollectionType;
    this->SetSegment(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCameraMotionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSegment() != Mp7JrsNodePtr())
		result.Insert(GetSegment());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCameraMotionType_ExtMethodImpl.h


