 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtImplInclude.h



#include "Mp7JrsColorSpaceType_ColorTransMat_LocalType.h"
#include "Mp7JrsColorSpaceType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsColorSpaceType::IMp7JrsColorSpaceType()
{

// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtPropInit.h

}

IMp7JrsColorSpaceType::~IMp7JrsColorSpaceType()
{
// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtPropCleanup.h

}

Mp7JrsColorSpaceType::Mp7JrsColorSpaceType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsColorSpaceType::~Mp7JrsColorSpaceType()
{
	Cleanup();
}

void Mp7JrsColorSpaceType::Init()
{

	// Init attributes
	m_colorReferenceFlag = false; // Value
	m_colorReferenceFlag_Default = false;
	m_colorReferenceFlag_Exist = false;
	m_type = Mp7JrsColorSpaceType_type_LocalType::UninitializedEnumeration;

	// Init elements (element, union sequence choice all any)
	
	m_ColorTransMat = Mp7JrsColorSpaceType_ColorTransMat_LocalPtr(); // Class
	m_ColorTransMat_Exist = false;


// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtMyPropInit.h

}

void Mp7JrsColorSpaceType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_ColorTransMat);
}

void Mp7JrsColorSpaceType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ColorSpaceTypePtr, since we
	// might need GetBase(), which isn't defined in IColorSpaceType
    const Mp7JrsPtr< Mp7JrsColorSpaceType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	if (tmp->ExistcolorReferenceFlag())
	{
		this->SetcolorReferenceFlag(tmp->GetcolorReferenceFlag());
	}
	else
	{
		InvalidatecolorReferenceFlag();
	}
	}
	{
		this->Settype(tmp->Gettype());
	}
	if (tmp->IsValidColorTransMat())
	{
		// Mp7JrsFactory::DeleteObject(m_ColorTransMat);
		this->SetColorTransMat(Mp7JrsFactory::CloneObject(tmp->GetColorTransMat()));
	}
	else
	{
		InvalidateColorTransMat();
	}
}

Mp7JrsNodePtr Mp7JrsColorSpaceType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsColorSpaceType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


bool Mp7JrsColorSpaceType::GetcolorReferenceFlag() const
{
	if (this->ExistcolorReferenceFlag()) {
		return m_colorReferenceFlag;
	} else {
		return m_colorReferenceFlag_Default;
	}
}

bool Mp7JrsColorSpaceType::ExistcolorReferenceFlag() const
{
	return m_colorReferenceFlag_Exist;
}
void Mp7JrsColorSpaceType::SetcolorReferenceFlag(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsColorSpaceType::SetcolorReferenceFlag().");
	}
    m_colorReferenceFlag = item;
	m_colorReferenceFlag_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsColorSpaceType::InvalidatecolorReferenceFlag()
{
	m_colorReferenceFlag_Exist = false;
}
Mp7JrsColorSpaceType_type_LocalType::Enumeration Mp7JrsColorSpaceType::Gettype() const
{
	return m_type;
}

void Mp7JrsColorSpaceType::Settype(Mp7JrsColorSpaceType_type_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsColorSpaceType::Settype().");
	}
    m_type = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsColorSpaceType_ColorTransMat_LocalPtr Mp7JrsColorSpaceType::GetColorTransMat() const
{
	return m_ColorTransMat;
}

// Element is optional
bool Mp7JrsColorSpaceType::IsValidColorTransMat() const
{
	return m_ColorTransMat_Exist;
}

void Mp7JrsColorSpaceType::SetColorTransMat(const Mp7JrsColorSpaceType_ColorTransMat_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsColorSpaceType::SetColorTransMat().");
	}
	if (m_ColorTransMat != item || m_ColorTransMat_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorTransMat);
		m_ColorTransMat = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorTransMat) m_ColorTransMat->SetParent(m_myself.getPointer());
		if (m_ColorTransMat && m_ColorTransMat->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorSpaceType_ColorTransMat_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorTransMat->UseTypeAttribute = true;
		}
		m_ColorTransMat_Exist = true;
		if(m_ColorTransMat != Mp7JrsColorSpaceType_ColorTransMat_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorTransMat->SetContentName(XMLString::transcode("ColorTransMat"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsColorSpaceType::InvalidateColorTransMat()
{
	m_ColorTransMat_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsColorSpaceType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ColorTransMat")) == 0)
	{
		// ColorTransMat is simple element ColorSpaceType_ColorTransMat_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorTransMat()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorSpaceType_ColorTransMat_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorSpaceType_ColorTransMat_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorTransMat(p, client);
					if((p = GetColorTransMat()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ColorSpaceType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ColorSpaceType");
	}		
	
	return result;
}

XMLCh * Mp7JrsColorSpaceType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsColorSpaceType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsColorSpaceType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ColorSpaceType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_colorReferenceFlag_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_colorReferenceFlag);
		element->setAttributeNS(X(""), X("colorReferenceFlag"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
	// Enumeration
	if(m_type != Mp7JrsColorSpaceType_type_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsColorSpaceType_type_LocalType::ToText(m_type);
		element->setAttributeNS(X(""), X("type"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	// Class
	
	if (m_ColorTransMat != Mp7JrsColorSpaceType_ColorTransMat_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorTransMat->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorTransMat"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorTransMat->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsColorSpaceType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("colorReferenceFlag")))
	{
        // deserialize value type
        this->SetcolorReferenceFlag(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("colorReferenceFlag"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("colorReferenceFlag")))
	{
        // deserialize value type
        this->SetcolorReferenceFlag(Mp7JrsConvert::TextToBool(parent->getAttribute(X("colorReferenceFlag"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("type")))
	{
        this->Settype(Mp7JrsColorSpaceType_type_LocalType::Parse(parent->getAttributeNS(X(""), X("type"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("type")))
	{
        this->Settype(Mp7JrsColorSpaceType_type_LocalType::Parse(parent->getAttribute(X("type"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorTransMat"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorTransMat")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorSpaceType_ColorTransMat_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorTransMat(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsColorSpaceType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("ColorTransMat")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorSpaceType_ColorTransMat_LocalType;
    }
    this->SetColorTransMat(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsColorSpaceType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetColorTransMat() != Mp7JrsNodePtr())
		result.Insert(GetColorTransMat());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsColorSpaceType_ExtMethodImpl.h


