 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtImplInclude.h



#include "Mp7JrsCompleteDescriptionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsContentManagementType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey

#include <assert.h>
IMp7JrsContentManagementType::IMp7JrsContentManagementType()
{

// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtPropInit.h

}

IMp7JrsContentManagementType::~IMp7JrsContentManagementType()
{
// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtPropCleanup.h

}

Mp7JrsContentManagementType::Mp7JrsContentManagementType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsContentManagementType::~Mp7JrsContentManagementType()
{
	Cleanup();
}

void Mp7JrsContentManagementType::Init()
{
	// Init base
	m_Base = CreateCompleteDescriptionType;
	m_Base->m_myself = m_myself.getPointer();
	




// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtMyPropInit.h

}

void Mp7JrsContentManagementType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsContentManagementType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ContentManagementTypePtr, since we
	// might need GetBase(), which isn't defined in IContentManagementType
    const Mp7JrsPtr< Mp7JrsContentManagementType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsCompleteDescriptionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsContentManagementType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
}

Mp7JrsNodePtr Mp7JrsContentManagementType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsContentManagementType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsCompleteDescriptionType > Mp7JrsContentManagementType::GetBase() const
{
	return m_Base;
}

Mp7JrsDescriptionMetadataPtr Mp7JrsContentManagementType::GetDescriptionMetadata() const
{
	return GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsContentManagementType::IsValidDescriptionMetadata() const
{
	return GetBase()->IsValidDescriptionMetadata();
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsContentManagementType::GetRelationships() const
{
	return GetBase()->GetRelationships();
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsContentManagementType::GetOrderingKey() const
{
	return GetBase()->GetOrderingKey();
}

void Mp7JrsContentManagementType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContentManagementType::SetDescriptionMetadata().");
	}
	GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsContentManagementType::InvalidateDescriptionMetadata()
{
	GetBase()->InvalidateDescriptionMetadata();
}
void Mp7JrsContentManagementType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContentManagementType::SetRelationships().");
	}
	GetBase()->SetRelationships(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsContentManagementType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContentManagementType::SetOrderingKey().");
	}
	GetBase()->SetOrderingKey(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsContentManagementType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ContentManagementType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ContentManagementType");
	}		
	
	return result;
}

XMLCh * Mp7JrsContentManagementType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsContentManagementType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsContentManagementType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ContentManagementType"));
	// Element serialization:

}

bool Mp7JrsContentManagementType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsCompleteDescriptionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsContentManagementType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsContentManagementType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsContentManagementType_ExtMethodImpl.h


