
// Based on Convert_cpp.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"


#if defined(XML_WIN32) || defined(WIN32) // Xerces 3.0 doesn't know XML_WIN32 and XML_VISUALCPP
#	if defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP)
#		include <stdio.h> // sprintf
#		include <stdlib.h> // atoi, atof
#	endif
#elif defined(XML_MACOS) || defined(APPLE) // Xerces 3.0 doesn't know XML_MACOS
#		include <stdio.h> // sprintf
#		include <stdlib.h> // atoi, atof
#elif defined(XML_LINUX) || defined(LINUX) // Xerces 3.0 doesn't know XML_LINUX
#		include <stdio.h> // sprintf
#		include <stdlib.h> // atoi, atof
#else // XML_LINUX ...
#	error "TODO: Add platform specific headers for conversion funcs."
#endif


XMLCh * Mp7JrsConvert::BinToText(int src)
{
	XMLCh * buf = Mp7JrsUtil::allocate(32); // new XMLCh[32];
	XMLString::binToText((int)src, buf, 31, 10);
	return buf;
}

XMLCh * Mp7JrsConvert::BinToText(unsigned int src)
{
	XMLCh * buf = Mp7JrsUtil::allocate(32); // new XMLCh[32];
	XMLString::binToText(src, buf, 31, 10);
	return buf;
}

XMLCh * Mp7JrsConvert::BinToText(float src)
{
#if defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP) || defined (XML_MACOS) || defined(XML_LINUX)
	char buf[64] = {0};
	sprintf(buf, "%f", src);
	return XMLString::transcode(buf);
#else	
#	error "TODO: Add platform specific code to convert data"
#endif
}

XMLCh * Mp7JrsConvert::BinToText(double src)
{
#if defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP)
	char buf[128] = {0};
	_snprintf(buf, 127, "%lf", src);
	return XMLString::transcode(buf);
#elif defined(XML_MACOS) || defined(XML_LINUX) 	
	char buf[128] = {0};
	snprintf(buf, 127, "%lf", src);
	return XMLString::transcode(buf);
#else	
#	error "TODO: Add platform specific code to convert data"
#endif
}

int Mp7JrsConvert::TextToInt(const XMLCh* text)
{
	int result = 0;
	if(text != NULL)
	{
#if	defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP) || defined(XML_MACOS) || defined(XML_LINUX)
		char * buf = XMLString::transcode(text);
		result = atoi(buf);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
#		else
#			error "TODO: Add platform specific code to convert data"		
#		endif
	}
	return result;
}

unsigned int Mp7JrsConvert::TextToUnsignedInt(const XMLCh * text)
{
	double result = 0.0;
	if(text != NULL)
	{
#		if defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP) || defined(XML_MACOS) || defined(XML_LINUX)
		char * buf = XMLString::transcode(text);
		result = atof(buf);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
#		else
#			error "TODO: Add platform specific code to convert data"		
#		endif
	}
	return (unsigned int) result;
}

bool Mp7JrsConvert::TextToBool(const XMLCh* text)
{
	bool result = false;
	if(text != NULL)
	{
#		if defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP) || defined(XML_MACOS) || defined(XML_LINUX)
		if(XMLString::compareString(text, X("1")) == 0 
		|| XMLString::compareString(text, X("true")) == 0)
			return true;
		else 
		if(XMLString::compareString(text, X("0")) == 0
		|| XMLString::compareString(text, X("false")) == 0)
			return false;
#		else
#			error "TODO: Add platform specific code to convert data"		
#		endif
	}
	return result;
}

XMLCh * Mp7JrsConvert::TextToString(const XMLCh * text)
{
	if(text != NULL)
	{
		return XMLString::replicate(text);
	}
	return (XMLCh*) NULL;
}

float Mp7JrsConvert::TextToFloat(const XMLCh * text)
{
	double result = 0.0;
	if(text != NULL)
	{
#		if defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP) || defined(XML_MACOS) || defined(XML_LINUX)
		char * buf = XMLString::transcode(text);
		result = atof(buf);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
#		else
#			error "TODO: Add platform specific code to convert data"		
#		endif
	}
	return (float)result;
}

double Mp7JrsConvert::TextToDouble(const XMLCh * text)
{
	double result = 0.0;
	if(text != NULL)
	{
#		if  defined(WIN32) || defined(XML_BORLAND) || defined(XML_VISUALCPP) || defined(XML_MACOS) || defined(XML_LINUX)
		char * buf = XMLString::transcode(text);
		result = atof(buf);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
#		else
#			error "TODO: Add platform specific code to convert data"		
#		endif
	}
	return result;
}

// Convert "-" or "+" to -1 or +1, must be able to deal with strings like "-123"
int Mp7JrsConvert::TextToSign(const XMLCh * text)
{
	int result = +1; // default is positive sign
	if(text != NULL)
	{
		if(XMLString::startsWith(text, X("-")))
		{
			return -1;
		}
	}
	return result;
}
