 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtImplInclude.h



#include "Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionType.h"
#include "Mp7JrsDiscreteDistributionType.h"
#include "Mp7JrsDiscreteHiddenMarkovModelType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsDiscreteHiddenMarkovModelType_LocalType::IMp7JrsDiscreteHiddenMarkovModelType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtPropInit.h

}

IMp7JrsDiscreteHiddenMarkovModelType_LocalType::~IMp7JrsDiscreteHiddenMarkovModelType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtPropCleanup.h

}

Mp7JrsDiscreteHiddenMarkovModelType_LocalType::Mp7JrsDiscreteHiddenMarkovModelType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsDiscreteHiddenMarkovModelType_LocalType::~Mp7JrsDiscreteHiddenMarkovModelType_LocalType()
{
	Cleanup();
}

void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_NumOfObservationsPerState = (0); // Value

	m_Observation = Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr(); // Collection
	m_ObservationDistribution = Mp7JrsPtr< Mp7JrsNode >(); // Class


// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtMyPropInit.h

}

void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Observation);
	// Mp7JrsFactory::DeleteObject(m_ObservationDistribution);
}

void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use DiscreteHiddenMarkovModelType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IDiscreteHiddenMarkovModelType_LocalType
    const Mp7JrsPtr< Mp7JrsDiscreteHiddenMarkovModelType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		this->SetNumOfObservationsPerState(tmp->GetNumOfObservationsPerState());
		// Mp7JrsFactory::DeleteObject(m_Observation);
		this->SetObservation(Mp7JrsFactory::CloneObject(tmp->GetObservation()));
		// Mp7JrsFactory::DeleteObject(m_ObservationDistribution);
		this->SetObservationDistribution(Mp7JrsFactory::CloneObject(tmp->GetObservationDistribution()));
}

Mp7JrsNodePtr Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


unsigned Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetNumOfObservationsPerState() const
{
	return m_NumOfObservationsPerState;
}

Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetObservation() const
{
	return m_Observation;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetObservationDistribution() const
{
	return m_ObservationDistribution;
}

void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::SetNumOfObservationsPerState(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDiscreteHiddenMarkovModelType_LocalType::SetNumOfObservationsPerState().");
	}
	if (m_NumOfObservationsPerState != item)
	{
		m_NumOfObservationsPerState = item;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::SetObservation(const Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDiscreteHiddenMarkovModelType_LocalType::SetObservation().");
	}
	if (m_Observation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Observation);
		m_Observation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Observation) m_Observation->SetParent(m_myself.getPointer());
		if(m_Observation != Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Observation->SetContentName(XMLString::transcode("Observation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::SetObservationDistribution(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDiscreteHiddenMarkovModelType_LocalType::SetObservationDistribution().");
	}
	if (m_ObservationDistribution != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ObservationDistribution);
		m_ObservationDistribution = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ObservationDistribution) m_ObservationDistribution->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_ObservationDistribution) m_ObservationDistribution->UseTypeAttribute = true;
		if(m_ObservationDistribution != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ObservationDistribution->SetContentName(XMLString::transcode("ObservationDistribution"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: DiscreteHiddenMarkovModelType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsDiscreteHiddenMarkovModelType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsDiscreteHiddenMarkovModelType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsDiscreteHiddenMarkovModelType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("DiscreteHiddenMarkovModelType_LocalType"));
	// Element serialization:
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_NumOfObservationsPerState);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("NumOfObservationsPerState"), false);
		XMLString::release(&tmp);
		
	}
	if (m_Observation != Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Observation->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_ObservationDistribution != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ObservationDistribution->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ObservationDistribution"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_ObservationDistribution->UseTypeAttribute = true;
		}
		m_ObservationDistribution->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsDiscreteHiddenMarkovModelType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("NumOfObservationsPerState"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetNumOfObservationsPerState(Mp7JrsConvert::TextToUnsignedInt(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Observation"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Observation")) == 0))
        {
            // Deserialize factory type
            Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr tmp = CreateDiscreteHiddenMarkovModelType_Observation_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetObservation(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ObservationDistribution"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ObservationDistribution")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDiscreteDistributionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetObservationDistribution(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsDiscreteHiddenMarkovModelType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Observation")) == 0))
  {
    Mp7JrsDiscreteHiddenMarkovModelType_Observation_CollectionPtr tmp = CreateDiscreteHiddenMarkovModelType_Observation_CollectionType;
    this->SetObservation(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("ObservationDistribution")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDiscreteDistributionType;
    }
    this->SetObservationDistribution(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsDiscreteHiddenMarkovModelType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetObservation() != Mp7JrsNodePtr())
		result.Insert(GetObservation());
	if (GetObservationDistribution() != Mp7JrsNodePtr())
		result.Insert(GetObservationDistribution());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsDiscreteHiddenMarkovModelType_LocalType_ExtMethodImpl.h


