 

// Core library class
// based on NodeRefCollectionType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


#include <stdio.h>
#include "Mp7JrsDominantColorType_Value_Index_CollectionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsDominantColorType_Value_Index_CollectionType_ExtImplInclude.h


#include "Mp7Jrsunsigned12.h" // Collection itemtype



Mp7JrsDominantColorType_Value_Index_CollectionType::Mp7JrsDominantColorType_Value_Index_CollectionType()
{
	m_Item = new Mp7JrsValueVectorOf<Mp7JrsNodePtr>(0); // XXX
	dontSerializeWithContentName = true; // HACK this is beta, next time make it better!

// no includefile for extension defined 
// file Mp7JrsDominantColorType_Value_Index_CollectionType_ExtPropInit.h


}

Mp7JrsDominantColorType_Value_Index_CollectionType::~Mp7JrsDominantColorType_Value_Index_CollectionType()
{
// no includefile for extension defined 
// file Mp7JrsDominantColorType_Value_Index_CollectionType_ExtPropCleanup.h


	delete m_Item;
}

void Mp7JrsDominantColorType_Value_Index_CollectionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	const Mp7JrsDominantColorType_Value_Index_CollectionPtr tmp = original;
	if (tmp == NULL) return;  // EXIT: passed argument of wrong type

	removeAllElements();
	for(unsigned int i = 0; i < tmp->size(); i++)
	{
		Mp7JrsNodePtr dummy = Mp7JrsFactory::CloneObject(tmp->elementAt(i));
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (dummy) dummy->SetParent(m_myself.getPointer());
		m_Item->addElement(dummy);
	}
}

// no includefile for extension defined 
// file Mp7JrsDominantColorType_Value_Index_CollectionType_ExtMethodImpl.h


void Mp7JrsDominantColorType_Value_Index_CollectionType::Initialize(unsigned int maxElems)
{
	m_Item->ensureExtraCapacity(maxElems);
}

void Mp7JrsDominantColorType_Value_Index_CollectionType::addElement(const Mp7JrsNodePtr &toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType_Value_Index_CollectionType::addElement().");
	}
	m_Item->addElement(toAdd);
	if (toAdd) {
		// sat 2004-05-05: factor this out into method when there's a spare hour to compile ...
		toAdd->SetParent(m_myself.getPointer());
		if (!toAdd->GetContentName()) {
			// It's a LocalType (presumably) -- the content name isn't used.
			toAdd->SetContentName(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""));
		}
		
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_ADDED,
		client, toAdd);
}

void Mp7JrsDominantColorType_Value_Index_CollectionType::setElementAt(const Mp7JrsNodePtr &toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType_Value_Index_CollectionType::setElementAt().");
	}
	Mp7JrsNodePtr previous = m_Item->elementAt(setAt);
	if (previous) previous->SetParent(Mp7JrsNodePtr());
	m_Item->setElementAt(toSet, setAt);

	if (toSet) {
		// sat 2004-05-05: factor this out into method when there's a spare hour to compile ...
		toSet->SetParent(m_myself.getPointer());
		if (!toSet->GetContentName()) {
			// It's a LocalType (presumably) -- the content name isn't used.
			toSet->SetContentName(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""));
		}
		
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, toSet, setAt);
}

void Mp7JrsDominantColorType_Value_Index_CollectionType::insertElementAt(const Mp7JrsNodePtr &toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType_Value_Index_CollectionType::insertElementAt().");
	}
	m_Item->insertElementAt(toInsert, insertAt);
	if (toInsert) {
		// sat 2004-05-05: factor this out into method when there's a spare hour to compile ...
		toInsert->SetParent(m_myself.getPointer());
		if (!toInsert->GetContentName()) {
			// It's a LocalType (presumably) -- the content name isn't used.
			toInsert->SetContentName(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""));
		}
		
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_INSERTED,
		client, toInsert, insertAt);
}

/*
Mp7JrsNodePtr Mp7JrsDominantColorType_Value_Index_CollectionType::orphanElementAt(unsigned int orphanAt)
{
	return m_Item->orphanElementAt(orphanAt);
}
*/

void Mp7JrsDominantColorType_Value_Index_CollectionType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType_Value_Index_CollectionType::removeAllElements().");
	}

	for (unsigned int i = 0; i < m_Item->size(); i++) {
		// FIXME: handle elementAt(removeAt) == NULL
		m_Item->elementAt(i)->SetParent(Mp7JrsNodePtr());
	}

	m_Item->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDominantColorType_Value_Index_CollectionType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType_Value_Index_CollectionType::removeElementAt().");
	}
	// FIXME: handle elementAt(removeAt) == NULL
	m_Item->elementAt(removeAt)->SetParent(Mp7JrsNodePtr());
	m_Item->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsDominantColorType_Value_Index_CollectionType::containsElement(const Mp7JrsNodePtr &toCheck)
{
	return m_Item->containsElement(toCheck);
}

unsigned int Mp7JrsDominantColorType_Value_Index_CollectionType::curCapacity() const
{
	return m_Item->curCapacity();
}

const Mp7JrsNodePtr Mp7JrsDominantColorType_Value_Index_CollectionType::elementAt(unsigned int getAt) const
{
	return m_Item->elementAt(getAt);
}

Mp7JrsNodePtr Mp7JrsDominantColorType_Value_Index_CollectionType::elementAt(unsigned int getAt)
{
	return m_Item->elementAt(getAt);
}

Mp7JrsNodeEnum Mp7JrsDominantColorType_Value_Index_CollectionType::GetElementsOfType(unsigned int id) const
{
	Mp7JrsEnumerator< Mp7JrsNodePtr > result;
	for (unsigned int i = 0; i < size(); i++)
	{
		if (elementAt(i)->GetClassId() == id)
		{
			result.Insert(elementAt(i));
		}
	}
	return result;
}

Mp7JrsNodeEnum Mp7JrsDominantColorType_Value_Index_CollectionType::GetElements() const
{
	Mp7JrsEnumerator< Mp7JrsPtr<Mp7JrsNode> > result;
	for (unsigned int i = 0; i < size(); i++)
	{
		result.Insert(elementAt(i));
	}
	return result;
}

unsigned int Mp7JrsDominantColorType_Value_Index_CollectionType::size() const
{
	return m_Item->size();
}

void Mp7JrsDominantColorType_Value_Index_CollectionType::ensureExtraCapacity(unsigned int length)
{
	m_Item->ensureExtraCapacity(length);
}

// JRS: addition to Xerces collection
int Mp7JrsDominantColorType_Value_Index_CollectionType::elementIndexOf(const Mp7JrsNodePtr &toCheck) const
{
	for(unsigned int i = 0; i < size(); i++)
	{
		if(elementAt(i) == toCheck)
		{
			return (int) i;
		}
	}
	return -1; // not found
}

XMLCh * Mp7JrsDominantColorType_Value_Index_CollectionType::ToText() const
{
	XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
	XMLCh * delim = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(" ");
	for(unsigned int i = 0; i < size(); i++)
	{
		XMLCh * tmp = (elementAt(i))->ToText();
		Mp7JrsUtil::CatString(&buf, tmp);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		if(i < size() - 1)
		{
			Mp7JrsUtil::CatString(&buf, delim);
		}
	}
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&delim);
	// Note: You must release this buffer
	return buf;
}

bool Mp7JrsDominantColorType_Value_Index_CollectionType::Parse(const XMLCh * const txt)
{
	if(txt == NULL)
	{
		return false;
	}
	// simple type collection 
	XERCES_CPP_NAMESPACE_QUALIFIER BaseRefVectorOf<XMLCh> * tmp = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::tokenizeString(txt);
	if(tmp != NULL)
	{
		for(unsigned int i = 0; i < tmp->size(); i++)
		{
			Mp7JrsNodePtr item = Mp7Jrsunsigned12Ptr();
			item = Createunsigned12;
			if(item->Parse(tmp->elementAt(i)))
			{
				addElement(item);
			}
		}
	}
	delete tmp;
	return true;
}

/** 
 * Item type is TextOnly. 
 */
void Mp7JrsDominantColorType_Value_Index_CollectionType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
	// The element we serialize into
	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
	if (!newElem) newElem = &localNewElem;

	if (*newElem) {
		element = *newElem;
	} else if(parent) {
		element = parent;
	} else {
		element = doc->getDocumentElement();
	}

	// This is for simpleTypelist collections
	if(parent && !(*newElem) && this->GetContentName() != (XMLCh*)NULL && !dontSerializeWithContentName)
	{
//OLD		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
		DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
		element->appendChild(elem);
		*newElem = elem;
		element = elem;
	}
// OLD	if(this->UseTypeAttribute && ! element->hasAttribute(X("1")))
//	{
//		// FTT TODO check this without enums
//		element->setAttribute(X("xsi:type"), X("unsigned12"));
//	}
  Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("unsigned12"));


// no includefile for extension defined 
// file Mp7JrsDominantColorType_Value_Index_CollectionType_ExtPreSerialize.h


	for(unsigned int i = 0; i < size(); i++)
	{
		// Collection is SimpleTypeList -> serialize text of elements
		XMLCh *text = elementAt(i)->ToText();
		element->appendChild(doc->createTextNode(text));
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&text);
		if(i < size() - 1)
		{
			element->appendChild(doc->createTextNode(X(" ")));
		}
	}
}

bool Mp7JrsDominantColorType_Value_Index_CollectionType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsDominantColorType_Value_Index_CollectionType has no attributes so forward it to Parse()
	return Parse(txt);
}

XMLCh* Mp7JrsDominantColorType_Value_Index_CollectionType::ContentToString() const
{
	// Mp7JrsDominantColorType_Value_Index_CollectionType has no attributes so forward it to ToText()
	return ToText();
}

bool Mp7JrsDominantColorType_Value_Index_CollectionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(parent == NULL)
	{
		return false;
	}
	* current = parent; // Init with first element - check this!

	// Deserialize collection
	// Itemtype = TextOnly
	// Collection is simpleType list
	BaseRefVectorOf<XMLCh> * tmp = XMLString::tokenizeString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		for(unsigned int i = 0; i < tmp->size(); i++)
		{
			Mp7JrsNodePtr item = Mp7Jrsunsigned12Ptr();
			NamespaceHelper xsitype;
			if(Mp7JrsUtil::GetXsiType(parent, doc, &xsitype))
// OLD			if(parent->hasAttribute(X("xsi:type")))
  		{
  			XMLCh * tname = Mp7JrsUtil::QualifiedName(&xsitype);
  			unsigned int id = Mp7JrsFactory::GetTypeIndex(tname);
	  		if(tname) Mp7JrsUtil::deallocate(&tname);
// OLD				unsigned int id = Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type")));
				item = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject(id));
				item->UseTypeAttribute = true;
			}
			else
			{
				item = Createunsigned12;
			}
			if(item->Parse(tmp->elementAt(i)))
			{
				addElement(item);
				// TODO Check do we need elementnames?
			}
		}
	}
	delete tmp;

// no includefile for extension defined 
// file Mp7JrsDominantColorType_Value_Index_CollectionType_ExtPostDeserialize.h


	return true;
}

inline
bool am_in_collection(int next_slash, int next_bracket, int next_close_bracket)
{
	return next_bracket >= 0
		&& next_close_bracket > next_bracket + 1 // at least one digit
		&& (next_slash < 0		// either at end of string ...
			|| next_slash > next_close_bracket); // ... or collection
												 // is in current part
}

Mp7JrsNodePtr Mp7JrsDominantColorType_Value_Index_CollectionType::NodeFromXPath(const XMLCh *xpath) const
{
	const XMLCh *work_string = xpath;
	const XMLCh *content_name = GetContentName();
	Mp7JrsNodePtr me = m_myself.getPointer();
	Mp7JrsNodeCollectionPtr me_as_collection = me;
	Mp7JrsNodePtr parent = GetParent();

	if (XMLString::startsWith(work_string, X("/"))) {
		if (!parent)
			work_string++;		// consume leading slash
		else {
			Mp7JrsNodePtr root = parent;
			while (root->GetParent())
				root = root->GetParent();
			return root->NodeFromXPath(work_string); // EXIT: parse from root
		}
	}

	if (XMLString::stringLen(work_string) == 0)
		return me;

	if (XMLString::startsWith(work_string, X("..")))
		return UpwardWithXPath(work_string);

	XMLCh slash = X("/")[0];
	XMLCh bracket = X("[")[0];
	XMLCh close_bracket = X("]")[0];
	int next_slash = XMLString::indexOf(work_string, slash);
	int next_bracket = XMLString::indexOf(work_string, bracket);
	int next_close_bracket = XMLString::indexOf(work_string, close_bracket);

	// We're a collection -- find n, nth element, parse from there
	int n;
	if (!am_in_collection(next_slash, next_bracket, next_close_bracket))
	{
		if (!content_name) {
			// A collection can be a a choice too; there's a valid
			// test case where we needed to descend into a
			// collection for the xpath string "Name".  The
			// "!content_name" clause is just a futile attempt to
			// cut back the lossage a bit; I'm assuming that none
			// of these freak collections has it set.
			Mp7JrsNodeEnum children = me_as_collection->GetElements();
			while (children.HasNext()) {
				Mp7JrsNodePtr result = children.GetNext()->NodeFromXPath(xpath);
				if (result != Mp7JrsNodePtr())
					return result;	// EXIT: success!
			}
		}
		// fall-through case: no collection index found, malformed
		// xpath string or xpath string didn't match the children.
		return Mp7JrsNodePtr(); // EXIT
	}
	XMLCh *n_str = Mp7JrsUtil::allocate(next_close_bracket - next_bracket + 1);
	XMLString::moveChars(n_str, work_string + next_bracket + 1,
		next_close_bracket - next_bracket - 1);
	n = XMLString::parseInt(n_str) - 1;	// XPath indices start
	// with 1, ours start with
	// 0 -- adjust
	XMLString::release(&n_str);
	if (n >= 0 && n < (int)me_as_collection->size()) {
		// if the parent is a LocalType, our ContentName has to
		// match; otherwise, it's allowed to be unset.  if it's
		// set, it has to match.
		if ((content_name
				&& XMLString::startsWith(work_string, content_name)
				&& !XMLString::isAlphaNum(
					work_string[XMLString::stringLen(content_name)])))
		{
			if (next_slash != -1)
				if (dontSerializeWithContentName)
					return me_as_collection->elementAt(n)->NodeFromXPath(
						work_string);
				else
					return me_as_collection->elementAt(n)->NodeFromXPath(
						work_string + next_slash + 1);
			else
				// EXIT: success!  (xpath ended with ".../elemName[n]")
				return me_as_collection->elementAt(n);
		}
		else if (!content_name)
		{
			// It's probably a local collection (collection ->
			// LocalType -> Type) -- the name will be picked up by
			// the Type node, so pass in full work string here.
			return me_as_collection->elementAt(n)->NodeFromXPath(
				work_string); // EXIT
		}
		else
			return Mp7JrsNodePtr(); // EXIT: wrong content name
	} else
	return Mp7JrsNodePtr();	// EXIT: index out of range

}

