
// Based on RefCollectionType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include "Mp7JrsElectronicAddressType_Telephone_CollectionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_Telephone_CollectionType_ExtImplInclude.h


Mp7JrsElectronicAddressType_Telephone_CollectionType::Mp7JrsElectronicAddressType_Telephone_CollectionType()
{
	m_Item = new XERCES_CPP_NAMESPACE_QUALIFIER RefVectorOf <XMLCh>(0);

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_Telephone_CollectionType_ExtPropInit.h

}

Mp7JrsElectronicAddressType_Telephone_CollectionType::~Mp7JrsElectronicAddressType_Telephone_CollectionType()
{

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_Telephone_CollectionType_ExtPropCleanup.h


	delete m_Item;
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	const Mp7JrsElectronicAddressType_Telephone_CollectionPtr tmp = original;
	if (tmp == NULL) return;  // EXIT: wrong argument type

	m_Item->removeAllElements();
	for(unsigned int i = 0; i < tmp->size(); i++)
	{
		// This is necessary when Xerces and Mp7Jrs are in
        // different libraries
		XMLCh *text = tmp->elementAt(i);
		XMLCh *ours = new XMLCh[XMLString::stringLen(text) + 1];
		XMLString::copyString(ours, text);
		m_Item->addElement(ours);
	}
}

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_Telephone_CollectionType_ExtMethodImpl.h


void Mp7JrsElectronicAddressType_Telephone_CollectionType::Initialize(unsigned int maxElems)
{
	m_Item->ensureExtraCapacity(maxElems);
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::addElement(XMLCh * const toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType_Telephone_CollectionType::addElement().");
	}
	
	// baw 27 04 2004: copy string to ensure that the string is allocated in Mp7Jrs
	XMLCh *ours = new XMLCh[XMLString::stringLen(toAdd) + 1];
	XMLString::copyString(ours, toAdd);
	m_Item->addElement(ours);	
		
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::setElementAt(XMLCh * const toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType_Telephone_CollectionType::setElementAt().");
	}
	
	// baw 27 04 2004: copy string to ensure that the string is allocated in Mp7Jrs
	XMLCh *ours = new XMLCh[XMLString::stringLen(toSet) + 1];
	XMLString::copyString(ours, toSet);
	m_Item->setElementAt(ours,setAt);

	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::insertElementAt(XMLCh * const toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType_Telephone_CollectionType::insertElementAt().");
	}

	// baw 27 04 2004: copy string to ensure that the string is allocated in Mp7Jrs
	XMLCh *ours = new XMLCh[XMLString::stringLen(toInsert) + 1];
	XMLString::copyString(ours, toInsert);
	m_Item->insertElementAt(ours,insertAt);

	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsElectronicAddressType_Telephone_CollectionType::orphanElementAt(unsigned int orphanAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType_Telephone_CollectionType::orphanElementAt().");
	}
	return m_Item->orphanElementAt(orphanAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType_Telephone_CollectionType::removeAllElements().");
	}
	m_Item->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsElectronicAddressType_Telephone_CollectionType::removeElementAt().");
	}
	m_Item->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsElectronicAddressType_Telephone_CollectionType::containsElement(const XMLCh * toCheck)
{
	return m_Item->containsElement(toCheck);
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::cleanup()
{
	m_Item->cleanup();
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::reinitialize()
{
	m_Item->reinitialize();
}

unsigned int Mp7JrsElectronicAddressType_Telephone_CollectionType::curCapacity() const
{
	return m_Item->curCapacity();
}

const XMLCh * Mp7JrsElectronicAddressType_Telephone_CollectionType::elementAt(unsigned int getAt) const
{
	return m_Item->elementAt(getAt);
}

XMLCh * Mp7JrsElectronicAddressType_Telephone_CollectionType::elementAt(unsigned int getAt)
{
	return m_Item->elementAt(getAt);
}

unsigned int Mp7JrsElectronicAddressType_Telephone_CollectionType::size() const
{
	return m_Item->size();
}

void Mp7JrsElectronicAddressType_Telephone_CollectionType::ensureExtraCapacity(unsigned int length)
{
	m_Item->ensureExtraCapacity(length);
}

// JRS: addition to Xerces collection
int Mp7JrsElectronicAddressType_Telephone_CollectionType::elementIndexOf(XMLCh * const toCheck) const
{
	for(unsigned int i = 0; i < m_Item->size(); i++)
	{
		if(m_Item->elementAt(i) == toCheck)
		{
			return (int) i;
		}
	}
	return -1;
}

XMLCh * Mp7JrsElectronicAddressType_Telephone_CollectionType::ToText() const
{
	XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
	XMLCh * delim = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(" ");
	for(unsigned int i = 0; i < m_Item->size(); i++)
	{
		XMLCh * tmp = (m_Item->elementAt(i));
		Mp7JrsUtil::CatString(&buf, tmp);
		if(i < m_Item->size() - 1)
		{
			Mp7JrsUtil::CatString(&buf, delim);
		}
	}
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&delim);
	// Note: You must release this buffer;
	return buf;
}

bool Mp7JrsElectronicAddressType_Telephone_CollectionType::Parse(const XMLCh * const txt)
{
	if(txt == NULL)
	{
		return false;
	}
	XERCES_CPP_NAMESPACE_QUALIFIER BaseRefVectorOf<XMLCh> * tmp = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::tokenizeString(txt);
	if(tmp != NULL)
	{
		for(unsigned int i = 0; i < tmp->size(); i++)
		{
			// This is necessary when Xerces and Mp7Jrs are in
            // different libraries
			XMLCh *text = tmp->elementAt(i);
			XMLCh *ours = new XMLCh[XMLString::stringLen(text) + 1];
			XMLString::copyString(ours, text);
			m_Item->addElement(ours);
		}
	}
	delete tmp; 
	return m_Item->size() > 0;
}


void Mp7JrsElectronicAddressType_Telephone_CollectionType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent,XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
	// The element we serialize into
	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
	if (!newElem) newElem = &localNewElem;

	if (*newElem) {
		element = *newElem;
	} else if(parent) {
		element = parent;
	} else {
		element = doc->getDocumentElement();
	}

// no includefile for extension defined 
// file Mp7JrsElectronicAddressType_Telephone_CollectionType_ExtPreSerialize.h


	// Collection is not simpleType list
	if (GetContentName()) {
		for(unsigned int i = 0; i < m_Item->size(); i++)
		{
// OLD			XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
      DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			element->appendChild(elem);
			elem->appendChild(doc->createTextNode(m_Item->elementAt(i)));
		}
	}
}

bool Mp7JrsElectronicAddressType_Telephone_CollectionType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
	// Deserialize collection
	if(parent == NULL)
	{
		return false;
	}

	* current = parent; // Init with first element - check this!
	// Deserialize XMLCh * collection
	// Collection is not simpleType list
	while((parent != NULL) && (
		Mp7JrsUtil::HasNodeName(parent, X("Telephone"), X("urn:mpeg:mpeg7:schema:2004"))
// OLD2		(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Telephone")) == 0)
))
	{
		// This is necessary when Xerces uses a different heap
		XMLCh *text = Mp7JrsUtil::GetElementText(parent);
		XMLCh *ours = new XMLCh[XMLString::stringLen(text) + 1];
		XMLString::copyString(ours, text);
		m_Item->addElement(ours);
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// NOTE: Use the following includefile for extension
// #include "Mp7JrsRelationType_target_CollectionType_ExtPostDeserialize.h"

	return true;
}

