 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsEventType_ExtImplInclude.h



#include "Mp7JrsSemanticBaseType.h"
#include "Mp7JrsEventType_CollectionType.h"
#include "Mp7JrsEventType_SemanticPlace_CollectionType.h"
#include "Mp7JrsEventType_SemanticTime_CollectionType.h"
#include "Mp7JrsAbstractionLevelType.h"
#include "Mp7JrsSemanticBaseType_Label_CollectionType.h"
#include "Mp7JrsTextAnnotationType.h"
#include "Mp7JrsSemanticBaseType_Property_CollectionType.h"
#include "Mp7JrsSemanticBaseType_MediaOccurrence_CollectionType.h"
#include "Mp7JrsSemanticBaseType_Relation_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsEventType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTermUseType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Label
#include "Mp7JrsSemanticBaseType_MediaOccurrence_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:MediaOccurrence
#include "Mp7JrsRelationType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relation
#include "Mp7JrsEventType_LocalType.h" // Choice collection Event
#include "Mp7JrsEventType.h" // Choice collection element Event
#include "Mp7JrsReferenceType.h" // Choice collection element EventRef
#include "Mp7JrsSemanticPlaceType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SemanticPlace
#include "Mp7JrsSemanticTimeType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SemanticTime

#include <assert.h>
IMp7JrsEventType::IMp7JrsEventType()
{

// no includefile for extension defined 
// file Mp7JrsEventType_ExtPropInit.h

}

IMp7JrsEventType::~IMp7JrsEventType()
{
// no includefile for extension defined 
// file Mp7JrsEventType_ExtPropCleanup.h

}

Mp7JrsEventType::Mp7JrsEventType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsEventType::~Mp7JrsEventType()
{
	Cleanup();
}

void Mp7JrsEventType::Init()
{
	// Init base
	m_Base = CreateSemanticBaseType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_EventType_LocalType = Mp7JrsEventType_CollectionPtr(); // Collection
	m_SemanticPlace = Mp7JrsEventType_SemanticPlace_CollectionPtr(); // Collection
	m_SemanticTime = Mp7JrsEventType_SemanticTime_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsEventType_ExtMyPropInit.h

}

void Mp7JrsEventType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsEventType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_EventType_LocalType);
	// Mp7JrsFactory::DeleteObject(m_SemanticPlace);
	// Mp7JrsFactory::DeleteObject(m_SemanticTime);
}

void Mp7JrsEventType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use EventTypePtr, since we
	// might need GetBase(), which isn't defined in IEventType
    const Mp7JrsPtr< Mp7JrsEventType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsSemanticBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsEventType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_EventType_LocalType);
		this->SetEventType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetEventType_LocalType()));
		// Mp7JrsFactory::DeleteObject(m_SemanticPlace);
		this->SetSemanticPlace(Mp7JrsFactory::CloneObject(tmp->GetSemanticPlace()));
		// Mp7JrsFactory::DeleteObject(m_SemanticTime);
		this->SetSemanticTime(Mp7JrsFactory::CloneObject(tmp->GetSemanticTime()));
}

Mp7JrsNodePtr Mp7JrsEventType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsEventType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsSemanticBaseType > Mp7JrsEventType::GetBase() const
{
	return m_Base;
}

Mp7JrsEventType_CollectionPtr Mp7JrsEventType::GetEventType_LocalType() const
{
	return m_EventType_LocalType;
}

Mp7JrsEventType_SemanticPlace_CollectionPtr Mp7JrsEventType::GetSemanticPlace() const
{
	return m_SemanticPlace;
}

Mp7JrsEventType_SemanticTime_CollectionPtr Mp7JrsEventType::GetSemanticTime() const
{
	return m_SemanticTime;
}

void Mp7JrsEventType::SetEventType_LocalType(const Mp7JrsEventType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetEventType_LocalType().");
	}
	if (m_EventType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_EventType_LocalType);
		m_EventType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_EventType_LocalType) m_EventType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::SetSemanticPlace(const Mp7JrsEventType_SemanticPlace_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetSemanticPlace().");
	}
	if (m_SemanticPlace != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SemanticPlace);
		m_SemanticPlace = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SemanticPlace) m_SemanticPlace->SetParent(m_myself.getPointer());
		if(m_SemanticPlace != Mp7JrsEventType_SemanticPlace_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SemanticPlace->SetContentName(XMLString::transcode("SemanticPlace"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::SetSemanticTime(const Mp7JrsEventType_SemanticTime_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetSemanticTime().");
	}
	if (m_SemanticTime != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SemanticTime);
		m_SemanticTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SemanticTime) m_SemanticTime->SetParent(m_myself.getPointer());
		if(m_SemanticTime != Mp7JrsEventType_SemanticTime_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SemanticTime->SetContentName(XMLString::transcode("SemanticTime"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsAbstractionLevelPtr Mp7JrsEventType::GetAbstractionLevel() const
{
	return GetBase()->GetAbstractionLevel();
}

// Element is optional
bool Mp7JrsEventType::IsValidAbstractionLevel() const
{
	return GetBase()->IsValidAbstractionLevel();
}

Mp7JrsSemanticBaseType_Label_CollectionPtr Mp7JrsEventType::GetLabel() const
{
	return GetBase()->GetLabel();
}

Mp7JrsTextAnnotationPtr Mp7JrsEventType::GetDefinition() const
{
	return GetBase()->GetDefinition();
}

// Element is optional
bool Mp7JrsEventType::IsValidDefinition() const
{
	return GetBase()->IsValidDefinition();
}

Mp7JrsSemanticBaseType_Property_CollectionPtr Mp7JrsEventType::GetProperty() const
{
	return GetBase()->GetProperty();
}

Mp7JrsSemanticBaseType_MediaOccurrence_CollectionPtr Mp7JrsEventType::GetMediaOccurrence() const
{
	return GetBase()->GetMediaOccurrence();
}

Mp7JrsSemanticBaseType_Relation_CollectionPtr Mp7JrsEventType::GetRelation() const
{
	return GetBase()->GetRelation();
}

void Mp7JrsEventType::SetAbstractionLevel(const Mp7JrsAbstractionLevelPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetAbstractionLevel().");
	}
	GetBase()->SetAbstractionLevel(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::InvalidateAbstractionLevel()
{
	GetBase()->InvalidateAbstractionLevel();
}
void Mp7JrsEventType::SetLabel(const Mp7JrsSemanticBaseType_Label_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetLabel().");
	}
	GetBase()->SetLabel(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::SetDefinition(const Mp7JrsTextAnnotationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetDefinition().");
	}
	GetBase()->SetDefinition(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::InvalidateDefinition()
{
	GetBase()->InvalidateDefinition();
}
void Mp7JrsEventType::SetProperty(const Mp7JrsSemanticBaseType_Property_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetProperty().");
	}
	GetBase()->SetProperty(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::SetMediaOccurrence(const Mp7JrsSemanticBaseType_MediaOccurrence_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetMediaOccurrence().");
	}
	GetBase()->SetMediaOccurrence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::SetRelation(const Mp7JrsSemanticBaseType_Relation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetRelation().");
	}
	GetBase()->SetRelation(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsEventType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsEventType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsEventType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsEventType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsEventType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsEventType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsEventType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsEventType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsEventType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsEventType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsEventType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsEventType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsEventType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsEventType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsEventType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsEventType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsEventType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsEventType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsEventType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Event")) == 0)
	{
		// Event is contained in itemtype EventType_LocalType
		// in choice collection EventType_CollectionType
		
		context->Found = true;
		Mp7JrsEventType_CollectionPtr coll = GetEventType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateEventType_CollectionType;
				SetEventType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsEventType_LocalPtr)coll->elementAt(i))->GetEvent()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsEventType_LocalPtr item = CreateEventType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:EventType")))) != empty)
			{
				// Is type allowed
				Mp7JrsEventPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsEventType_LocalPtr)coll->elementAt(i))->SetEvent(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsEventType_LocalPtr)coll->elementAt(i))->GetEvent()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("EventRef")) == 0)
	{
		// EventRef is contained in itemtype EventType_LocalType
		// in choice collection EventType_CollectionType
		
		context->Found = true;
		Mp7JrsEventType_CollectionPtr coll = GetEventType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateEventType_CollectionType;
				SetEventType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsEventType_LocalPtr)coll->elementAt(i))->GetEventRef()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsEventType_LocalPtr item = CreateEventType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsEventType_LocalPtr)coll->elementAt(i))->SetEventRef(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsEventType_LocalPtr)coll->elementAt(i))->GetEventRef()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SemanticPlace")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:SemanticPlace is item of type SemanticPlaceType
		// in element collection EventType_SemanticPlace_CollectionType
		
		context->Found = true;
		Mp7JrsEventType_SemanticPlace_CollectionPtr coll = GetSemanticPlace();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateEventType_SemanticPlace_CollectionType;
				SetSemanticPlace(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticPlaceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticPlacePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SemanticTime")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:SemanticTime is item of type SemanticTimeType
		// in element collection EventType_SemanticTime_CollectionType
		
		context->Found = true;
		Mp7JrsEventType_SemanticTime_CollectionPtr coll = GetSemanticTime();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateEventType_SemanticTime_CollectionType;
				SetSemanticTime(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for EventType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "EventType");
	}		
	
	return result;
}

XMLCh * Mp7JrsEventType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsEventType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsEventType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsEventType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("EventType"));
	// Element serialization:
	if (m_EventType_LocalType != Mp7JrsEventType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_EventType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_SemanticPlace != Mp7JrsEventType_SemanticPlace_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SemanticPlace->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_SemanticTime != Mp7JrsEventType_SemanticTime_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SemanticTime->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsEventType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsSemanticBaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:Event
            Mp7JrsUtil::HasNodeName(parent, X("Event"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Event")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:EventRef
            Mp7JrsUtil::HasNodeName(parent, X("EventRef"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:EventRef")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsEventType_CollectionPtr tmp = CreateEventType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetEventType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("SemanticPlace"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("SemanticPlace")) == 0))
        {
            // Deserialize factory type
            Mp7JrsEventType_SemanticPlace_CollectionPtr tmp = CreateEventType_SemanticPlace_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSemanticPlace(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("SemanticTime"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("SemanticTime")) == 0))
        {
            // Deserialize factory type
            Mp7JrsEventType_SemanticTime_CollectionPtr tmp = CreateEventType_SemanticTime_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSemanticTime(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsEventType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsEventType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Event"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Event")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("EventRef"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("EventRef")) == 0)
    )
  {
    Mp7JrsEventType_CollectionPtr tmp = CreateEventType_CollectionType;
    this->SetEventType_LocalType(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("SemanticPlace")) == 0))
  {
    Mp7JrsEventType_SemanticPlace_CollectionPtr tmp = CreateEventType_SemanticPlace_CollectionType;
    this->SetSemanticPlace(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("SemanticTime")) == 0))
  {
    Mp7JrsEventType_SemanticTime_CollectionPtr tmp = CreateEventType_SemanticTime_CollectionType;
    this->SetSemanticTime(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsEventType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetEventType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetEventType_LocalType());
	if (GetSemanticPlace() != Mp7JrsNodePtr())
		result.Insert(GetSemanticPlace());
	if (GetSemanticTime() != Mp7JrsNodePtr())
		result.Insert(GetSemanticTime());
	if (GetAbstractionLevel() != Mp7JrsNodePtr())
		result.Insert(GetAbstractionLevel());
	if (GetLabel() != Mp7JrsNodePtr())
		result.Insert(GetLabel());
	if (GetDefinition() != Mp7JrsNodePtr())
		result.Insert(GetDefinition());
	if (GetProperty() != Mp7JrsNodePtr())
		result.Insert(GetProperty());
	if (GetMediaOccurrence() != Mp7JrsNodePtr())
		result.Insert(GetMediaOccurrence());
	if (GetRelation() != Mp7JrsNodePtr())
		result.Insert(GetRelation());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsEventType_ExtMethodImpl.h


