 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtImplInclude.h



#include "Mp7JrspaddingType.h"
#include "Mp7JrsFilterType.h"
#include "Mp7JrsSignalPlaneSampleType.h"
#include "Mp7JrsFilteringType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsFilteringType::IMp7JrsFilteringType()
{

// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtPropInit.h

}

IMp7JrsFilteringType::~IMp7JrsFilteringType()
{
// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtPropCleanup.h

}

Mp7JrsFilteringType::Mp7JrsFilteringType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFilteringType::~Mp7JrsFilteringType()
{
	Cleanup();
}

void Mp7JrsFilteringType::Init()
{

	// Init attributes
	m_xPad = Mp7JrspaddingPtr(); // Create emtpy class ptr
	m_xPad_Exist = false;
	m_yPad = Mp7JrspaddingPtr(); // Create emtpy class ptr
	m_yPad_Exist = false;
	m_zPad = Mp7JrspaddingPtr(); // Create emtpy class ptr
	m_zPad_Exist = false;
	m_tPad = Mp7JrspaddingPtr(); // Create emtpy class ptr
	m_tPad_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Filter = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_PadSize = Mp7JrsSignalPlaneSamplePtr(); // Class
	m_PadSize_Exist = false;
	m_Shift = Mp7JrsSignalPlaneSamplePtr(); // Class
	m_Shift_Exist = false;
	m_CropStart = Mp7JrsSignalPlaneSamplePtr(); // Class
	m_CropStart_Exist = false;
	m_CropEnd = Mp7JrsSignalPlaneSamplePtr(); // Class
	m_CropEnd_Exist = false;


// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtMyPropInit.h

}

void Mp7JrsFilteringType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_xPad);
	// Mp7JrsFactory::DeleteObject(m_yPad);
	// Mp7JrsFactory::DeleteObject(m_zPad);
	// Mp7JrsFactory::DeleteObject(m_tPad);
	// Mp7JrsFactory::DeleteObject(m_Filter);
	// Mp7JrsFactory::DeleteObject(m_PadSize);
	// Mp7JrsFactory::DeleteObject(m_Shift);
	// Mp7JrsFactory::DeleteObject(m_CropStart);
	// Mp7JrsFactory::DeleteObject(m_CropEnd);
}

void Mp7JrsFilteringType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FilteringTypePtr, since we
	// might need GetBase(), which isn't defined in IFilteringType
    const Mp7JrsPtr< Mp7JrsFilteringType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_xPad);
	if (tmp->ExistxPad())
	{
		this->SetxPad(Mp7JrsFactory::CloneObject(tmp->GetxPad()));
	}
	else
	{
		InvalidatexPad();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_yPad);
	if (tmp->ExistyPad())
	{
		this->SetyPad(Mp7JrsFactory::CloneObject(tmp->GetyPad()));
	}
	else
	{
		InvalidateyPad();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_zPad);
	if (tmp->ExistzPad())
	{
		this->SetzPad(Mp7JrsFactory::CloneObject(tmp->GetzPad()));
	}
	else
	{
		InvalidatezPad();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_tPad);
	if (tmp->ExisttPad())
	{
		this->SettPad(Mp7JrsFactory::CloneObject(tmp->GettPad()));
	}
	else
	{
		InvalidatetPad();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Filter);
		this->SetFilter(Mp7JrsFactory::CloneObject(tmp->GetFilter()));
	if (tmp->IsValidPadSize())
	{
		// Mp7JrsFactory::DeleteObject(m_PadSize);
		this->SetPadSize(Mp7JrsFactory::CloneObject(tmp->GetPadSize()));
	}
	else
	{
		InvalidatePadSize();
	}
	if (tmp->IsValidShift())
	{
		// Mp7JrsFactory::DeleteObject(m_Shift);
		this->SetShift(Mp7JrsFactory::CloneObject(tmp->GetShift()));
	}
	else
	{
		InvalidateShift();
	}
	if (tmp->IsValidCropStart())
	{
		// Mp7JrsFactory::DeleteObject(m_CropStart);
		this->SetCropStart(Mp7JrsFactory::CloneObject(tmp->GetCropStart()));
	}
	else
	{
		InvalidateCropStart();
	}
	if (tmp->IsValidCropEnd())
	{
		// Mp7JrsFactory::DeleteObject(m_CropEnd);
		this->SetCropEnd(Mp7JrsFactory::CloneObject(tmp->GetCropEnd()));
	}
	else
	{
		InvalidateCropEnd();
	}
}

Mp7JrsNodePtr Mp7JrsFilteringType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsFilteringType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrspaddingPtr Mp7JrsFilteringType::GetxPad() const
{
	if (this->ExistxPad()) {
		return m_xPad;
	} else {
		return m_xPad_Default;
	}
}

bool Mp7JrsFilteringType::ExistxPad() const
{
	return m_xPad_Exist;
}
void Mp7JrsFilteringType::SetxPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetxPad().");
	}
    m_xPad = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_xPad) m_xPad->SetParent(m_myself.getPointer());
	m_xPad_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidatexPad()
{
	m_xPad_Exist = false;
}
Mp7JrspaddingPtr Mp7JrsFilteringType::GetyPad() const
{
	if (this->ExistyPad()) {
		return m_yPad;
	} else {
		return m_yPad_Default;
	}
}

bool Mp7JrsFilteringType::ExistyPad() const
{
	return m_yPad_Exist;
}
void Mp7JrsFilteringType::SetyPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetyPad().");
	}
    m_yPad = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_yPad) m_yPad->SetParent(m_myself.getPointer());
	m_yPad_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidateyPad()
{
	m_yPad_Exist = false;
}
Mp7JrspaddingPtr Mp7JrsFilteringType::GetzPad() const
{
	if (this->ExistzPad()) {
		return m_zPad;
	} else {
		return m_zPad_Default;
	}
}

bool Mp7JrsFilteringType::ExistzPad() const
{
	return m_zPad_Exist;
}
void Mp7JrsFilteringType::SetzPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetzPad().");
	}
    m_zPad = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_zPad) m_zPad->SetParent(m_myself.getPointer());
	m_zPad_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidatezPad()
{
	m_zPad_Exist = false;
}
Mp7JrspaddingPtr Mp7JrsFilteringType::GettPad() const
{
	if (this->ExisttPad()) {
		return m_tPad;
	} else {
		return m_tPad_Default;
	}
}

bool Mp7JrsFilteringType::ExisttPad() const
{
	return m_tPad_Exist;
}
void Mp7JrsFilteringType::SettPad(const Mp7JrspaddingPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SettPad().");
	}
    m_tPad = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_tPad) m_tPad->SetParent(m_myself.getPointer());
	m_tPad_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidatetPad()
{
	m_tPad_Exist = false;
}
Mp7JrsPtr< Mp7JrsNode > Mp7JrsFilteringType::GetFilter() const
{
	return m_Filter;
}

Mp7JrsSignalPlaneSamplePtr Mp7JrsFilteringType::GetPadSize() const
{
	return m_PadSize;
}

// Element is optional
bool Mp7JrsFilteringType::IsValidPadSize() const
{
	return m_PadSize_Exist;
}

Mp7JrsSignalPlaneSamplePtr Mp7JrsFilteringType::GetShift() const
{
	return m_Shift;
}

// Element is optional
bool Mp7JrsFilteringType::IsValidShift() const
{
	return m_Shift_Exist;
}

Mp7JrsSignalPlaneSamplePtr Mp7JrsFilteringType::GetCropStart() const
{
	return m_CropStart;
}

// Element is optional
bool Mp7JrsFilteringType::IsValidCropStart() const
{
	return m_CropStart_Exist;
}

Mp7JrsSignalPlaneSamplePtr Mp7JrsFilteringType::GetCropEnd() const
{
	return m_CropEnd;
}

// Element is optional
bool Mp7JrsFilteringType::IsValidCropEnd() const
{
	return m_CropEnd_Exist;
}

void Mp7JrsFilteringType::SetFilter(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetFilter().");
	}
	if (m_Filter != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Filter);
		m_Filter = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Filter) m_Filter->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Filter) m_Filter->UseTypeAttribute = true;
		if(m_Filter != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Filter->SetContentName(XMLString::transcode("Filter"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::SetPadSize(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetPadSize().");
	}
	if (m_PadSize != item || m_PadSize_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_PadSize);
		m_PadSize = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PadSize) m_PadSize->SetParent(m_myself.getPointer());
		if (m_PadSize && m_PadSize->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_PadSize->UseTypeAttribute = true;
		}
		m_PadSize_Exist = true;
		if(m_PadSize != Mp7JrsSignalPlaneSamplePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PadSize->SetContentName(XMLString::transcode("PadSize"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidatePadSize()
{
	m_PadSize_Exist = false;
}
void Mp7JrsFilteringType::SetShift(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetShift().");
	}
	if (m_Shift != item || m_Shift_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Shift);
		m_Shift = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Shift) m_Shift->SetParent(m_myself.getPointer());
		if (m_Shift && m_Shift->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Shift->UseTypeAttribute = true;
		}
		m_Shift_Exist = true;
		if(m_Shift != Mp7JrsSignalPlaneSamplePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Shift->SetContentName(XMLString::transcode("Shift"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidateShift()
{
	m_Shift_Exist = false;
}
void Mp7JrsFilteringType::SetCropStart(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetCropStart().");
	}
	if (m_CropStart != item || m_CropStart_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_CropStart);
		m_CropStart = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CropStart) m_CropStart->SetParent(m_myself.getPointer());
		if (m_CropStart && m_CropStart->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_CropStart->UseTypeAttribute = true;
		}
		m_CropStart_Exist = true;
		if(m_CropStart != Mp7JrsSignalPlaneSamplePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CropStart->SetContentName(XMLString::transcode("CropStart"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidateCropStart()
{
	m_CropStart_Exist = false;
}
void Mp7JrsFilteringType::SetCropEnd(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFilteringType::SetCropEnd().");
	}
	if (m_CropEnd != item || m_CropEnd_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_CropEnd);
		m_CropEnd = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CropEnd) m_CropEnd->SetParent(m_myself.getPointer());
		if (m_CropEnd && m_CropEnd->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_CropEnd->UseTypeAttribute = true;
		}
		m_CropEnd_Exist = true;
		if(m_CropEnd != Mp7JrsSignalPlaneSamplePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CropEnd->SetContentName(XMLString::transcode("CropEnd"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFilteringType::InvalidateCropEnd()
{
	m_CropEnd_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsFilteringType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Filter")) == 0)
	{
		// Filter is simple abstract element FilterType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFilter()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsFilterPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFilter(p, client);
					if((p = GetFilter()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PadSize")) == 0)
	{
		// PadSize is simple element SignalPlaneSampleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPadSize()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneSamplePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPadSize(p, client);
					if((p = GetPadSize()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Shift")) == 0)
	{
		// Shift is simple element SignalPlaneSampleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetShift()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneSamplePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetShift(p, client);
					if((p = GetShift()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CropStart")) == 0)
	{
		// CropStart is simple element SignalPlaneSampleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCropStart()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneSamplePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCropStart(p, client);
					if((p = GetCropStart()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CropEnd")) == 0)
	{
		// CropEnd is simple element SignalPlaneSampleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCropEnd()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneSamplePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCropEnd(p, client);
					if((p = GetCropEnd()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for FilteringType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "FilteringType");
	}		
	
	return result;
}

XMLCh * Mp7JrsFilteringType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFilteringType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsFilteringType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FilteringType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_xPad_Exist)
	{
	// Class
	if (m_xPad != Mp7JrspaddingPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_xPad->ToText();
		element->setAttributeNS(X(""), X("xPad"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_yPad_Exist)
	{
	// Class
	if (m_yPad != Mp7JrspaddingPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_yPad->ToText();
		element->setAttributeNS(X(""), X("yPad"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_zPad_Exist)
	{
	// Class
	if (m_zPad != Mp7JrspaddingPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_zPad->ToText();
		element->setAttributeNS(X(""), X("zPad"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_tPad_Exist)
	{
	// Class
	if (m_tPad != Mp7JrspaddingPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_tPad->ToText();
		element->setAttributeNS(X(""), X("tPad"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Filter != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Filter->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Filter"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Filter->UseTypeAttribute = true;
		}
		m_Filter->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_PadSize != Mp7JrsSignalPlaneSamplePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_PadSize->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("PadSize"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_PadSize->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Shift != Mp7JrsSignalPlaneSamplePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Shift->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Shift"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Shift->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_CropStart != Mp7JrsSignalPlaneSamplePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_CropStart->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("CropStart"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_CropStart->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_CropEnd != Mp7JrsSignalPlaneSamplePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_CropEnd->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("CropEnd"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_CropEnd->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsFilteringType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("xPad")))
	{
        // Deserialize class type
		Mp7JrspaddingPtr tmp = CreatepaddingType;
		tmp->Parse(parent->getAttributeNS(X(""), X("xPad")));
        this->SetxPad(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("xPad")))
	{
		// Deserialize class type
        Mp7JrspaddingPtr tmp = CreatepaddingType;
        tmp->Parse(parent->getAttribute(X("xPad")));
        this->SetxPad(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("yPad")))
	{
        // Deserialize class type
		Mp7JrspaddingPtr tmp = CreatepaddingType;
		tmp->Parse(parent->getAttributeNS(X(""), X("yPad")));
        this->SetyPad(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("yPad")))
	{
		// Deserialize class type
        Mp7JrspaddingPtr tmp = CreatepaddingType;
        tmp->Parse(parent->getAttribute(X("yPad")));
        this->SetyPad(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("zPad")))
	{
        // Deserialize class type
		Mp7JrspaddingPtr tmp = CreatepaddingType;
		tmp->Parse(parent->getAttributeNS(X(""), X("zPad")));
        this->SetzPad(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("zPad")))
	{
		// Deserialize class type
        Mp7JrspaddingPtr tmp = CreatepaddingType;
        tmp->Parse(parent->getAttribute(X("zPad")));
        this->SetzPad(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("tPad")))
	{
        // Deserialize class type
		Mp7JrspaddingPtr tmp = CreatepaddingType;
		tmp->Parse(parent->getAttributeNS(X(""), X("tPad")));
        this->SettPad(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("tPad")))
	{
		// Deserialize class type
        Mp7JrspaddingPtr tmp = CreatepaddingType;
        tmp->Parse(parent->getAttribute(X("tPad")));
        this->SettPad(tmp);
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Filter"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Filter")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateFilterType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFilter(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("PadSize"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("PadSize")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneSampleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPadSize(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Shift"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Shift")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneSampleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetShift(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("CropStart"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("CropStart")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneSampleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetCropStart(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("CropEnd"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("CropEnd")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneSampleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetCropEnd(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFilteringType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Filter")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateFilterType;
    }
    this->SetFilter(child);
  }
  if (XMLString::compareString(elementname, X("PadSize")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneSampleType;
    }
    this->SetPadSize(child);
  }
  if (XMLString::compareString(elementname, X("Shift")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneSampleType;
    }
    this->SetShift(child);
  }
  if (XMLString::compareString(elementname, X("CropStart")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneSampleType;
    }
    this->SetCropStart(child);
  }
  if (XMLString::compareString(elementname, X("CropEnd")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneSampleType;
    }
    this->SetCropEnd(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFilteringType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetFilter() != Mp7JrsNodePtr())
		result.Insert(GetFilter());
	if (GetPadSize() != Mp7JrsNodePtr())
		result.Insert(GetPadSize());
	if (GetShift() != Mp7JrsNodePtr())
		result.Insert(GetShift());
	if (GetCropStart() != Mp7JrsNodePtr())
		result.Insert(GetCropStart());
	if (GetCropEnd() != Mp7JrsNodePtr())
		result.Insert(GetCropEnd());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFilteringType_ExtMethodImpl.h


