 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtImplInclude.h



#include "Mp7JrsFinancialType_AccountItem_CollectionType.h"
#include "Mp7JrsFinancialType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsFinancialType_AccountItem_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:AccountItem

#include <assert.h>
IMp7JrsFinancialType::IMp7JrsFinancialType()
{

// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtPropInit.h

}

IMp7JrsFinancialType::~IMp7JrsFinancialType()
{
// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtPropCleanup.h

}

Mp7JrsFinancialType::Mp7JrsFinancialType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFinancialType::~Mp7JrsFinancialType()
{
	Cleanup();
}

void Mp7JrsFinancialType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_AccountItem = Mp7JrsFinancialType_AccountItem_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtMyPropInit.h

}

void Mp7JrsFinancialType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_AccountItem);
}

void Mp7JrsFinancialType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FinancialTypePtr, since we
	// might need GetBase(), which isn't defined in IFinancialType
    const Mp7JrsPtr< Mp7JrsFinancialType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_AccountItem);
		this->SetAccountItem(Mp7JrsFactory::CloneObject(tmp->GetAccountItem()));
}

Mp7JrsNodePtr Mp7JrsFinancialType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsFinancialType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsFinancialType_AccountItem_CollectionPtr Mp7JrsFinancialType::GetAccountItem() const
{
	return m_AccountItem;
}

void Mp7JrsFinancialType::SetAccountItem(const Mp7JrsFinancialType_AccountItem_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFinancialType::SetAccountItem().");
	}
	if (m_AccountItem != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AccountItem);
		m_AccountItem = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AccountItem) m_AccountItem->SetParent(m_myself.getPointer());
		if(m_AccountItem != Mp7JrsFinancialType_AccountItem_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AccountItem->SetContentName(XMLString::transcode("AccountItem"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsFinancialType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("AccountItem")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:AccountItem is item of type FinancialType_AccountItem_LocalType
		// in element collection FinancialType_AccountItem_CollectionType
		
		context->Found = true;
		Mp7JrsFinancialType_AccountItem_CollectionPtr coll = GetAccountItem();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateFinancialType_AccountItem_CollectionType;
				SetAccountItem(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FinancialType_AccountItem_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFinancialType_AccountItem_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for FinancialType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "FinancialType");
	}		
	
	return result;
}

XMLCh * Mp7JrsFinancialType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFinancialType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsFinancialType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FinancialType"));
	// Element serialization:
	if (m_AccountItem != Mp7JrsFinancialType_AccountItem_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_AccountItem->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsFinancialType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("AccountItem"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("AccountItem")) == 0))
        {
            // Deserialize factory type
            Mp7JrsFinancialType_AccountItem_CollectionPtr tmp = CreateFinancialType_AccountItem_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetAccountItem(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFinancialType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("AccountItem")) == 0))
  {
    Mp7JrsFinancialType_AccountItem_CollectionPtr tmp = CreateFinancialType_AccountItem_CollectionType;
    this->SetAccountItem(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFinancialType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetAccountItem() != Mp7JrsNodePtr())
		result.Insert(GetAccountItem());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFinancialType_ExtMethodImpl.h


