 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtImplInclude.h



#include "Mp7JrsViewType.h"
#include "Mp7JrsPartitionType.h"
#include "Mp7JrsFilteringType.h"
#include "Mp7JrsSignalPlaneSampleType.h"
#include "Mp7JrsSignalType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsFrequencyViewType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsFrequencyViewType::IMp7JrsFrequencyViewType()
{

// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtPropInit.h

}

IMp7JrsFrequencyViewType::~IMp7JrsFrequencyViewType()
{
// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtPropCleanup.h

}

Mp7JrsFrequencyViewType::Mp7JrsFrequencyViewType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFrequencyViewType::~Mp7JrsFrequencyViewType()
{
	Cleanup();
}

void Mp7JrsFrequencyViewType::Init()
{
	// Init base
	m_Base = CreateViewType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_FrequencyPartition = Mp7JrsPartitionPtr(); // Class
	m_Filtering = Mp7JrsFilteringPtr(); // Class
	m_Filtering_Exist = false;
	m_DownSamplingFactor = Mp7JrsSignalPlaneSamplePtr(); // Class
	m_DownSamplingFactor_Exist = false;


// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtMyPropInit.h

}

void Mp7JrsFrequencyViewType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_FrequencyPartition);
	// Mp7JrsFactory::DeleteObject(m_Filtering);
	// Mp7JrsFactory::DeleteObject(m_DownSamplingFactor);
}

void Mp7JrsFrequencyViewType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FrequencyViewTypePtr, since we
	// might need GetBase(), which isn't defined in IFrequencyViewType
    const Mp7JrsPtr< Mp7JrsFrequencyViewType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsViewPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsFrequencyViewType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_FrequencyPartition);
		this->SetFrequencyPartition(Mp7JrsFactory::CloneObject(tmp->GetFrequencyPartition()));
	if (tmp->IsValidFiltering())
	{
		// Mp7JrsFactory::DeleteObject(m_Filtering);
		this->SetFiltering(Mp7JrsFactory::CloneObject(tmp->GetFiltering()));
	}
	else
	{
		InvalidateFiltering();
	}
	if (tmp->IsValidDownSamplingFactor())
	{
		// Mp7JrsFactory::DeleteObject(m_DownSamplingFactor);
		this->SetDownSamplingFactor(Mp7JrsFactory::CloneObject(tmp->GetDownSamplingFactor()));
	}
	else
	{
		InvalidateDownSamplingFactor();
	}
}

Mp7JrsNodePtr Mp7JrsFrequencyViewType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsFrequencyViewType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsViewType > Mp7JrsFrequencyViewType::GetBase() const
{
	return m_Base;
}

Mp7JrsPartitionPtr Mp7JrsFrequencyViewType::GetFrequencyPartition() const
{
	return m_FrequencyPartition;
}

Mp7JrsFilteringPtr Mp7JrsFrequencyViewType::GetFiltering() const
{
	return m_Filtering;
}

// Element is optional
bool Mp7JrsFrequencyViewType::IsValidFiltering() const
{
	return m_Filtering_Exist;
}

Mp7JrsSignalPlaneSamplePtr Mp7JrsFrequencyViewType::GetDownSamplingFactor() const
{
	return m_DownSamplingFactor;
}

// Element is optional
bool Mp7JrsFrequencyViewType::IsValidDownSamplingFactor() const
{
	return m_DownSamplingFactor_Exist;
}

void Mp7JrsFrequencyViewType::SetFrequencyPartition(const Mp7JrsPartitionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetFrequencyPartition().");
	}
	if (m_FrequencyPartition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FrequencyPartition);
		m_FrequencyPartition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FrequencyPartition) m_FrequencyPartition->SetParent(m_myself.getPointer());
		if (m_FrequencyPartition && m_FrequencyPartition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PartitionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FrequencyPartition->UseTypeAttribute = true;
		}
		if(m_FrequencyPartition != Mp7JrsPartitionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FrequencyPartition->SetContentName(XMLString::transcode("FrequencyPartition"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::SetFiltering(const Mp7JrsFilteringPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetFiltering().");
	}
	if (m_Filtering != item || m_Filtering_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Filtering);
		m_Filtering = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Filtering) m_Filtering->SetParent(m_myself.getPointer());
		if (m_Filtering && m_Filtering->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FilteringType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Filtering->UseTypeAttribute = true;
		}
		m_Filtering_Exist = true;
		if(m_Filtering != Mp7JrsFilteringPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Filtering->SetContentName(XMLString::transcode("Filtering"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidateFiltering()
{
	m_Filtering_Exist = false;
}
void Mp7JrsFrequencyViewType::SetDownSamplingFactor(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetDownSamplingFactor().");
	}
	if (m_DownSamplingFactor != item || m_DownSamplingFactor_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DownSamplingFactor);
		m_DownSamplingFactor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DownSamplingFactor) m_DownSamplingFactor->SetParent(m_myself.getPointer());
		if (m_DownSamplingFactor && m_DownSamplingFactor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DownSamplingFactor->UseTypeAttribute = true;
		}
		m_DownSamplingFactor_Exist = true;
		if(m_DownSamplingFactor != Mp7JrsSignalPlaneSamplePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DownSamplingFactor->SetContentName(XMLString::transcode("DownSamplingFactor"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidateDownSamplingFactor()
{
	m_DownSamplingFactor_Exist = false;
}
Mp7JrsSignalPtr Mp7JrsFrequencyViewType::GetTarget() const
{
	return GetBase()->GetTarget();
}

Mp7JrsSignalPtr Mp7JrsFrequencyViewType::GetSource() const
{
	return GetBase()->GetSource();
}

// Element is optional
bool Mp7JrsFrequencyViewType::IsValidSource() const
{
	return GetBase()->IsValidSource();
}

void Mp7JrsFrequencyViewType::SetTarget(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetTarget().");
	}
	GetBase()->SetTarget(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::SetSource(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetSource().");
	}
	GetBase()->SetSource(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidateSource()
{
	GetBase()->InvalidateSource();
}
XMLCh * Mp7JrsFrequencyViewType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsFrequencyViewType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsFrequencyViewType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsFrequencyViewType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsFrequencyViewType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsFrequencyViewType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsFrequencyViewType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsFrequencyViewType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsFrequencyViewType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsFrequencyViewType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsFrequencyViewType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsFrequencyViewType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsFrequencyViewType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsFrequencyViewType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsFrequencyViewType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyViewType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsFrequencyViewType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsFrequencyViewType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyViewType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsFrequencyViewType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("FrequencyPartition")) == 0)
	{
		// FrequencyPartition is simple element PartitionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFrequencyPartition()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PartitionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPartitionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFrequencyPartition(p, client);
					if((p = GetFrequencyPartition()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Filtering")) == 0)
	{
		// Filtering is simple element FilteringType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFiltering()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FilteringType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFilteringPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFiltering(p, client);
					if((p = GetFiltering()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DownSamplingFactor")) == 0)
	{
		// DownSamplingFactor is simple element SignalPlaneSampleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDownSamplingFactor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneSamplePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDownSamplingFactor(p, client);
					if((p = GetDownSamplingFactor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for FrequencyViewType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "FrequencyViewType");
	}		
	
	return result;
}

XMLCh * Mp7JrsFrequencyViewType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsFrequencyViewType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsFrequencyViewType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FrequencyViewType"));
	// Element serialization:
	// Class
	
	if (m_FrequencyPartition != Mp7JrsPartitionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FrequencyPartition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FrequencyPartition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FrequencyPartition->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Filtering != Mp7JrsFilteringPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Filtering->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Filtering"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Filtering->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_DownSamplingFactor != Mp7JrsSignalPlaneSamplePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DownSamplingFactor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DownSamplingFactor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DownSamplingFactor->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsFrequencyViewType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsViewType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FrequencyPartition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FrequencyPartition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePartitionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFrequencyPartition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Filtering"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Filtering")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateFilteringType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFiltering(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DownSamplingFactor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DownSamplingFactor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneSampleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDownSamplingFactor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFrequencyViewType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("FrequencyPartition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePartitionType;
    }
    this->SetFrequencyPartition(child);
  }
  if (XMLString::compareString(elementname, X("Filtering")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateFilteringType;
    }
    this->SetFiltering(child);
  }
  if (XMLString::compareString(elementname, X("DownSamplingFactor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneSampleType;
    }
    this->SetDownSamplingFactor(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFrequencyViewType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetFrequencyPartition() != Mp7JrsNodePtr())
		result.Insert(GetFrequencyPartition());
	if (GetFiltering() != Mp7JrsNodePtr())
		result.Insert(GetFiltering());
	if (GetDownSamplingFactor() != Mp7JrsNodePtr())
		result.Insert(GetDownSamplingFactor());
	if (GetTarget() != Mp7JrsNodePtr())
		result.Insert(GetTarget());
	if (GetSource() != Mp7JrsNodePtr())
		result.Insert(GetSource());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFrequencyViewType_ExtMethodImpl.h


