 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtImplInclude.h



#include "Mp7JrsRelationType.h"
#include "Mp7JrstermReferenceListType.h"
#include "Mp7JrstermReferenceType.h"
#include "Mp7JrsRelationType_source_CollectionType.h"
#include "Mp7JrsRelationType_target_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsGeneralRelationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsGeneralRelationType::IMp7JrsGeneralRelationType()
{

// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtPropInit.h

}

IMp7JrsGeneralRelationType::~IMp7JrsGeneralRelationType()
{
// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtPropCleanup.h

}

Mp7JrsGeneralRelationType::Mp7JrsGeneralRelationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGeneralRelationType::~Mp7JrsGeneralRelationType()
{
	Cleanup();
}

void Mp7JrsGeneralRelationType::Init()
{
	// Init base
	m_Base = CreateRelationType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_typelist = Mp7JrstermReferenceListPtr(); // Collection
	m_typelist_Exist = false;
	m_generalSource = Mp7JrstermReferenceListPtr(); // Collection
	m_generalSource_Exist = false;
	m_generalTarget = Mp7JrstermReferenceListPtr(); // Collection
	m_generalTarget_Exist = false;



// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtMyPropInit.h

}

void Mp7JrsGeneralRelationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_typelist);
	// Mp7JrsFactory::DeleteObject(m_generalSource);
	// Mp7JrsFactory::DeleteObject(m_generalTarget);
}

void Mp7JrsGeneralRelationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GeneralRelationTypePtr, since we
	// might need GetBase(), which isn't defined in IGeneralRelationType
    const Mp7JrsPtr< Mp7JrsGeneralRelationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsRelationPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsGeneralRelationType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_typelist);
	if (tmp->Existtypelist())
	{
		this->Settypelist(Mp7JrsFactory::CloneObject(tmp->Gettypelist()));
	}
	else
	{
		Invalidatetypelist();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_generalSource);
	if (tmp->ExistgeneralSource())
	{
		this->SetgeneralSource(Mp7JrsFactory::CloneObject(tmp->GetgeneralSource()));
	}
	else
	{
		InvalidategeneralSource();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_generalTarget);
	if (tmp->ExistgeneralTarget())
	{
		this->SetgeneralTarget(Mp7JrsFactory::CloneObject(tmp->GetgeneralTarget()));
	}
	else
	{
		InvalidategeneralTarget();
	}
	}
}

Mp7JrsNodePtr Mp7JrsGeneralRelationType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsGeneralRelationType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsRelationType > Mp7JrsGeneralRelationType::GetBase() const
{
	return m_Base;
}

Mp7JrstermReferenceListPtr Mp7JrsGeneralRelationType::Gettypelist() const
{
	return m_typelist;
}

bool Mp7JrsGeneralRelationType::Existtypelist() const
{
	return m_typelist_Exist;
}
void Mp7JrsGeneralRelationType::Settypelist(const Mp7JrstermReferenceListPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Settypelist().");
	}
    m_typelist = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_typelist) m_typelist->SetParent(m_myself.getPointer());
	m_typelist_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidatetypelist()
{
	m_typelist_Exist = false;
}
Mp7JrstermReferenceListPtr Mp7JrsGeneralRelationType::GetgeneralSource() const
{
	return m_generalSource;
}

bool Mp7JrsGeneralRelationType::ExistgeneralSource() const
{
	return m_generalSource_Exist;
}
void Mp7JrsGeneralRelationType::SetgeneralSource(const Mp7JrstermReferenceListPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SetgeneralSource().");
	}
    m_generalSource = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_generalSource) m_generalSource->SetParent(m_myself.getPointer());
	m_generalSource_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::InvalidategeneralSource()
{
	m_generalSource_Exist = false;
}
Mp7JrstermReferenceListPtr Mp7JrsGeneralRelationType::GetgeneralTarget() const
{
	return m_generalTarget;
}

bool Mp7JrsGeneralRelationType::ExistgeneralTarget() const
{
	return m_generalTarget_Exist;
}
void Mp7JrsGeneralRelationType::SetgeneralTarget(const Mp7JrstermReferenceListPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SetgeneralTarget().");
	}
    m_generalTarget = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_generalTarget) m_generalTarget->SetParent(m_myself.getPointer());
	m_generalTarget_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::InvalidategeneralTarget()
{
	m_generalTarget_Exist = false;
}
Mp7JrstermReferencePtr Mp7JrsGeneralRelationType::Gettype() const
{
	return GetBase()->Gettype();
}

bool Mp7JrsGeneralRelationType::Existtype() const
{
	return GetBase()->Existtype();
}
void Mp7JrsGeneralRelationType::Settype(const Mp7JrstermReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Settype().");
	}
	GetBase()->Settype(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidatetype()
{
	GetBase()->Invalidatetype();
}
Mp7JrsRelationType_source_CollectionPtr Mp7JrsGeneralRelationType::Getsource() const
{
	return GetBase()->Getsource();
}

bool Mp7JrsGeneralRelationType::Existsource() const
{
	return GetBase()->Existsource();
}
void Mp7JrsGeneralRelationType::Setsource(const Mp7JrsRelationType_source_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Setsource().");
	}
	GetBase()->Setsource(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidatesource()
{
	GetBase()->Invalidatesource();
}
Mp7JrsRelationType_target_CollectionPtr Mp7JrsGeneralRelationType::Gettarget() const
{
	return GetBase()->Gettarget();
}

bool Mp7JrsGeneralRelationType::Existtarget() const
{
	return GetBase()->Existtarget();
}
void Mp7JrsGeneralRelationType::Settarget(const Mp7JrsRelationType_target_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Settarget().");
	}
	GetBase()->Settarget(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidatetarget()
{
	GetBase()->Invalidatetarget();
}
bool Mp7JrsGeneralRelationType::Getdirected() const
{
	return GetBase()->Getdirected();
}

bool Mp7JrsGeneralRelationType::Existdirected() const
{
	return GetBase()->Existdirected();
}
void Mp7JrsGeneralRelationType::Setdirected(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Setdirected().");
	}
	GetBase()->Setdirected(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidatedirected()
{
	GetBase()->Invalidatedirected();
}
Mp7JrszeroToOnePtr Mp7JrsGeneralRelationType::Getstrength() const
{
	return GetBase()->Getstrength();
}

bool Mp7JrsGeneralRelationType::Existstrength() const
{
	return GetBase()->Existstrength();
}
void Mp7JrsGeneralRelationType::Setstrength(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Setstrength().");
	}
	GetBase()->Setstrength(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidatestrength()
{
	GetBase()->Invalidatestrength();
}
XMLCh * Mp7JrsGeneralRelationType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsGeneralRelationType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsGeneralRelationType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsGeneralRelationType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsGeneralRelationType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsGeneralRelationType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsGeneralRelationType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsGeneralRelationType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsGeneralRelationType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsGeneralRelationType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsGeneralRelationType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsGeneralRelationType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsGeneralRelationType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsGeneralRelationType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsGeneralRelationType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGeneralRelationType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsGeneralRelationType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsGeneralRelationType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGeneralRelationType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsGeneralRelationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for GeneralRelationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "GeneralRelationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsGeneralRelationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGeneralRelationType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsGeneralRelationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GeneralRelationType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_typelist_Exist)
	{
	// Collection
	if(m_typelist != Mp7JrstermReferenceListPtr())
	{
		XMLCh * tmp = m_typelist->ToText();
		element->setAttributeNS(X(""), X("typelist"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_generalSource_Exist)
	{
	// Collection
	if(m_generalSource != Mp7JrstermReferenceListPtr())
	{
		XMLCh * tmp = m_generalSource->ToText();
		element->setAttributeNS(X(""), X("generalSource"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_generalTarget_Exist)
	{
	// Collection
	if(m_generalTarget != Mp7JrstermReferenceListPtr())
	{
		XMLCh * tmp = m_generalTarget->ToText();
		element->setAttributeNS(X(""), X("generalTarget"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsGeneralRelationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("typelist")))
	{
        // Deserialize collection type
		Mp7JrstermReferenceListPtr tmp = CreatetermReferenceListType;
		tmp->Parse(parent->getAttributeNS(X(""), X("typelist")));
        this->Settypelist(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("typelist")))
	{
     
		// Deserialize collection type
        Mp7JrstermReferenceListPtr tmp = CreatetermReferenceListType;
        tmp->Parse(parent->getAttribute(X("typelist")));
        this->Settypelist(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("generalSource")))
	{
        // Deserialize collection type
		Mp7JrstermReferenceListPtr tmp = CreatetermReferenceListType;
		tmp->Parse(parent->getAttributeNS(X(""), X("generalSource")));
        this->SetgeneralSource(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("generalSource")))
	{
     
		// Deserialize collection type
        Mp7JrstermReferenceListPtr tmp = CreatetermReferenceListType;
        tmp->Parse(parent->getAttribute(X("generalSource")));
        this->SetgeneralSource(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("generalTarget")))
	{
        // Deserialize collection type
		Mp7JrstermReferenceListPtr tmp = CreatetermReferenceListType;
		tmp->Parse(parent->getAttributeNS(X(""), X("generalTarget")));
        this->SetgeneralTarget(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("generalTarget")))
	{
     
		// Deserialize collection type
        Mp7JrstermReferenceListPtr tmp = CreatetermReferenceListType;
        tmp->Parse(parent->getAttribute(X("generalTarget")));
        this->SetgeneralTarget(tmp);
        * current = parent;
    }

	// Extensionbase is Mp7JrsRelationType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGeneralRelationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGeneralRelationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGeneralRelationType_ExtMethodImpl.h


