 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGraphType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsGraphType_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsGraphType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsGraphType_LocalType.h" // Choice collection Node
#include "Mp7JrsGraphType_Node_LocalType.h" // Choice collection element Node
#include "Mp7JrsRelationType.h" // Choice collection element Relation

#include <assert.h>
IMp7JrsGraphType::IMp7JrsGraphType()
{

// no includefile for extension defined 
// file Mp7JrsGraphType_ExtPropInit.h

}

IMp7JrsGraphType::~IMp7JrsGraphType()
{
// no includefile for extension defined 
// file Mp7JrsGraphType_ExtPropCleanup.h

}

Mp7JrsGraphType::Mp7JrsGraphType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGraphType::~Mp7JrsGraphType()
{
	Cleanup();
}

void Mp7JrsGraphType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_GraphType_LocalType = Mp7JrsGraphType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsGraphType_ExtMyPropInit.h

}

void Mp7JrsGraphType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGraphType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_GraphType_LocalType);
}

void Mp7JrsGraphType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GraphTypePtr, since we
	// might need GetBase(), which isn't defined in IGraphType
    const Mp7JrsPtr< Mp7JrsGraphType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsGraphType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_GraphType_LocalType);
		this->SetGraphType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetGraphType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsGraphType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsGraphType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsGraphType::GetBase() const
{
	return m_Base;
}

Mp7JrsGraphType_CollectionPtr Mp7JrsGraphType::GetGraphType_LocalType() const
{
	return m_GraphType_LocalType;
}

void Mp7JrsGraphType::SetGraphType_LocalType(const Mp7JrsGraphType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::SetGraphType_LocalType().");
	}
	if (m_GraphType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_GraphType_LocalType);
		m_GraphType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GraphType_LocalType) m_GraphType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsGraphType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsGraphType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsGraphType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsGraphType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsGraphType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsGraphType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsGraphType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsGraphType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsGraphType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsGraphType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsGraphType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsGraphType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsGraphType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsGraphType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsGraphType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsGraphType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsGraphType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsGraphType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Node")) == 0)
	{
		// Node is contained in itemtype GraphType_LocalType
		// in choice collection GraphType_CollectionType
		
		context->Found = true;
		Mp7JrsGraphType_CollectionPtr coll = GetGraphType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateGraphType_CollectionType;
				SetGraphType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsGraphType_LocalPtr)coll->elementAt(i))->GetNode()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsGraphType_LocalPtr item = CreateGraphType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GraphType_Node_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsGraphType_Node_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsGraphType_LocalPtr)coll->elementAt(i))->SetNode(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsGraphType_LocalPtr)coll->elementAt(i))->GetNode()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Relation")) == 0)
	{
		// Relation is contained in itemtype GraphType_LocalType
		// in choice collection GraphType_CollectionType
		
		context->Found = true;
		Mp7JrsGraphType_CollectionPtr coll = GetGraphType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateGraphType_CollectionType;
				SetGraphType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsGraphType_LocalPtr)coll->elementAt(i))->GetRelation()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsGraphType_LocalPtr item = CreateGraphType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRelationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsGraphType_LocalPtr)coll->elementAt(i))->SetRelation(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsGraphType_LocalPtr)coll->elementAt(i))->GetRelation()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for GraphType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "GraphType");
	}		
	
	return result;
}

XMLCh * Mp7JrsGraphType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsGraphType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsGraphType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsGraphType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GraphType"));
	// Element serialization:
	if (m_GraphType_LocalType != Mp7JrsGraphType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_GraphType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsGraphType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:Node
            Mp7JrsUtil::HasNodeName(parent, X("Node"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Node")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:Relation
            Mp7JrsUtil::HasNodeName(parent, X("Relation"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Relation")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsGraphType_CollectionPtr tmp = CreateGraphType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetGraphType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsGraphType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGraphType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Node"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Node")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Relation"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Relation")) == 0)
    )
  {
    Mp7JrsGraphType_CollectionPtr tmp = CreateGraphType_CollectionType;
    this->SetGraphType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGraphType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetGraphType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetGraphType_LocalType());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGraphType_ExtMethodImpl.h


