 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsHierarchicalSummaryType_ExtImplInclude.h



#include "Mp7JrsSummaryType.h"
#include "Mp7JrsHierarchicalSummaryType_components_CollectionType.h"
#include "Mp7JrsSummaryThemeListType.h"
#include "Mp7JrsHierarchicalSummaryType_CollectionType.h"
#include "Mp7JrsSummaryType_Name_CollectionType.h"
#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsHierarchicalSummaryType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Name
#include "Mp7JrsHierarchicalSummaryType_LocalType.h" // Choice collection SummarySegmentGroup
#include "Mp7JrsSummarySegmentGroupType.h" // Choice collection element SummarySegmentGroup

#include <assert.h>
IMp7JrsHierarchicalSummaryType::IMp7JrsHierarchicalSummaryType()
{

// no includefile for extension defined 
// file Mp7JrsHierarchicalSummaryType_ExtPropInit.h

}

IMp7JrsHierarchicalSummaryType::~IMp7JrsHierarchicalSummaryType()
{
// no includefile for extension defined 
// file Mp7JrsHierarchicalSummaryType_ExtPropCleanup.h

}

Mp7JrsHierarchicalSummaryType::Mp7JrsHierarchicalSummaryType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsHierarchicalSummaryType::~Mp7JrsHierarchicalSummaryType()
{
	Cleanup();
}

void Mp7JrsHierarchicalSummaryType::Init()
{
	// Init base
	m_Base = CreateSummaryType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_components = Mp7JrsHierarchicalSummaryType_components_CollectionPtr(); // Collection
	m_components_Exist = false;
	m_hierarchy = Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::UninitializedEnumeration;

	// Init elements (element, union sequence choice all any)
	
	m_SummaryThemeList = Mp7JrsSummaryThemeListPtr(); // Class
	m_SummaryThemeList_Exist = false;
	m_HierarchicalSummaryType_LocalType = Mp7JrsHierarchicalSummaryType_CollectionPtr(); // Collection


// begin extension included
// file Mp7JrsHierarchicalSummaryType_ExtMyPropInit.h

	// set by default independent
	m_hierarchy = Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::independent;

	// create an empty segment group collection
	m_HierarchicalSummaryType_LocalType = CreateHierarchicalSummaryType_CollectionType;
// end extension included

}

void Mp7JrsHierarchicalSummaryType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsHierarchicalSummaryType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_components);
	// Mp7JrsFactory::DeleteObject(m_SummaryThemeList);
	// Mp7JrsFactory::DeleteObject(m_HierarchicalSummaryType_LocalType);
}

void Mp7JrsHierarchicalSummaryType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use HierarchicalSummaryTypePtr, since we
	// might need GetBase(), which isn't defined in IHierarchicalSummaryType
    const Mp7JrsPtr< Mp7JrsHierarchicalSummaryType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsSummaryPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsHierarchicalSummaryType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_components);
	if (tmp->Existcomponents())
	{
		this->Setcomponents(Mp7JrsFactory::CloneObject(tmp->Getcomponents()));
	}
	else
	{
		Invalidatecomponents();
	}
	}
	{
		this->Sethierarchy(tmp->Gethierarchy());
	}
	if (tmp->IsValidSummaryThemeList())
	{
		// Mp7JrsFactory::DeleteObject(m_SummaryThemeList);
		this->SetSummaryThemeList(Mp7JrsFactory::CloneObject(tmp->GetSummaryThemeList()));
	}
	else
	{
		InvalidateSummaryThemeList();
	}
		// Mp7JrsFactory::DeleteObject(m_HierarchicalSummaryType_LocalType);
		this->SetHierarchicalSummaryType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetHierarchicalSummaryType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsHierarchicalSummaryType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsHierarchicalSummaryType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsSummaryType > Mp7JrsHierarchicalSummaryType::GetBase() const
{
	return m_Base;
}

Mp7JrsHierarchicalSummaryType_components_CollectionPtr Mp7JrsHierarchicalSummaryType::Getcomponents() const
{
	return m_components;
}

bool Mp7JrsHierarchicalSummaryType::Existcomponents() const
{
	return m_components_Exist;
}
void Mp7JrsHierarchicalSummaryType::Setcomponents(const Mp7JrsHierarchicalSummaryType_components_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::Setcomponents().");
	}
    m_components = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_components) m_components->SetParent(m_myself.getPointer());
	m_components_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::Invalidatecomponents()
{
	m_components_Exist = false;
}
Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::Enumeration Mp7JrsHierarchicalSummaryType::Gethierarchy() const
{
	return m_hierarchy;
}

void Mp7JrsHierarchicalSummaryType::Sethierarchy(Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::Sethierarchy().");
	}
    m_hierarchy = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsSummaryThemeListPtr Mp7JrsHierarchicalSummaryType::GetSummaryThemeList() const
{
	return m_SummaryThemeList;
}

// Element is optional
bool Mp7JrsHierarchicalSummaryType::IsValidSummaryThemeList() const
{
	return m_SummaryThemeList_Exist;
}

Mp7JrsHierarchicalSummaryType_CollectionPtr Mp7JrsHierarchicalSummaryType::GetHierarchicalSummaryType_LocalType() const
{
	return m_HierarchicalSummaryType_LocalType;
}

void Mp7JrsHierarchicalSummaryType::SetSummaryThemeList(const Mp7JrsSummaryThemeListPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetSummaryThemeList().");
	}
	if (m_SummaryThemeList != item || m_SummaryThemeList_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SummaryThemeList);
		m_SummaryThemeList = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SummaryThemeList) m_SummaryThemeList->SetParent(m_myself.getPointer());
		if (m_SummaryThemeList && m_SummaryThemeList->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SummaryThemeListType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SummaryThemeList->UseTypeAttribute = true;
		}
		m_SummaryThemeList_Exist = true;
		if(m_SummaryThemeList != Mp7JrsSummaryThemeListPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SummaryThemeList->SetContentName(XMLString::transcode("SummaryThemeList"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidateSummaryThemeList()
{
	m_SummaryThemeList_Exist = false;
}
void Mp7JrsHierarchicalSummaryType::SetHierarchicalSummaryType_LocalType(const Mp7JrsHierarchicalSummaryType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetHierarchicalSummaryType_LocalType().");
	}
	if (m_HierarchicalSummaryType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_HierarchicalSummaryType_LocalType);
		m_HierarchicalSummaryType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HierarchicalSummaryType_LocalType) m_HierarchicalSummaryType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsSummaryType_Name_CollectionPtr Mp7JrsHierarchicalSummaryType::GetName() const
{
	return GetBase()->GetName();
}

Mp7JrsUniqueIDPtr Mp7JrsHierarchicalSummaryType::GetSourceID() const
{
	return GetBase()->GetSourceID();
}

// Element is optional
bool Mp7JrsHierarchicalSummaryType::IsValidSourceID() const
{
	return GetBase()->IsValidSourceID();
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsHierarchicalSummaryType::GetSourceLocator() const
{
	return GetBase()->GetSourceLocator();
}

// Element is optional
bool Mp7JrsHierarchicalSummaryType::IsValidSourceLocator() const
{
	return GetBase()->IsValidSourceLocator();
}

Mp7JrsReferencePtr Mp7JrsHierarchicalSummaryType::GetSourceInformation() const
{
	return GetBase()->GetSourceInformation();
}

// Element is optional
bool Mp7JrsHierarchicalSummaryType::IsValidSourceInformation() const
{
	return GetBase()->IsValidSourceInformation();
}

void Mp7JrsHierarchicalSummaryType::SetName(const Mp7JrsSummaryType_Name_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetName().");
	}
	GetBase()->SetName(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::SetSourceID(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetSourceID().");
	}
	GetBase()->SetSourceID(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidateSourceID()
{
	GetBase()->InvalidateSourceID();
}
void Mp7JrsHierarchicalSummaryType::SetSourceLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetSourceLocator().");
	}
	GetBase()->SetSourceLocator(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidateSourceLocator()
{
	GetBase()->InvalidateSourceLocator();
}
void Mp7JrsHierarchicalSummaryType::SetSourceInformation(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetSourceInformation().");
	}
	GetBase()->SetSourceInformation(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidateSourceInformation()
{
	GetBase()->InvalidateSourceInformation();
}
XMLCh * Mp7JrsHierarchicalSummaryType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsHierarchicalSummaryType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsHierarchicalSummaryType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsHierarchicalSummaryType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsHierarchicalSummaryType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsHierarchicalSummaryType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsHierarchicalSummaryType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsHierarchicalSummaryType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsHierarchicalSummaryType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsHierarchicalSummaryType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsHierarchicalSummaryType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsHierarchicalSummaryType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsHierarchicalSummaryType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsHierarchicalSummaryType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsHierarchicalSummaryType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHierarchicalSummaryType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsHierarchicalSummaryType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsHierarchicalSummaryType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHierarchicalSummaryType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsHierarchicalSummaryType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("SummaryThemeList")) == 0)
	{
		// SummaryThemeList is simple element SummaryThemeListType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSummaryThemeList()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SummaryThemeListType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSummaryThemeListPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSummaryThemeList(p, client);
					if((p = GetSummaryThemeList()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SummarySegmentGroup")) == 0)
	{
		// SummarySegmentGroup is contained in itemtype HierarchicalSummaryType_LocalType
		// in choice collection HierarchicalSummaryType_CollectionType
		
		context->Found = true;
		Mp7JrsHierarchicalSummaryType_CollectionPtr coll = GetHierarchicalSummaryType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateHierarchicalSummaryType_CollectionType;
				SetHierarchicalSummaryType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsHierarchicalSummaryType_LocalPtr)coll->elementAt(i))->GetSummarySegmentGroup()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsHierarchicalSummaryType_LocalPtr item = CreateHierarchicalSummaryType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SummarySegmentGroupType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSummarySegmentGroupPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsHierarchicalSummaryType_LocalPtr)coll->elementAt(i))->SetSummarySegmentGroup(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsHierarchicalSummaryType_LocalPtr)coll->elementAt(i))->GetSummarySegmentGroup()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SummarySegmentGroupRef")) == 0)
	{
		// SummarySegmentGroupRef is contained in itemtype HierarchicalSummaryType_LocalType
		// in choice collection HierarchicalSummaryType_CollectionType
		
		context->Found = true;
		Mp7JrsHierarchicalSummaryType_CollectionPtr coll = GetHierarchicalSummaryType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateHierarchicalSummaryType_CollectionType;
				SetHierarchicalSummaryType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsHierarchicalSummaryType_LocalPtr)coll->elementAt(i))->GetSummarySegmentGroupRef()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsHierarchicalSummaryType_LocalPtr item = CreateHierarchicalSummaryType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsHierarchicalSummaryType_LocalPtr)coll->elementAt(i))->SetSummarySegmentGroupRef(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsHierarchicalSummaryType_LocalPtr)coll->elementAt(i))->GetSummarySegmentGroupRef()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for HierarchicalSummaryType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "HierarchicalSummaryType");
	}		
	
	return result;
}

XMLCh * Mp7JrsHierarchicalSummaryType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsHierarchicalSummaryType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsHierarchicalSummaryType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsHierarchicalSummaryType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("HierarchicalSummaryType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_components_Exist)
	{
	// Collection
	if(m_components != Mp7JrsHierarchicalSummaryType_components_CollectionPtr())
	{
		XMLCh * tmp = m_components->ToText();
		element->setAttributeNS(X(""), X("components"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
	// Enumeration
	if(m_hierarchy != Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::ToText(m_hierarchy);
		element->setAttributeNS(X(""), X("hierarchy"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	// Class
	
	if (m_SummaryThemeList != Mp7JrsSummaryThemeListPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SummaryThemeList->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SummaryThemeList"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SummaryThemeList->Serialize(doc, element, &elem);
	}
	if (m_HierarchicalSummaryType_LocalType != Mp7JrsHierarchicalSummaryType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_HierarchicalSummaryType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsHierarchicalSummaryType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("components")))
	{
        // Deserialize collection type
		Mp7JrsHierarchicalSummaryType_components_CollectionPtr tmp = CreateHierarchicalSummaryType_components_CollectionType;
		tmp->Parse(parent->getAttributeNS(X(""), X("components")));
        this->Setcomponents(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("components")))
	{
     
		// Deserialize collection type
        Mp7JrsHierarchicalSummaryType_components_CollectionPtr tmp = CreateHierarchicalSummaryType_components_CollectionType;
        tmp->Parse(parent->getAttribute(X("components")));
        this->Setcomponents(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("hierarchy")))
	{
        this->Sethierarchy(Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::Parse(parent->getAttributeNS(X(""), X("hierarchy"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("hierarchy")))
	{
        this->Sethierarchy(Mp7JrsHierarchicalSummaryType_hierarchy_LocalType::Parse(parent->getAttribute(X("hierarchy"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsSummaryType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SummaryThemeList"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SummaryThemeList")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSummaryThemeListType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSummaryThemeList(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:SummarySegmentGroup
            Mp7JrsUtil::HasNodeName(parent, X("SummarySegmentGroup"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:SummarySegmentGroup")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:SummarySegmentGroupRef
            Mp7JrsUtil::HasNodeName(parent, X("SummarySegmentGroupRef"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:SummarySegmentGroupRef")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsHierarchicalSummaryType_CollectionPtr tmp = CreateHierarchicalSummaryType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetHierarchicalSummaryType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsHierarchicalSummaryType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsHierarchicalSummaryType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("SummaryThemeList")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSummaryThemeListType;
    }
    this->SetSummaryThemeList(child);
  }
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("SummarySegmentGroup"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("SummarySegmentGroup")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("SummarySegmentGroupRef"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("SummarySegmentGroupRef")) == 0)
    )
  {
    Mp7JrsHierarchicalSummaryType_CollectionPtr tmp = CreateHierarchicalSummaryType_CollectionType;
    this->SetHierarchicalSummaryType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsHierarchicalSummaryType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSummaryThemeList() != Mp7JrsNodePtr())
		result.Insert(GetSummaryThemeList());
	if (GetHierarchicalSummaryType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetHierarchicalSummaryType_LocalType());
	if (GetName() != Mp7JrsNodePtr())
		result.Insert(GetName());
	if (GetSourceID() != Mp7JrsNodePtr())
		result.Insert(GetSourceID());
	if (GetSourceLocator() != Mp7JrsNodePtr())
		result.Insert(GetSourceLocator());
	if (GetSourceInformation() != Mp7JrsNodePtr())
		result.Insert(GetSourceInformation());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// begin extension included
// file Mp7JrsHierarchicalSummaryType_ExtMethodImpl.h
#include "Mp7JrsSummarySegmentGroupType.h"
#include "Mp7JrsHierarchicalSummaryType_LocalType.h"

void IMp7JrsHierarchicalSummaryType::AddSegmentGroup(Mp7JrsSummarySegmentGroupPtr segmentGroup, Mp7JrsClientID client) {
	Mp7JrsHierarchicalSummaryType_LocalPtr seggrp = CreateHierarchicalSummaryType_LocalType;
	seggrp->SetSummarySegmentGroup(segmentGroup,client);
	GetHierarchicalSummaryType_LocalType()->addElement(seggrp,client);
}
// end extension included


