 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtImplInclude.h



#include "Mp7JrsDominantColorType.h"
#include "Mp7JrsScalableColorType.h"
#include "Mp7JrsColorLayoutType.h"
#include "Mp7JrsColorStructureType.h"
#include "Mp7JrsIlluminationInvariantColorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsIlluminationInvariantColorType::IMp7JrsIlluminationInvariantColorType()
{

// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtPropInit.h

}

IMp7JrsIlluminationInvariantColorType::~IMp7JrsIlluminationInvariantColorType()
{
// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtPropCleanup.h

}

Mp7JrsIlluminationInvariantColorType::Mp7JrsIlluminationInvariantColorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsIlluminationInvariantColorType::~Mp7JrsIlluminationInvariantColorType()
{
	Cleanup();
}

void Mp7JrsIlluminationInvariantColorType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_DominantColor = Mp7JrsDominantColorPtr(); // Class
	m_ScalableColor = Mp7JrsScalableColorPtr(); // Class
	m_ColorLayout = Mp7JrsColorLayoutPtr(); // Class
	m_ColorStructure = Mp7JrsColorStructurePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtMyPropInit.h

}

void Mp7JrsIlluminationInvariantColorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	// Mp7JrsFactory::DeleteObject(m_ColorLayout);
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
}

void Mp7JrsIlluminationInvariantColorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use IlluminationInvariantColorTypePtr, since we
	// might need GetBase(), which isn't defined in IIlluminationInvariantColorType
    const Mp7JrsPtr< Mp7JrsIlluminationInvariantColorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_DominantColor);
		this->SetDominantColor(Mp7JrsFactory::CloneObject(tmp->GetDominantColor()));
		// Mp7JrsFactory::DeleteObject(m_ScalableColor);
		this->SetScalableColor(Mp7JrsFactory::CloneObject(tmp->GetScalableColor()));
		// Mp7JrsFactory::DeleteObject(m_ColorLayout);
		this->SetColorLayout(Mp7JrsFactory::CloneObject(tmp->GetColorLayout()));
		// Mp7JrsFactory::DeleteObject(m_ColorStructure);
		this->SetColorStructure(Mp7JrsFactory::CloneObject(tmp->GetColorStructure()));
}

Mp7JrsNodePtr Mp7JrsIlluminationInvariantColorType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsIlluminationInvariantColorType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsDominantColorPtr Mp7JrsIlluminationInvariantColorType::GetDominantColor() const
{
	return m_DominantColor;
}

Mp7JrsScalableColorPtr Mp7JrsIlluminationInvariantColorType::GetScalableColor() const
{
	return m_ScalableColor;
}

Mp7JrsColorLayoutPtr Mp7JrsIlluminationInvariantColorType::GetColorLayout() const
{
	return m_ColorLayout;
}

Mp7JrsColorStructurePtr Mp7JrsIlluminationInvariantColorType::GetColorStructure() const
{
	return m_ColorStructure;
}

// implementing setter for choice 
/* element */
void Mp7JrsIlluminationInvariantColorType::SetDominantColor(const Mp7JrsDominantColorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsIlluminationInvariantColorType::SetDominantColor().");
	}
	if (m_DominantColor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_DominantColor);
		m_DominantColor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DominantColor) m_DominantColor->SetParent(m_myself.getPointer());
		if (m_DominantColor && m_DominantColor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DominantColorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DominantColor->UseTypeAttribute = true;
		}
		if(m_DominantColor != Mp7JrsDominantColorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DominantColor->SetContentName(XMLString::transcode("DominantColor"));
		}
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	m_ScalableColor = Mp7JrsScalableColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorLayout);
	m_ColorLayout = Mp7JrsColorLayoutPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
	m_ColorStructure = Mp7JrsColorStructurePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsIlluminationInvariantColorType::SetScalableColor(const Mp7JrsScalableColorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsIlluminationInvariantColorType::SetScalableColor().");
	}
	if (m_ScalableColor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ScalableColor);
		m_ScalableColor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ScalableColor) m_ScalableColor->SetParent(m_myself.getPointer());
		if (m_ScalableColor && m_ScalableColor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ScalableColorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ScalableColor->UseTypeAttribute = true;
		}
		if(m_ScalableColor != Mp7JrsScalableColorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ScalableColor->SetContentName(XMLString::transcode("ScalableColor"));
		}
	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	m_DominantColor = Mp7JrsDominantColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorLayout);
	m_ColorLayout = Mp7JrsColorLayoutPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
	m_ColorStructure = Mp7JrsColorStructurePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsIlluminationInvariantColorType::SetColorLayout(const Mp7JrsColorLayoutPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsIlluminationInvariantColorType::SetColorLayout().");
	}
	if (m_ColorLayout != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorLayout);
		m_ColorLayout = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorLayout) m_ColorLayout->SetParent(m_myself.getPointer());
		if (m_ColorLayout && m_ColorLayout->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorLayoutType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorLayout->UseTypeAttribute = true;
		}
		if(m_ColorLayout != Mp7JrsColorLayoutPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorLayout->SetContentName(XMLString::transcode("ColorLayout"));
		}
	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	m_DominantColor = Mp7JrsDominantColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	m_ScalableColor = Mp7JrsScalableColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorStructure);
	m_ColorStructure = Mp7JrsColorStructurePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsIlluminationInvariantColorType::SetColorStructure(const Mp7JrsColorStructurePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsIlluminationInvariantColorType::SetColorStructure().");
	}
	if (m_ColorStructure != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorStructure);
		m_ColorStructure = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorStructure) m_ColorStructure->SetParent(m_myself.getPointer());
		if (m_ColorStructure && m_ColorStructure->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorStructureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorStructure->UseTypeAttribute = true;
		}
		if(m_ColorStructure != Mp7JrsColorStructurePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorStructure->SetContentName(XMLString::transcode("ColorStructure"));
		}
	// Mp7JrsFactory::DeleteObject(m_DominantColor);
	m_DominantColor = Mp7JrsDominantColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ScalableColor);
	m_ScalableColor = Mp7JrsScalableColorPtr();
	// Mp7JrsFactory::DeleteObject(m_ColorLayout);
	m_ColorLayout = Mp7JrsColorLayoutPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsIlluminationInvariantColorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("DominantColor")) == 0)
	{
		// DominantColor is simple element DominantColorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDominantColor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DominantColorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDominantColorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDominantColor(p, client);
					if((p = GetDominantColor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ScalableColor")) == 0)
	{
		// ScalableColor is simple element ScalableColorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetScalableColor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ScalableColorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsScalableColorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetScalableColor(p, client);
					if((p = GetScalableColor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ColorLayout")) == 0)
	{
		// ColorLayout is simple element ColorLayoutType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorLayout()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorLayoutType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorLayoutPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorLayout(p, client);
					if((p = GetColorLayout()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ColorStructure")) == 0)
	{
		// ColorStructure is simple element ColorStructureType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorStructure()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorStructureType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorStructurePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorStructure(p, client);
					if((p = GetColorStructure()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for IlluminationInvariantColorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "IlluminationInvariantColorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsIlluminationInvariantColorType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsIlluminationInvariantColorType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsIlluminationInvariantColorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("IlluminationInvariantColorType"));
	// Element serialization:
	// Class
	
	if (m_DominantColor != Mp7JrsDominantColorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DominantColor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DominantColor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DominantColor->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ScalableColor != Mp7JrsScalableColorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ScalableColor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ScalableColor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ScalableColor->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ColorLayout != Mp7JrsColorLayoutPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorLayout->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorLayout"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorLayout->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ColorStructure != Mp7JrsColorStructurePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorStructure->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorStructure"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorStructure->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsIlluminationInvariantColorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DominantColor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DominantColor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDominantColorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDominantColor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ScalableColor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ScalableColor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateScalableColorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetScalableColor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorLayout"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorLayout")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorLayoutType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorLayout(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorStructure"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorStructure")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorStructureType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorStructure(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsIlluminationInvariantColorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("DominantColor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDominantColorType;
    }
    this->SetDominantColor(child);
  }
  if (XMLString::compareString(elementname, X("ScalableColor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateScalableColorType;
    }
    this->SetScalableColor(child);
  }
  if (XMLString::compareString(elementname, X("ColorLayout")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorLayoutType;
    }
    this->SetColorLayout(child);
  }
  if (XMLString::compareString(elementname, X("ColorStructure")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorStructureType;
    }
    this->SetColorStructure(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsIlluminationInvariantColorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDominantColor() != Mp7JrsNodePtr())
		result.Insert(GetDominantColor());
	if (GetScalableColor() != Mp7JrsNodePtr())
		result.Insert(GetScalableColor());
	if (GetColorLayout() != Mp7JrsNodePtr())
		result.Insert(GetColorLayout());
	if (GetColorStructure() != Mp7JrsNodePtr())
		result.Insert(GetColorStructure());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsIlluminationInvariantColorType_ExtMethodImpl.h


