 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtImplInclude.h



#include "Mp7JrsmimeType.h"
#include "Mp7JrsInlineMediaType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsInlineMediaType::IMp7JrsInlineMediaType()
{

// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtPropInit.h

}

IMp7JrsInlineMediaType::~IMp7JrsInlineMediaType()
{
// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtPropCleanup.h

}

Mp7JrsInlineMediaType::Mp7JrsInlineMediaType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsInlineMediaType::~Mp7JrsInlineMediaType()
{
	Cleanup();
}

void Mp7JrsInlineMediaType::Init()
{

	// Init attributes
	m_type = Mp7JrsmimePtr(); // Pattern

	// Init elements (element, union sequence choice all any)
	
	// TODO: Binary data for elements is not implemented the full way  (just as string)
	m_MediaData16 = NULL; // Binary
	// TODO: Binary data for elements is not implemented the full way  (just as string)
	m_MediaData64 = NULL; // Binary


// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtMyPropInit.h

}

void Mp7JrsInlineMediaType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_type); 
	// TODO Binary data for elements is not implemented (just as string)
	XMLString::release(&this->m_MediaData16);
	this->m_MediaData16 = NULL;
	// TODO Binary data for elements is not implemented (just as string)
	XMLString::release(&this->m_MediaData64);
	this->m_MediaData64 = NULL;
}

void Mp7JrsInlineMediaType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use InlineMediaTypePtr, since we
	// might need GetBase(), which isn't defined in IInlineMediaType
    const Mp7JrsPtr< Mp7JrsInlineMediaType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_type); 
		this->Settype(Mp7JrsFactory::CloneObject(tmp->Gettype()));
	}
		// TODO Binary data for elements is not implemented  (just as string)
		this->SetMediaData16(XMLString::replicate(tmp->GetMediaData16()));
		// TODO Binary data for elements is not implemented  (just as string)
		this->SetMediaData64(XMLString::replicate(tmp->GetMediaData64()));
}

Mp7JrsNodePtr Mp7JrsInlineMediaType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsInlineMediaType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsmimePtr Mp7JrsInlineMediaType::Gettype() const
{
	return m_type;
}

void Mp7JrsInlineMediaType::Settype(const Mp7JrsmimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInlineMediaType::Settype().");
	}
    m_type = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_type) m_type->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsInlineMediaType::GetMediaData16() const
{
	return m_MediaData16;
}

XMLCh * Mp7JrsInlineMediaType::GetMediaData64() const
{
	return m_MediaData64;
}

// implementing setter for choice 
/* element */
void Mp7JrsInlineMediaType::SetMediaData16(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInlineMediaType::SetMediaData16().");
	}
	if (m_MediaData16 != item)
	{
		// TODO Binary data for elements is not implemented (just as string)
		XMLString::release(&m_MediaData16);
		m_MediaData16 = item;
	// TODO Binary data for elements is not implemented (just as string)
	// TODO Binary data for elements is not implemented (just as string)
  //	if(m_MediaData64 = NULL; // FTT Shouldn't it be XMLString::release()?
  if(!m_MediaData64) XMLString::release(&m_MediaData64); // Just for testing
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsInlineMediaType::SetMediaData64(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInlineMediaType::SetMediaData64().");
	}
	if (m_MediaData64 != item)
	{
		// TODO Binary data for elements is not implemented (just as string)
		XMLString::release(&m_MediaData64);
		m_MediaData64 = item;
	// TODO Binary data for elements is not implemented (just as string)
	// TODO Binary data for elements is not implemented (just as string)
  //	if(m_MediaData16 = NULL; // FTT Shouldn't it be XMLString::release()?
  if(!m_MediaData16) XMLString::release(&m_MediaData16); // Just for testing
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsInlineMediaType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsInlineMediaType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsInlineMediaType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsInlineMediaType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("InlineMediaType"));
    // Attribute Serialization:
	// Pattern
	if (m_type != Mp7JrsmimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_type->ToText();
		element->setAttributeNS(X(""), X("type"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	// TODO Binary data for elements is not implemented (just as string)
	if(m_MediaData16 != (XMLCh*)NULL)Mp7JrsUtil::SerializeTextNode(element, doc, m_MediaData16, X("urn:mpeg:mpeg7:schema:2004"), X("MediaData16"), false);
	
#if 0 // TODELETE  old code	
	if(m_MediaData16 != (XMLCh*)NULL) 
	{
		DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaData16"));
		const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
		elem->setPrefix(prefix);
		
		DOMText * text = doc->createTextNode(m_MediaData16);
		elem->appendChild(text);
		element->appendChild(elem);
	}
#endif		
	// TODO Binary data for elements is not implemented (just as string)
	if(m_MediaData64 != (XMLCh*)NULL)Mp7JrsUtil::SerializeTextNode(element, doc, m_MediaData64, X("urn:mpeg:mpeg7:schema:2004"), X("MediaData64"), false);
	
#if 0 // TODELETE  old code	
	if(m_MediaData64 != (XMLCh*)NULL) 
	{
		DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaData64"));
		const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
		elem->setPrefix(prefix);
		
		DOMText * text = doc->createTextNode(m_MediaData64);
		elem->appendChild(text);
		element->appendChild(elem);
	}
#endif		

}

bool Mp7JrsInlineMediaType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("type")))
	{
        Mp7JrsmimePtr tmp = CreatemimeType;
        tmp->Parse(parent->getAttributeNS(X(""), X("type")));
        this->Settype(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("type")))
	{
        Mp7JrsmimePtr tmp = CreatemimeType;
        tmp->Parse(parent->getAttribute(X("type")));
        this->Settype(tmp);
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
    {
        // Deserialize binary element
        // TODO Binary data for elements is not implemented (just as string)
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaData16"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName() , X("MediaData16")) == 0))
        {
            // Deserialize binary (XMLCh *)
            this->SetMediaData16(Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent)));
            found = true;
            * current = parent;
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize binary element
        // TODO Binary data for elements is not implemented (just as string)
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaData64"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName() , X("MediaData64")) == 0))
        {
            // Deserialize binary (XMLCh *)
            this->SetMediaData64(Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent)));
            found = true;
            * current = parent;
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsInlineMediaType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsInlineMediaType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsInlineMediaType_ExtMethodImpl.h


