 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtImplInclude.h



#include "Mp7JrsKeywordAnnotationType_Keyword_CollectionType.h"
#include "Mp7JrsKeywordAnnotationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsKeywordAnnotationType_Keyword_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Keyword

#include <assert.h>
IMp7JrsKeywordAnnotationType::IMp7JrsKeywordAnnotationType()
{

// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtPropInit.h

}

IMp7JrsKeywordAnnotationType::~IMp7JrsKeywordAnnotationType()
{
// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtPropCleanup.h

}

Mp7JrsKeywordAnnotationType::Mp7JrsKeywordAnnotationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsKeywordAnnotationType::~Mp7JrsKeywordAnnotationType()
{
	Cleanup();
}

void Mp7JrsKeywordAnnotationType::Init()
{

	// Init attributes
	m_lang = NULL; // String
	m_lang_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Keyword = Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtMyPropInit.h

}

void Mp7JrsKeywordAnnotationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtMyPropCleanup.h


	XMLString::release(&m_lang);
	// Mp7JrsFactory::DeleteObject(m_Keyword);
}

void Mp7JrsKeywordAnnotationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use KeywordAnnotationTypePtr, since we
	// might need GetBase(), which isn't defined in IKeywordAnnotationType
    const Mp7JrsPtr< Mp7JrsKeywordAnnotationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	XMLString::release(&m_lang);
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Keyword);
		this->SetKeyword(Mp7JrsFactory::CloneObject(tmp->GetKeyword()));
}

Mp7JrsNodePtr Mp7JrsKeywordAnnotationType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsKeywordAnnotationType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


XMLCh * Mp7JrsKeywordAnnotationType::Getlang() const
{
	return m_lang;
}

bool Mp7JrsKeywordAnnotationType::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsKeywordAnnotationType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsKeywordAnnotationType::Setlang().");
	}
    m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsKeywordAnnotationType::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr Mp7JrsKeywordAnnotationType::GetKeyword() const
{
	return m_Keyword;
}

void Mp7JrsKeywordAnnotationType::SetKeyword(const Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsKeywordAnnotationType::SetKeyword().");
	}
	if (m_Keyword != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Keyword);
		m_Keyword = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Keyword) m_Keyword->SetParent(m_myself.getPointer());
		if(m_Keyword != Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Keyword->SetContentName(XMLString::transcode("Keyword"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsKeywordAnnotationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Keyword")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Keyword is item of type KeywordAnnotationType_Keyword_LocalType
		// in element collection KeywordAnnotationType_Keyword_CollectionType
		
		context->Found = true;
		Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr coll = GetKeyword();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateKeywordAnnotationType_Keyword_CollectionType;
				SetKeyword(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:KeywordAnnotationType_Keyword_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsKeywordAnnotationType_Keyword_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for KeywordAnnotationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "KeywordAnnotationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsKeywordAnnotationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsKeywordAnnotationType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsKeywordAnnotationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("KeywordAnnotationType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Keyword != Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Keyword->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsKeywordAnnotationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttribute(X("lang"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Keyword"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Keyword")) == 0))
        {
            // Deserialize factory type
            Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr tmp = CreateKeywordAnnotationType_Keyword_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetKeyword(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsKeywordAnnotationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Keyword")) == 0))
  {
    Mp7JrsKeywordAnnotationType_Keyword_CollectionPtr tmp = CreateKeywordAnnotationType_Keyword_CollectionType;
    this->SetKeyword(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsKeywordAnnotationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetKeyword() != Mp7JrsNodePtr())
		result.Insert(GetKeyword());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsKeywordAnnotationType_ExtMethodImpl.h


