 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsControlledTermUseType.h"
#include "Mp7JrsMediaFormatType_BitRate_LocalType.h"
#include "Mp7JrsMediaFormatType_ScalableCoding_LocalType2.h"
#include "Mp7JrsMediaFormatType_VisualCoding_LocalType.h"
#include "Mp7JrsMediaFormatType_AudioCoding_LocalType.h"
#include "Mp7JrsMediaFormatType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsMediaFormatType::IMp7JrsMediaFormatType()
{

// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtPropInit.h

}

IMp7JrsMediaFormatType::~IMp7JrsMediaFormatType()
{
// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtPropCleanup.h

}

Mp7JrsMediaFormatType::Mp7JrsMediaFormatType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaFormatType::~Mp7JrsMediaFormatType()
{
	Cleanup();
}

void Mp7JrsMediaFormatType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Content = Mp7JrsControlledTermUsePtr(); // Class
	m_Medium = Mp7JrsControlledTermUsePtr(); // Class
	m_Medium_Exist = false;
	m_FileFormat = Mp7JrsControlledTermUsePtr(); // Class
	m_FileFormat_Exist = false;
	m_FileSize = (0); // Value

	m_FileSize_Exist = false;
	m_System = Mp7JrsControlledTermUsePtr(); // Class
	m_System_Exist = false;
	m_Bandwidth = (0.0); // Value

	m_Bandwidth_Exist = false;
	m_BitRate = Mp7JrsMediaFormatType_BitRate_LocalPtr(); // Class with content 
	m_BitRate_Exist = false;
	m_TargetChannelBitRate = (0); // Value

	m_TargetChannelBitRate_Exist = false;
	m_ScalableCoding = Mp7JrsMediaFormatType_ScalableCoding_Local2Ptr(); // Class
	m_ScalableCoding_Exist = false;
	m_VisualCoding = Mp7JrsMediaFormatType_VisualCoding_LocalPtr(); // Class
	m_VisualCoding_Exist = false;
	m_AudioCoding = Mp7JrsMediaFormatType_AudioCoding_LocalPtr(); // Class
	m_AudioCoding_Exist = false;
	m_SceneCodingFormat = Mp7JrsControlledTermUsePtr(); // Class
	m_SceneCodingFormat_Exist = false;
	m_GraphicsCodingFormat = Mp7JrsControlledTermUsePtr(); // Class
	m_GraphicsCodingFormat_Exist = false;
	m_OtherCodingFormat = Mp7JrsControlledTermUsePtr(); // Class
	m_OtherCodingFormat_Exist = false;


// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtMyPropInit.h

}

void Mp7JrsMediaFormatType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Content);
	// Mp7JrsFactory::DeleteObject(m_Medium);
	// Mp7JrsFactory::DeleteObject(m_FileFormat);
	// Mp7JrsFactory::DeleteObject(m_System);
	// Mp7JrsFactory::DeleteObject(m_BitRate);
	// Mp7JrsFactory::DeleteObject(m_ScalableCoding);
	// Mp7JrsFactory::DeleteObject(m_VisualCoding);
	// Mp7JrsFactory::DeleteObject(m_AudioCoding);
	// Mp7JrsFactory::DeleteObject(m_SceneCodingFormat);
	// Mp7JrsFactory::DeleteObject(m_GraphicsCodingFormat);
	// Mp7JrsFactory::DeleteObject(m_OtherCodingFormat);
}

void Mp7JrsMediaFormatType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaFormatTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaFormatType
    const Mp7JrsPtr< Mp7JrsMediaFormatType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMediaFormatType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Content);
		this->SetContent(Mp7JrsFactory::CloneObject(tmp->GetContent()));
	if (tmp->IsValidMedium())
	{
		// Mp7JrsFactory::DeleteObject(m_Medium);
		this->SetMedium(Mp7JrsFactory::CloneObject(tmp->GetMedium()));
	}
	else
	{
		InvalidateMedium();
	}
	if (tmp->IsValidFileFormat())
	{
		// Mp7JrsFactory::DeleteObject(m_FileFormat);
		this->SetFileFormat(Mp7JrsFactory::CloneObject(tmp->GetFileFormat()));
	}
	else
	{
		InvalidateFileFormat();
	}
	if (tmp->IsValidFileSize())
	{
		this->SetFileSize(tmp->GetFileSize());
	}
	else
	{
		InvalidateFileSize();
	}
	if (tmp->IsValidSystem())
	{
		// Mp7JrsFactory::DeleteObject(m_System);
		this->SetSystem(Mp7JrsFactory::CloneObject(tmp->GetSystem()));
	}
	else
	{
		InvalidateSystem();
	}
	if (tmp->IsValidBandwidth())
	{
		this->SetBandwidth(tmp->GetBandwidth());
	}
	else
	{
		InvalidateBandwidth();
	}
	if (tmp->IsValidBitRate())
	{
		// Mp7JrsFactory::DeleteObject(m_BitRate);
		this->SetBitRate(Mp7JrsFactory::CloneObject(tmp->GetBitRate()));
	}
	else
	{
		InvalidateBitRate();
	}
	if (tmp->IsValidTargetChannelBitRate())
	{
		this->SetTargetChannelBitRate(tmp->GetTargetChannelBitRate());
	}
	else
	{
		InvalidateTargetChannelBitRate();
	}
	if (tmp->IsValidScalableCoding())
	{
		// Mp7JrsFactory::DeleteObject(m_ScalableCoding);
		this->SetScalableCoding(Mp7JrsFactory::CloneObject(tmp->GetScalableCoding()));
	}
	else
	{
		InvalidateScalableCoding();
	}
	if (tmp->IsValidVisualCoding())
	{
		// Mp7JrsFactory::DeleteObject(m_VisualCoding);
		this->SetVisualCoding(Mp7JrsFactory::CloneObject(tmp->GetVisualCoding()));
	}
	else
	{
		InvalidateVisualCoding();
	}
	if (tmp->IsValidAudioCoding())
	{
		// Mp7JrsFactory::DeleteObject(m_AudioCoding);
		this->SetAudioCoding(Mp7JrsFactory::CloneObject(tmp->GetAudioCoding()));
	}
	else
	{
		InvalidateAudioCoding();
	}
	if (tmp->IsValidSceneCodingFormat())
	{
		// Mp7JrsFactory::DeleteObject(m_SceneCodingFormat);
		this->SetSceneCodingFormat(Mp7JrsFactory::CloneObject(tmp->GetSceneCodingFormat()));
	}
	else
	{
		InvalidateSceneCodingFormat();
	}
	if (tmp->IsValidGraphicsCodingFormat())
	{
		// Mp7JrsFactory::DeleteObject(m_GraphicsCodingFormat);
		this->SetGraphicsCodingFormat(Mp7JrsFactory::CloneObject(tmp->GetGraphicsCodingFormat()));
	}
	else
	{
		InvalidateGraphicsCodingFormat();
	}
	if (tmp->IsValidOtherCodingFormat())
	{
		// Mp7JrsFactory::DeleteObject(m_OtherCodingFormat);
		this->SetOtherCodingFormat(Mp7JrsFactory::CloneObject(tmp->GetOtherCodingFormat()));
	}
	else
	{
		InvalidateOtherCodingFormat();
	}
}

Mp7JrsNodePtr Mp7JrsMediaFormatType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMediaFormatType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsMediaFormatType::GetBase() const
{
	return m_Base;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetContent() const
{
	return m_Content;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetMedium() const
{
	return m_Medium;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidMedium() const
{
	return m_Medium_Exist;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetFileFormat() const
{
	return m_FileFormat;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidFileFormat() const
{
	return m_FileFormat_Exist;
}

unsigned Mp7JrsMediaFormatType::GetFileSize() const
{
	return m_FileSize;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidFileSize() const
{
	return m_FileSize_Exist;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetSystem() const
{
	return m_System;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidSystem() const
{
	return m_System_Exist;
}

float Mp7JrsMediaFormatType::GetBandwidth() const
{
	return m_Bandwidth;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidBandwidth() const
{
	return m_Bandwidth_Exist;
}

Mp7JrsMediaFormatType_BitRate_LocalPtr Mp7JrsMediaFormatType::GetBitRate() const
{
	return m_BitRate;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidBitRate() const
{
	return m_BitRate_Exist;
}

unsigned Mp7JrsMediaFormatType::GetTargetChannelBitRate() const
{
	return m_TargetChannelBitRate;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidTargetChannelBitRate() const
{
	return m_TargetChannelBitRate_Exist;
}

Mp7JrsMediaFormatType_ScalableCoding_Local2Ptr Mp7JrsMediaFormatType::GetScalableCoding() const
{
	return m_ScalableCoding;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidScalableCoding() const
{
	return m_ScalableCoding_Exist;
}

Mp7JrsMediaFormatType_VisualCoding_LocalPtr Mp7JrsMediaFormatType::GetVisualCoding() const
{
	return m_VisualCoding;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidVisualCoding() const
{
	return m_VisualCoding_Exist;
}

Mp7JrsMediaFormatType_AudioCoding_LocalPtr Mp7JrsMediaFormatType::GetAudioCoding() const
{
	return m_AudioCoding;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidAudioCoding() const
{
	return m_AudioCoding_Exist;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetSceneCodingFormat() const
{
	return m_SceneCodingFormat;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidSceneCodingFormat() const
{
	return m_SceneCodingFormat_Exist;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetGraphicsCodingFormat() const
{
	return m_GraphicsCodingFormat;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidGraphicsCodingFormat() const
{
	return m_GraphicsCodingFormat_Exist;
}

Mp7JrsControlledTermUsePtr Mp7JrsMediaFormatType::GetOtherCodingFormat() const
{
	return m_OtherCodingFormat;
}

// Element is optional
bool Mp7JrsMediaFormatType::IsValidOtherCodingFormat() const
{
	return m_OtherCodingFormat_Exist;
}

void Mp7JrsMediaFormatType::SetContent(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetContent().");
	}
	if (m_Content != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Content);
		m_Content = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Content) m_Content->SetParent(m_myself.getPointer());
		if (m_Content && m_Content->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Content->UseTypeAttribute = true;
		}
		if(m_Content != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Content->SetContentName(XMLString::transcode("Content"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::SetMedium(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetMedium().");
	}
	if (m_Medium != item || m_Medium_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Medium);
		m_Medium = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Medium) m_Medium->SetParent(m_myself.getPointer());
		if (m_Medium && m_Medium->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Medium->UseTypeAttribute = true;
		}
		m_Medium_Exist = true;
		if(m_Medium != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Medium->SetContentName(XMLString::transcode("Medium"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateMedium()
{
	m_Medium_Exist = false;
}
void Mp7JrsMediaFormatType::SetFileFormat(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetFileFormat().");
	}
	if (m_FileFormat != item || m_FileFormat_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FileFormat);
		m_FileFormat = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FileFormat) m_FileFormat->SetParent(m_myself.getPointer());
		if (m_FileFormat && m_FileFormat->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FileFormat->UseTypeAttribute = true;
		}
		m_FileFormat_Exist = true;
		if(m_FileFormat != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FileFormat->SetContentName(XMLString::transcode("FileFormat"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateFileFormat()
{
	m_FileFormat_Exist = false;
}
void Mp7JrsMediaFormatType::SetFileSize(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetFileSize().");
	}
	if (m_FileSize != item || m_FileSize_Exist == false)
	{
		m_FileSize = item;
		m_FileSize_Exist = true;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateFileSize()
{
	m_FileSize_Exist = false;
}
void Mp7JrsMediaFormatType::SetSystem(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetSystem().");
	}
	if (m_System != item || m_System_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_System);
		m_System = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_System) m_System->SetParent(m_myself.getPointer());
		if (m_System && m_System->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_System->UseTypeAttribute = true;
		}
		m_System_Exist = true;
		if(m_System != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_System->SetContentName(XMLString::transcode("System"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateSystem()
{
	m_System_Exist = false;
}
void Mp7JrsMediaFormatType::SetBandwidth(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetBandwidth().");
	}
	if (m_Bandwidth != item || m_Bandwidth_Exist == false)
	{
		m_Bandwidth = item;
		m_Bandwidth_Exist = true;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateBandwidth()
{
	m_Bandwidth_Exist = false;
}
void Mp7JrsMediaFormatType::SetBitRate(const Mp7JrsMediaFormatType_BitRate_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetBitRate().");
	}
	if (m_BitRate != item || m_BitRate_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_BitRate);
		m_BitRate = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_BitRate) m_BitRate->SetParent(m_myself.getPointer());
		if (m_BitRate && m_BitRate->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_BitRate_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_BitRate->UseTypeAttribute = true;
		}
		m_BitRate_Exist = true;
		if(m_BitRate != Mp7JrsMediaFormatType_BitRate_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_BitRate->SetContentName(XMLString::transcode("BitRate"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateBitRate()
{
	m_BitRate_Exist = false;
}
void Mp7JrsMediaFormatType::SetTargetChannelBitRate(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetTargetChannelBitRate().");
	}
	if (m_TargetChannelBitRate != item || m_TargetChannelBitRate_Exist == false)
	{
		m_TargetChannelBitRate = item;
		m_TargetChannelBitRate_Exist = true;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateTargetChannelBitRate()
{
	m_TargetChannelBitRate_Exist = false;
}
void Mp7JrsMediaFormatType::SetScalableCoding(const Mp7JrsMediaFormatType_ScalableCoding_Local2Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetScalableCoding().");
	}
	if (m_ScalableCoding != item || m_ScalableCoding_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ScalableCoding);
		m_ScalableCoding = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ScalableCoding) m_ScalableCoding->SetParent(m_myself.getPointer());
		if (m_ScalableCoding && m_ScalableCoding->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_ScalableCoding_LocalType2"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ScalableCoding->UseTypeAttribute = true;
		}
		m_ScalableCoding_Exist = true;
		if(m_ScalableCoding != Mp7JrsMediaFormatType_ScalableCoding_Local2Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ScalableCoding->SetContentName(XMLString::transcode("ScalableCoding"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateScalableCoding()
{
	m_ScalableCoding_Exist = false;
}
void Mp7JrsMediaFormatType::SetVisualCoding(const Mp7JrsMediaFormatType_VisualCoding_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetVisualCoding().");
	}
	if (m_VisualCoding != item || m_VisualCoding_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_VisualCoding);
		m_VisualCoding = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VisualCoding) m_VisualCoding->SetParent(m_myself.getPointer());
		if (m_VisualCoding && m_VisualCoding->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_VisualCoding_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_VisualCoding->UseTypeAttribute = true;
		}
		m_VisualCoding_Exist = true;
		if(m_VisualCoding != Mp7JrsMediaFormatType_VisualCoding_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VisualCoding->SetContentName(XMLString::transcode("VisualCoding"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateVisualCoding()
{
	m_VisualCoding_Exist = false;
}
void Mp7JrsMediaFormatType::SetAudioCoding(const Mp7JrsMediaFormatType_AudioCoding_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetAudioCoding().");
	}
	if (m_AudioCoding != item || m_AudioCoding_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioCoding);
		m_AudioCoding = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioCoding) m_AudioCoding->SetParent(m_myself.getPointer());
		if (m_AudioCoding && m_AudioCoding->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_AudioCoding_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_AudioCoding->UseTypeAttribute = true;
		}
		m_AudioCoding_Exist = true;
		if(m_AudioCoding != Mp7JrsMediaFormatType_AudioCoding_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioCoding->SetContentName(XMLString::transcode("AudioCoding"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateAudioCoding()
{
	m_AudioCoding_Exist = false;
}
void Mp7JrsMediaFormatType::SetSceneCodingFormat(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetSceneCodingFormat().");
	}
	if (m_SceneCodingFormat != item || m_SceneCodingFormat_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SceneCodingFormat);
		m_SceneCodingFormat = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SceneCodingFormat) m_SceneCodingFormat->SetParent(m_myself.getPointer());
		if (m_SceneCodingFormat && m_SceneCodingFormat->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SceneCodingFormat->UseTypeAttribute = true;
		}
		m_SceneCodingFormat_Exist = true;
		if(m_SceneCodingFormat != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SceneCodingFormat->SetContentName(XMLString::transcode("SceneCodingFormat"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateSceneCodingFormat()
{
	m_SceneCodingFormat_Exist = false;
}
void Mp7JrsMediaFormatType::SetGraphicsCodingFormat(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetGraphicsCodingFormat().");
	}
	if (m_GraphicsCodingFormat != item || m_GraphicsCodingFormat_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_GraphicsCodingFormat);
		m_GraphicsCodingFormat = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GraphicsCodingFormat) m_GraphicsCodingFormat->SetParent(m_myself.getPointer());
		if (m_GraphicsCodingFormat && m_GraphicsCodingFormat->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_GraphicsCodingFormat->UseTypeAttribute = true;
		}
		m_GraphicsCodingFormat_Exist = true;
		if(m_GraphicsCodingFormat != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_GraphicsCodingFormat->SetContentName(XMLString::transcode("GraphicsCodingFormat"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateGraphicsCodingFormat()
{
	m_GraphicsCodingFormat_Exist = false;
}
void Mp7JrsMediaFormatType::SetOtherCodingFormat(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaFormatType::SetOtherCodingFormat().");
	}
	if (m_OtherCodingFormat != item || m_OtherCodingFormat_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_OtherCodingFormat);
		m_OtherCodingFormat = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_OtherCodingFormat) m_OtherCodingFormat->SetParent(m_myself.getPointer());
		if (m_OtherCodingFormat && m_OtherCodingFormat->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_OtherCodingFormat->UseTypeAttribute = true;
		}
		m_OtherCodingFormat_Exist = true;
		if(m_OtherCodingFormat != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_OtherCodingFormat->SetContentName(XMLString::transcode("OtherCodingFormat"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaFormatType::InvalidateOtherCodingFormat()
{
	m_OtherCodingFormat_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMediaFormatType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Content")) == 0)
	{
		// Content is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetContent()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetContent(p, client);
					if((p = GetContent()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Medium")) == 0)
	{
		// Medium is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMedium()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMedium(p, client);
					if((p = GetMedium()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FileFormat")) == 0)
	{
		// FileFormat is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFileFormat()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFileFormat(p, client);
					if((p = GetFileFormat()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("System")) == 0)
	{
		// System is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSystem()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSystem(p, client);
					if((p = GetSystem()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("BitRate")) == 0)
	{
		// BitRate is simple element MediaFormatType_BitRate_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetBitRate()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_BitRate_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaFormatType_BitRate_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetBitRate(p, client);
					if((p = GetBitRate()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ScalableCoding")) == 0)
	{
		// ScalableCoding is simple element MediaFormatType_ScalableCoding_LocalType2
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetScalableCoding()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_ScalableCoding_LocalType2")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaFormatType_ScalableCoding_Local2Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetScalableCoding(p, client);
					if((p = GetScalableCoding()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VisualCoding")) == 0)
	{
		// VisualCoding is simple element MediaFormatType_VisualCoding_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVisualCoding()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_VisualCoding_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaFormatType_VisualCoding_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVisualCoding(p, client);
					if((p = GetVisualCoding()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AudioCoding")) == 0)
	{
		// AudioCoding is simple element MediaFormatType_AudioCoding_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetAudioCoding()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaFormatType_AudioCoding_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaFormatType_AudioCoding_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetAudioCoding(p, client);
					if((p = GetAudioCoding()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SceneCodingFormat")) == 0)
	{
		// SceneCodingFormat is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSceneCodingFormat()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSceneCodingFormat(p, client);
					if((p = GetSceneCodingFormat()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("GraphicsCodingFormat")) == 0)
	{
		// GraphicsCodingFormat is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetGraphicsCodingFormat()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetGraphicsCodingFormat(p, client);
					if((p = GetGraphicsCodingFormat()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("OtherCodingFormat")) == 0)
	{
		// OtherCodingFormat is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetOtherCodingFormat()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetOtherCodingFormat(p, client);
					if((p = GetOtherCodingFormat()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaFormatType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaFormatType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMediaFormatType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMediaFormatType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMediaFormatType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaFormatType"));
	// Element serialization:
	// Class
	
	if (m_Content != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Content->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Content"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Content->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Medium != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Medium->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Medium"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Medium->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FileFormat != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FileFormat->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FileFormat"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FileFormat->Serialize(doc, element, &elem);
	}
	if(m_FileSize_Exist)
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_FileSize);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("FileSize"), true);
		XMLString::release(&tmp);
		
	}
	// Class
	
	if (m_System != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_System->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("System"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_System->Serialize(doc, element, &elem);
	}
	if(m_Bandwidth_Exist)
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_Bandwidth);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("Bandwidth"), true);
		XMLString::release(&tmp);
		
	}
	// Class with content		
	
	if (m_BitRate != Mp7JrsMediaFormatType_BitRate_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_BitRate->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("BitRate"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_BitRate->Serialize(doc, element, &elem);
	}
	if(m_TargetChannelBitRate_Exist)
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_TargetChannelBitRate);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("TargetChannelBitRate"), true);
		XMLString::release(&tmp);
		
	}
	// Class
	
	if (m_ScalableCoding != Mp7JrsMediaFormatType_ScalableCoding_Local2Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ScalableCoding->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ScalableCoding"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ScalableCoding->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_VisualCoding != Mp7JrsMediaFormatType_VisualCoding_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_VisualCoding->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("VisualCoding"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_VisualCoding->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_AudioCoding != Mp7JrsMediaFormatType_AudioCoding_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioCoding->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioCoding"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_AudioCoding->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SceneCodingFormat != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SceneCodingFormat->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SceneCodingFormat"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SceneCodingFormat->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_GraphicsCodingFormat != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_GraphicsCodingFormat->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("GraphicsCodingFormat"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_GraphicsCodingFormat->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_OtherCodingFormat != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_OtherCodingFormat->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("OtherCodingFormat"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_OtherCodingFormat->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMediaFormatType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Content"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Content")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetContent(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Medium"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Medium")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMedium(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FileFormat"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FileFormat")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFileFormat(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("FileSize"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetFileSize(Mp7JrsConvert::TextToUnsignedInt(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("System"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("System")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSystem(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("Bandwidth"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetBandwidth(Mp7JrsConvert::TextToFloat(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("BitRate"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("BitRate")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaFormatType_BitRate_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetBitRate(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("TargetChannelBitRate"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetTargetChannelBitRate(Mp7JrsConvert::TextToUnsignedInt(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ScalableCoding"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ScalableCoding")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaFormatType_ScalableCoding_LocalType2;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetScalableCoding(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("VisualCoding"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("VisualCoding")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaFormatType_VisualCoding_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetVisualCoding(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioCoding"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioCoding")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaFormatType_AudioCoding_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioCoding(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SceneCodingFormat"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SceneCodingFormat")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSceneCodingFormat(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("GraphicsCodingFormat"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("GraphicsCodingFormat")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetGraphicsCodingFormat(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("OtherCodingFormat"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("OtherCodingFormat")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetOtherCodingFormat(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaFormatType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Content")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetContent(child);
  }
  if (XMLString::compareString(elementname, X("Medium")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetMedium(child);
  }
  if (XMLString::compareString(elementname, X("FileFormat")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetFileFormat(child);
  }
  if (XMLString::compareString(elementname, X("System")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetSystem(child);
  }
  if (XMLString::compareString(elementname, X("BitRate")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaFormatType_BitRate_LocalType;
    }
    this->SetBitRate(child);
  }
  if (XMLString::compareString(elementname, X("ScalableCoding")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaFormatType_ScalableCoding_LocalType2;
    }
    this->SetScalableCoding(child);
  }
  if (XMLString::compareString(elementname, X("VisualCoding")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaFormatType_VisualCoding_LocalType;
    }
    this->SetVisualCoding(child);
  }
  if (XMLString::compareString(elementname, X("AudioCoding")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaFormatType_AudioCoding_LocalType;
    }
    this->SetAudioCoding(child);
  }
  if (XMLString::compareString(elementname, X("SceneCodingFormat")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetSceneCodingFormat(child);
  }
  if (XMLString::compareString(elementname, X("GraphicsCodingFormat")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetGraphicsCodingFormat(child);
  }
  if (XMLString::compareString(elementname, X("OtherCodingFormat")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetOtherCodingFormat(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaFormatType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetContent() != Mp7JrsNodePtr())
		result.Insert(GetContent());
	if (GetMedium() != Mp7JrsNodePtr())
		result.Insert(GetMedium());
	if (GetFileFormat() != Mp7JrsNodePtr())
		result.Insert(GetFileFormat());
	if (GetSystem() != Mp7JrsNodePtr())
		result.Insert(GetSystem());
	if (GetBitRate() != Mp7JrsNodePtr())
		result.Insert(GetBitRate());
	if (GetScalableCoding() != Mp7JrsNodePtr())
		result.Insert(GetScalableCoding());
	if (GetVisualCoding() != Mp7JrsNodePtr())
		result.Insert(GetVisualCoding());
	if (GetAudioCoding() != Mp7JrsNodePtr())
		result.Insert(GetAudioCoding());
	if (GetSceneCodingFormat() != Mp7JrsNodePtr())
		result.Insert(GetSceneCodingFormat());
	if (GetGraphicsCodingFormat() != Mp7JrsNodePtr())
		result.Insert(GetGraphicsCodingFormat());
	if (GetOtherCodingFormat() != Mp7JrsNodePtr())
		result.Insert(GetOtherCodingFormat());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMediaFormatType_ExtMethodImpl.h


