 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtImplInclude.h



#include "Mp7JrsVideoSegmentFeatureType.h"
#include "Mp7JrsShapeVariationType.h"
#include "Mp7JrsParametricMotionType.h"
#include "Mp7JrsMotionTrajectoryType.h"
#include "Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionType.h"
#include "Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionType.h"
#include "Mp7JrsMotionActivityType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsMovingRegionFeatureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsVisualTimeSeriesType.h" // Element collection urn:mpeg:mpeg7:schema:2004:TemporalTransition
#include "Mp7JrsGopGofFeature.h" // Element collection urn:mpeg:mpeg7:schema:2004:RepresentativeFeature

#include <assert.h>
IMp7JrsMovingRegionFeatureType::IMp7JrsMovingRegionFeatureType()
{

// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtPropInit.h

}

IMp7JrsMovingRegionFeatureType::~IMp7JrsMovingRegionFeatureType()
{
// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtPropCleanup.h

}

Mp7JrsMovingRegionFeatureType::Mp7JrsMovingRegionFeatureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMovingRegionFeatureType::~Mp7JrsMovingRegionFeatureType()
{
	Cleanup();
}

void Mp7JrsMovingRegionFeatureType::Init()
{
	// Init base
	m_Base = CreateVideoSegmentFeatureType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_ShapeVariation = Mp7JrsShapeVariationPtr(); // Class
	m_ShapeVariation_Exist = false;
	m_ParametricMotion = Mp7JrsParametricMotionPtr(); // Class
	m_MotionTrajectory = Mp7JrsMotionTrajectoryPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtMyPropInit.h

}

void Mp7JrsMovingRegionFeatureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_ShapeVariation);
	// Mp7JrsFactory::DeleteObject(m_ParametricMotion);
	// Mp7JrsFactory::DeleteObject(m_MotionTrajectory);
}

void Mp7JrsMovingRegionFeatureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MovingRegionFeatureTypePtr, since we
	// might need GetBase(), which isn't defined in IMovingRegionFeatureType
    const Mp7JrsPtr< Mp7JrsMovingRegionFeatureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVideoSegmentFeaturePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMovingRegionFeatureType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidShapeVariation())
	{
		// Mp7JrsFactory::DeleteObject(m_ShapeVariation);
		this->SetShapeVariation(Mp7JrsFactory::CloneObject(tmp->GetShapeVariation()));
	}
	else
	{
		InvalidateShapeVariation();
	}
		// Mp7JrsFactory::DeleteObject(m_ParametricMotion);
		this->SetParametricMotion(Mp7JrsFactory::CloneObject(tmp->GetParametricMotion()));
		// Mp7JrsFactory::DeleteObject(m_MotionTrajectory);
		this->SetMotionTrajectory(Mp7JrsFactory::CloneObject(tmp->GetMotionTrajectory()));
}

Mp7JrsNodePtr Mp7JrsMovingRegionFeatureType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMovingRegionFeatureType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVideoSegmentFeatureType > Mp7JrsMovingRegionFeatureType::GetBase() const
{
	return m_Base;
}

Mp7JrsShapeVariationPtr Mp7JrsMovingRegionFeatureType::GetShapeVariation() const
{
	return m_ShapeVariation;
}

// Element is optional
bool Mp7JrsMovingRegionFeatureType::IsValidShapeVariation() const
{
	return m_ShapeVariation_Exist;
}

Mp7JrsParametricMotionPtr Mp7JrsMovingRegionFeatureType::GetParametricMotion() const
{
	return m_ParametricMotion;
}

Mp7JrsMotionTrajectoryPtr Mp7JrsMovingRegionFeatureType::GetMotionTrajectory() const
{
	return m_MotionTrajectory;
}

void Mp7JrsMovingRegionFeatureType::SetShapeVariation(const Mp7JrsShapeVariationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetShapeVariation().");
	}
	if (m_ShapeVariation != item || m_ShapeVariation_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ShapeVariation);
		m_ShapeVariation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ShapeVariation) m_ShapeVariation->SetParent(m_myself.getPointer());
		if (m_ShapeVariation && m_ShapeVariation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ShapeVariation->UseTypeAttribute = true;
		}
		m_ShapeVariation_Exist = true;
		if(m_ShapeVariation != Mp7JrsShapeVariationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ShapeVariation->SetContentName(XMLString::transcode("ShapeVariation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::InvalidateShapeVariation()
{
	m_ShapeVariation_Exist = false;
}
	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMovingRegionFeatureType::SetParametricMotion(const Mp7JrsParametricMotionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetParametricMotion().");
	}
	if (m_ParametricMotion != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ParametricMotion);
		m_ParametricMotion = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ParametricMotion) m_ParametricMotion->SetParent(m_myself.getPointer());
		if (m_ParametricMotion && m_ParametricMotion->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ParametricMotionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ParametricMotion->UseTypeAttribute = true;
		}
		if(m_ParametricMotion != Mp7JrsParametricMotionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ParametricMotion->SetContentName(XMLString::transcode("ParametricMotion"));
		}
	// Mp7JrsFactory::DeleteObject(m_MotionTrajectory);
	m_MotionTrajectory = Mp7JrsMotionTrajectoryPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMovingRegionFeatureType::SetMotionTrajectory(const Mp7JrsMotionTrajectoryPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetMotionTrajectory().");
	}
	if (m_MotionTrajectory != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MotionTrajectory);
		m_MotionTrajectory = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MotionTrajectory) m_MotionTrajectory->SetParent(m_myself.getPointer());
		if (m_MotionTrajectory && m_MotionTrajectory->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionTrajectoryType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MotionTrajectory->UseTypeAttribute = true;
		}
		if(m_MotionTrajectory != Mp7JrsMotionTrajectoryPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MotionTrajectory->SetContentName(XMLString::transcode("MotionTrajectory"));
		}
	// Mp7JrsFactory::DeleteObject(m_ParametricMotion);
	m_ParametricMotion = Mp7JrsParametricMotionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr Mp7JrsMovingRegionFeatureType::GetTemporalTransition() const
{
	return GetBase()->GetTemporalTransition();
}

Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr Mp7JrsMovingRegionFeatureType::GetRepresentativeFeature() const
{
	return GetBase()->GetRepresentativeFeature();
}

Mp7JrsMotionActivityPtr Mp7JrsMovingRegionFeatureType::GetMotionActivity() const
{
	return GetBase()->GetMotionActivity();
}

// Element is optional
bool Mp7JrsMovingRegionFeatureType::IsValidMotionActivity() const
{
	return GetBase()->IsValidMotionActivity();
}

void Mp7JrsMovingRegionFeatureType::SetTemporalTransition(const Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetTemporalTransition().");
	}
	GetBase()->SetTemporalTransition(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::SetRepresentativeFeature(const Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetRepresentativeFeature().");
	}
	GetBase()->SetRepresentativeFeature(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::SetMotionActivity(const Mp7JrsMotionActivityPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetMotionActivity().");
	}
	GetBase()->SetMotionActivity(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::InvalidateMotionActivity()
{
	GetBase()->InvalidateMotionActivity();
}
XMLCh * Mp7JrsMovingRegionFeatureType::Getid() const
{
	return GetBase()->GetBase()->GetBase()->Getid();
}

bool Mp7JrsMovingRegionFeatureType::Existid() const
{
	return GetBase()->GetBase()->GetBase()->Existid();
}
void Mp7JrsMovingRegionFeatureType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::Setid().");
	}
	GetBase()->GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::Invalidateid()
{
	GetBase()->GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsMovingRegionFeatureType::GettimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsMovingRegionFeatureType::ExisttimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsMovingRegionFeatureType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SettimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::InvalidatetimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsMovingRegionFeatureType::GettimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsMovingRegionFeatureType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsMovingRegionFeatureType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SettimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsMovingRegionFeatureType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsMovingRegionFeatureType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsMovingRegionFeatureType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsMovingRegionFeatureType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsMovingRegionFeatureType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsMovingRegionFeatureType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMovingRegionFeatureType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsMovingRegionFeatureType::GetHeader() const
{
	return GetBase()->GetBase()->GetBase()->GetHeader();
}

void Mp7JrsMovingRegionFeatureType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMovingRegionFeatureType::SetHeader().");
	}
	GetBase()->GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMovingRegionFeatureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ShapeVariation")) == 0)
	{
		// ShapeVariation is simple element ShapeVariationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetShapeVariation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ShapeVariationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsShapeVariationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetShapeVariation(p, client);
					if((p = GetShapeVariation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ParametricMotion")) == 0)
	{
		// ParametricMotion is simple element ParametricMotionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetParametricMotion()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ParametricMotionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsParametricMotionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetParametricMotion(p, client);
					if((p = GetParametricMotion()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MotionTrajectory")) == 0)
	{
		// MotionTrajectory is simple element MotionTrajectoryType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMotionTrajectory()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionTrajectoryType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMotionTrajectoryPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMotionTrajectory(p, client);
					if((p = GetMotionTrajectory()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MovingRegionFeatureType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MovingRegionFeatureType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMovingRegionFeatureType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMovingRegionFeatureType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMovingRegionFeatureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MovingRegionFeatureType"));
	// Element serialization:
	// Class
	
	if (m_ShapeVariation != Mp7JrsShapeVariationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ShapeVariation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ShapeVariation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ShapeVariation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ParametricMotion != Mp7JrsParametricMotionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ParametricMotion->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ParametricMotion"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ParametricMotion->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MotionTrajectory != Mp7JrsMotionTrajectoryPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MotionTrajectory->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MotionTrajectory"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MotionTrajectory->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMovingRegionFeatureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVideoSegmentFeatureType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ShapeVariation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ShapeVariation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateShapeVariationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetShapeVariation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ParametricMotion"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ParametricMotion")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateParametricMotionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetParametricMotion(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MotionTrajectory"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MotionTrajectory")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMotionTrajectoryType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMotionTrajectory(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMovingRegionFeatureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("ShapeVariation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateShapeVariationType;
    }
    this->SetShapeVariation(child);
  }
  if (XMLString::compareString(elementname, X("ParametricMotion")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateParametricMotionType;
    }
    this->SetParametricMotion(child);
  }
  if (XMLString::compareString(elementname, X("MotionTrajectory")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMotionTrajectoryType;
    }
    this->SetMotionTrajectory(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMovingRegionFeatureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetShapeVariation() != Mp7JrsNodePtr())
		result.Insert(GetShapeVariation());
	if (GetTemporalTransition() != Mp7JrsNodePtr())
		result.Insert(GetTemporalTransition());
	if (GetRepresentativeFeature() != Mp7JrsNodePtr())
		result.Insert(GetRepresentativeFeature());
	if (GetMotionActivity() != Mp7JrsNodePtr())
		result.Insert(GetMotionActivity());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetParametricMotion() != Mp7JrsNodePtr())
		result.Insert(GetParametricMotion());
	if (GetMotionTrajectory() != Mp7JrsNodePtr())
		result.Insert(GetMotionTrajectory());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMovingRegionFeatureType_ExtMethodImpl.h


