 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtImplInclude.h



#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsMpeg7Type.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsMpeg7Type::IMp7JrsMpeg7Type()
{

// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtPropInit.h

}

IMp7JrsMpeg7Type::~IMp7JrsMpeg7Type()
{
// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtPropCleanup.h

}

Mp7JrsMpeg7Type::Mp7JrsMpeg7Type()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMpeg7Type::~Mp7JrsMpeg7Type()
{
	Cleanup();
}

void Mp7JrsMpeg7Type::Init()
{

	// Init attributes
	m_lang = NULL; // String
	m_lang_Exist = false;
	m_timeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_timeBase_Exist = false;
	m_timeUnit = Mp7JrsdurationPtr(); // Pattern
	m_timeUnit_Exist = false;
	m_mediaTimeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_mediaTimeBase_Exist = false;
	m_mediaTimeUnit = Mp7JrsmediaDurationPtr(); // Pattern
	m_mediaTimeUnit_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_DescriptionMetadata = Mp7JrsDescriptionMetadataPtr(); // Class
	m_DescriptionMetadata_Exist = false;


// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtMyPropInit.h

}

void Mp7JrsMpeg7Type::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtMyPropCleanup.h


	XMLString::release(&m_lang);
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
	// Mp7JrsFactory::DeleteObject(m_timeUnit); 
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); 
	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); 
	// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
}

void Mp7JrsMpeg7Type::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use Mpeg7TypePtr, since we
	// might need GetBase(), which isn't defined in IMpeg7Type
    const Mp7JrsPtr< Mp7JrsMpeg7Type > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	XMLString::release(&m_lang);
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
	if (tmp->ExisttimeBase())
	{
		this->SettimeBase(Mp7JrsFactory::CloneObject(tmp->GettimeBase()));
	}
	else
	{
		InvalidatetimeBase();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeUnit); 
	if (tmp->ExisttimeUnit())
	{
		this->SettimeUnit(Mp7JrsFactory::CloneObject(tmp->GettimeUnit()));
	}
	else
	{
		InvalidatetimeUnit();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); 
	if (tmp->ExistmediaTimeBase())
	{
		this->SetmediaTimeBase(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeBase()));
	}
	else
	{
		InvalidatemediaTimeBase();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); 
	if (tmp->ExistmediaTimeUnit())
	{
		this->SetmediaTimeUnit(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeUnit()));
	}
	else
	{
		InvalidatemediaTimeUnit();
	}
	}
	if (tmp->IsValidDescriptionMetadata())
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
		this->SetDescriptionMetadata(Mp7JrsFactory::CloneObject(tmp->GetDescriptionMetadata()));
	}
	else
	{
		InvalidateDescriptionMetadata();
	}
}

Mp7JrsNodePtr Mp7JrsMpeg7Type::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMpeg7Type::GetBaseRoot() const
{
	return m_myself.getPointer();
}


XMLCh * Mp7JrsMpeg7Type::Getlang() const
{
	return m_lang;
}

bool Mp7JrsMpeg7Type::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsMpeg7Type::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7Type::Setlang().");
	}
    m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7Type::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsMpeg7Type::GettimeBase() const
{
	return m_timeBase;
}

bool Mp7JrsMpeg7Type::ExisttimeBase() const
{
	return m_timeBase_Exist;
}
void Mp7JrsMpeg7Type::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7Type::SettimeBase().");
	}
    m_timeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeBase) m_timeBase->SetParent(m_myself.getPointer());
	m_timeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7Type::InvalidatetimeBase()
{
	m_timeBase_Exist = false;
}
Mp7JrsdurationPtr Mp7JrsMpeg7Type::GettimeUnit() const
{
	return m_timeUnit;
}

bool Mp7JrsMpeg7Type::ExisttimeUnit() const
{
	return m_timeUnit_Exist;
}
void Mp7JrsMpeg7Type::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7Type::SettimeUnit().");
	}
    m_timeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeUnit) m_timeUnit->SetParent(m_myself.getPointer());
	m_timeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7Type::InvalidatetimeUnit()
{
	m_timeUnit_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsMpeg7Type::GetmediaTimeBase() const
{
	return m_mediaTimeBase;
}

bool Mp7JrsMpeg7Type::ExistmediaTimeBase() const
{
	return m_mediaTimeBase_Exist;
}
void Mp7JrsMpeg7Type::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7Type::SetmediaTimeBase().");
	}
    m_mediaTimeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeBase) m_mediaTimeBase->SetParent(m_myself.getPointer());
	m_mediaTimeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7Type::InvalidatemediaTimeBase()
{
	m_mediaTimeBase_Exist = false;
}
Mp7JrsmediaDurationPtr Mp7JrsMpeg7Type::GetmediaTimeUnit() const
{
	return m_mediaTimeUnit;
}

bool Mp7JrsMpeg7Type::ExistmediaTimeUnit() const
{
	return m_mediaTimeUnit_Exist;
}
void Mp7JrsMpeg7Type::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7Type::SetmediaTimeUnit().");
	}
    m_mediaTimeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeUnit) m_mediaTimeUnit->SetParent(m_myself.getPointer());
	m_mediaTimeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7Type::InvalidatemediaTimeUnit()
{
	m_mediaTimeUnit_Exist = false;
}
Mp7JrsDescriptionMetadataPtr Mp7JrsMpeg7Type::GetDescriptionMetadata() const
{
	return m_DescriptionMetadata;
}

// Element is optional
bool Mp7JrsMpeg7Type::IsValidDescriptionMetadata() const
{
	return m_DescriptionMetadata_Exist;
}

void Mp7JrsMpeg7Type::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7Type::SetDescriptionMetadata().");
	}
	if (m_DescriptionMetadata != item || m_DescriptionMetadata_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
		m_DescriptionMetadata = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DescriptionMetadata) m_DescriptionMetadata->SetParent(m_myself.getPointer());
		if (m_DescriptionMetadata && m_DescriptionMetadata->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptionMetadataType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DescriptionMetadata->UseTypeAttribute = true;
		}
		m_DescriptionMetadata_Exist = true;
		if(m_DescriptionMetadata != Mp7JrsDescriptionMetadataPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DescriptionMetadata->SetContentName(XMLString::transcode("DescriptionMetadata"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7Type::InvalidateDescriptionMetadata()
{
	m_DescriptionMetadata_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMpeg7Type::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("DescriptionMetadata")) == 0)
	{
		// DescriptionMetadata is simple element DescriptionMetadataType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDescriptionMetadata()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptionMetadataType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDescriptionMetadataPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDescriptionMetadata(p, client);
					if((p = GetDescriptionMetadata()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for Mpeg7Type
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "Mpeg7Type");
	}		
	
	return result;
}

XMLCh * Mp7JrsMpeg7Type::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMpeg7Type::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsMpeg7Type::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("Mpeg7Type"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeBase_Exist)
	{
	// Pattern
	if (m_timeBase != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeBase->ToText();
		element->setAttributeNS(X(""), X("timeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeUnit_Exist)
	{
	// Pattern
	if (m_timeUnit != Mp7JrsdurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeUnit->ToText();
		element->setAttributeNS(X(""), X("timeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeBase_Exist)
	{
	// Pattern
	if (m_mediaTimeBase != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_mediaTimeBase->ToText();
		element->setAttributeNS(X(""), X("mediaTimeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeUnit_Exist)
	{
	// Pattern
	if (m_mediaTimeUnit != Mp7JrsmediaDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_mediaTimeUnit->ToText();
		element->setAttributeNS(X(""), X("mediaTimeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_DescriptionMetadata != Mp7JrsDescriptionMetadataPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DescriptionMetadata->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DescriptionMetadata"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DescriptionMetadata->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMpeg7Type::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttribute(X("lang"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeUnit")))
	{
        Mp7JrsdurationPtr tmp = CreatedurationType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeUnit")));
        this->SettimeUnit(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeUnit")))
	{
        Mp7JrsdurationPtr tmp = CreatedurationType;
        tmp->Parse(parent->getAttribute(X("timeUnit")));
        this->SettimeUnit(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("mediaTimeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("mediaTimeBase")));
        this->SetmediaTimeBase(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("mediaTimeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("mediaTimeBase")));
        this->SetmediaTimeBase(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("mediaTimeUnit")))
	{
        Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
        tmp->Parse(parent->getAttributeNS(X(""), X("mediaTimeUnit")));
        this->SetmediaTimeUnit(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("mediaTimeUnit")))
	{
        Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
        tmp->Parse(parent->getAttribute(X("mediaTimeUnit")));
        this->SetmediaTimeUnit(tmp);
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DescriptionMetadata"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DescriptionMetadata")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDescriptionMetadataType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDescriptionMetadata(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMpeg7Type::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("DescriptionMetadata")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDescriptionMetadataType;
    }
    this->SetDescriptionMetadata(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMpeg7Type::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMpeg7Type_ExtMethodImpl.h


