 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtImplInclude.h



#include "Mp7JrsMultipleViewType_CollectionType.h"
#include "Mp7JrsMultipleViewType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsMultipleViewType_LocalType.h" // Sequence collection Descriptor
#include "Mp7JrsVisualDType.h" // Sequence collection element Descriptor

#include <assert.h>
IMp7JrsMultipleViewType::IMp7JrsMultipleViewType()
{

// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtPropInit.h

}

IMp7JrsMultipleViewType::~IMp7JrsMultipleViewType()
{
// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtPropCleanup.h

}

Mp7JrsMultipleViewType::Mp7JrsMultipleViewType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMultipleViewType::~Mp7JrsMultipleViewType()
{
	Cleanup();
}

void Mp7JrsMultipleViewType::Init()
{

	// Init attributes
	m_fixedViewsFlag = false; // Value

	// Init elements (element, union sequence choice all any)
	
	m_MultipleViewType_LocalType = Mp7JrsMultipleViewType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtMyPropInit.h

}

void Mp7JrsMultipleViewType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_MultipleViewType_LocalType);
}

void Mp7JrsMultipleViewType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MultipleViewTypePtr, since we
	// might need GetBase(), which isn't defined in IMultipleViewType
    const Mp7JrsPtr< Mp7JrsMultipleViewType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
		this->SetfixedViewsFlag(tmp->GetfixedViewsFlag());
	}
		// Mp7JrsFactory::DeleteObject(m_MultipleViewType_LocalType);
		this->SetMultipleViewType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetMultipleViewType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsMultipleViewType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMultipleViewType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


bool Mp7JrsMultipleViewType::GetfixedViewsFlag() const
{
	return m_fixedViewsFlag;
}

void Mp7JrsMultipleViewType::SetfixedViewsFlag(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultipleViewType::SetfixedViewsFlag().");
	}
    m_fixedViewsFlag = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsMultipleViewType_CollectionPtr Mp7JrsMultipleViewType::GetMultipleViewType_LocalType() const
{
	return m_MultipleViewType_LocalType;
}

void Mp7JrsMultipleViewType::SetMultipleViewType_LocalType(const Mp7JrsMultipleViewType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultipleViewType::SetMultipleViewType_LocalType().");
	}
	if (m_MultipleViewType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MultipleViewType_LocalType);
		m_MultipleViewType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MultipleViewType_LocalType) m_MultipleViewType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMultipleViewType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Descriptor")) == 0)
	{
		// Descriptor is contained in itemtype MultipleViewType_LocalType
		// in sequence collection MultipleViewType_CollectionType

		context->Found = true;
		Mp7JrsMultipleViewType_CollectionPtr coll = GetMultipleViewType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMultipleViewType_CollectionType;
				SetMultipleViewType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 16))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(i))->GetDescriptor()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				// context->FreeIndexList.addElement(i); // FTT We don't allow to shift elems between different sequence items.
				++i;
			}
		}
		int start, stop;
		if(Mp7JrsXPathParseContext::PostProcessSequenceCollection(context, i, start, stop, result))
		{
			if(i >= 0)
			{
				Mp7JrsMultipleViewType_LocalPtr item = CreateMultipleViewType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			else // Just reorganise the existing nodes and then we can insert without creating another sequence item.
			{
				int j;
				if(start < stop)
				{
					for(j = start; j < stop; ++j)
					{
						((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(j))->SetDescriptor(
						((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(j+1))->GetDescriptor());
					}
				}
				else
				{
					for(j = start; j > stop; --j)
					{
						((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(j))->SetDescriptor(
						((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(j-1))->GetDescriptor());
					}
				}
				i = stop;
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsVisualDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(i))->SetDescriptor(p, client);
					 // Maybe some other restrictions are waiting...
					if((p = ((Mp7JrsMultipleViewType_LocalPtr)coll->elementAt(i))->GetDescriptor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MultipleViewType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MultipleViewType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMultipleViewType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMultipleViewType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsMultipleViewType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MultipleViewType"));
    // Attribute Serialization:
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_fixedViewsFlag);
		element->setAttributeNS(X(""), X("fixedViewsFlag"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	if (m_MultipleViewType_LocalType != Mp7JrsMultipleViewType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_MultipleViewType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMultipleViewType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("fixedViewsFlag")))
	{
        // deserialize value type
        this->SetfixedViewsFlag(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("fixedViewsFlag"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("fixedViewsFlag")))
	{
        // deserialize value type
        this->SetfixedViewsFlag(Mp7JrsConvert::TextToBool(parent->getAttribute(X("fixedViewsFlag"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Sequence Collection
        if((parent != NULL) && (
            (XMLString::compareString(parent->getNodeName()
            , X("urn:mpeg:mpeg7:schema:2004:IsVisible")) == 0)
                ||
            (XMLString::compareString(parent->getNodeName()
            , X("urn:mpeg:mpeg7:schema:2004:Descriptor")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsMultipleViewType_CollectionPtr tmp = CreateMultipleViewType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetMultipleViewType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMultipleViewType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
    // local type, so no need to check explicitly for a name (BAW 05 04 2004)
  {
    Mp7JrsMultipleViewType_CollectionPtr tmp = CreateMultipleViewType_CollectionType;
    this->SetMultipleViewType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMultipleViewType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMultipleViewType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetMultipleViewType_LocalType());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMultipleViewType_ExtMethodImpl.h


