 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPackageType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsPackageType_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsPackageType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsPackageType_LocalType.h" // Choice collection Scheme
#include "Mp7JrsPackageType_Scheme_LocalType.h" // Choice collection element Scheme
#include "Mp7JrsPackageType.h" // Choice collection element Package

#include <assert.h>
IMp7JrsPackageType::IMp7JrsPackageType()
{

// no includefile for extension defined 
// file Mp7JrsPackageType_ExtPropInit.h

}

IMp7JrsPackageType::~IMp7JrsPackageType()
{
// no includefile for extension defined 
// file Mp7JrsPackageType_ExtPropCleanup.h

}

Mp7JrsPackageType::Mp7JrsPackageType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPackageType::~Mp7JrsPackageType()
{
	Cleanup();
}

void Mp7JrsPackageType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_name = NULL; // String

	// Init elements (element, union sequence choice all any)
	
	m_PackageType_LocalType = Mp7JrsPackageType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsPackageType_ExtMyPropInit.h

}

void Mp7JrsPackageType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPackageType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_name);
	// Mp7JrsFactory::DeleteObject(m_PackageType_LocalType);
}

void Mp7JrsPackageType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PackageTypePtr, since we
	// might need GetBase(), which isn't defined in IPackageType
    const Mp7JrsPtr< Mp7JrsPackageType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsPackageType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_name);
		this->Setname(XMLString::replicate(tmp->Getname()));
	}
		// Mp7JrsFactory::DeleteObject(m_PackageType_LocalType);
		this->SetPackageType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetPackageType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsPackageType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsPackageType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsPackageType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsPackageType::Getname() const
{
	return m_name;
}

void Mp7JrsPackageType::Setname(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::Setname().");
	}
    m_name = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsPackageType_CollectionPtr Mp7JrsPackageType::GetPackageType_LocalType() const
{
	return m_PackageType_LocalType;
}

void Mp7JrsPackageType::SetPackageType_LocalType(const Mp7JrsPackageType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::SetPackageType_LocalType().");
	}
	if (m_PackageType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_PackageType_LocalType);
		m_PackageType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PackageType_LocalType) m_PackageType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsPackageType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsPackageType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsPackageType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPackageType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsPackageType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsPackageType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsPackageType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPackageType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsPackageType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsPackageType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsPackageType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPackageType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsPackageType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsPackageType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsPackageType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPackageType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsPackageType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsPackageType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsPackageType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPackageType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsPackageType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsPackageType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPackageType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsPackageType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Scheme")) == 0)
	{
		// Scheme is contained in itemtype PackageType_LocalType
		// in choice collection PackageType_CollectionType
		
		context->Found = true;
		Mp7JrsPackageType_CollectionPtr coll = GetPackageType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePackageType_CollectionType;
				SetPackageType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsPackageType_LocalPtr)coll->elementAt(i))->GetScheme()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsPackageType_LocalPtr item = CreatePackageType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PackageType_Scheme_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPackageType_Scheme_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsPackageType_LocalPtr)coll->elementAt(i))->SetScheme(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsPackageType_LocalPtr)coll->elementAt(i))->GetScheme()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Package")) == 0)
	{
		// Package is contained in itemtype PackageType_LocalType
		// in choice collection PackageType_CollectionType
		
		context->Found = true;
		Mp7JrsPackageType_CollectionPtr coll = GetPackageType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePackageType_CollectionType;
				SetPackageType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsPackageType_LocalPtr)coll->elementAt(i))->GetPackage()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsPackageType_LocalPtr item = CreatePackageType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PackageType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPackagePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsPackageType_LocalPtr)coll->elementAt(i))->SetPackage(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsPackageType_LocalPtr)coll->elementAt(i))->GetPackage()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PackageType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PackageType");
	}		
	
	return result;
}

XMLCh * Mp7JrsPackageType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPackageType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPackageType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsPackageType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PackageType"));
    // Attribute Serialization:
	// String
	if(m_name != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("name"), m_name);
	}
	// Element serialization:
	if (m_PackageType_LocalType != Mp7JrsPackageType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_PackageType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsPackageType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("name")))
	{
        // Deserialize string type
        this->Setname(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("name"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("name")))
	{
        // Deserialize string type
        this->Setname(Mp7JrsConvert::TextToString(parent->getAttribute(X("name"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:Scheme
            Mp7JrsUtil::HasNodeName(parent, X("Scheme"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Scheme")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:Package
            Mp7JrsUtil::HasNodeName(parent, X("Package"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Package")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsPackageType_CollectionPtr tmp = CreatePackageType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetPackageType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsPackageType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPackageType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Scheme"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Scheme")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Package"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Package")) == 0)
    )
  {
    Mp7JrsPackageType_CollectionPtr tmp = CreatePackageType_CollectionType;
    this->SetPackageType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPackageType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetPackageType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetPackageType_LocalType());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPackageType_ExtMethodImpl.h


