 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtImplInclude.h



#include "Mp7JrsPersonType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsPersonGroupType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsPersonGroupType_LocalType::IMp7JrsPersonGroupType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtPropInit.h

}

IMp7JrsPersonGroupType_LocalType::~IMp7JrsPersonGroupType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtPropCleanup.h

}

Mp7JrsPersonGroupType_LocalType::Mp7JrsPersonGroupType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPersonGroupType_LocalType::~Mp7JrsPersonGroupType_LocalType()
{
	Cleanup();
}

void Mp7JrsPersonGroupType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Member = Mp7JrsPersonPtr(); // Class
	m_MemberRef = Mp7JrsReferencePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtMyPropInit.h

}

void Mp7JrsPersonGroupType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Member);
	// Mp7JrsFactory::DeleteObject(m_MemberRef);
}

void Mp7JrsPersonGroupType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PersonGroupType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IPersonGroupType_LocalType
    const Mp7JrsPtr< Mp7JrsPersonGroupType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Member);
		this->SetMember(Mp7JrsFactory::CloneObject(tmp->GetMember()));
		// Mp7JrsFactory::DeleteObject(m_MemberRef);
		this->SetMemberRef(Mp7JrsFactory::CloneObject(tmp->GetMemberRef()));
}

Mp7JrsNodePtr Mp7JrsPersonGroupType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsPersonGroupType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsPersonPtr Mp7JrsPersonGroupType_LocalType::GetMember() const
{
	return m_Member;
}

Mp7JrsReferencePtr Mp7JrsPersonGroupType_LocalType::GetMemberRef() const
{
	return m_MemberRef;
}

// implementing setter for choice 
/* element */
void Mp7JrsPersonGroupType_LocalType::SetMember(const Mp7JrsPersonPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonGroupType_LocalType::SetMember().");
	}
	if (m_Member != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Member);
		m_Member = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Member) m_Member->SetParent(m_myself.getPointer());
		if (m_Member && m_Member->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PersonType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Member->UseTypeAttribute = true;
		}
		if(m_Member != Mp7JrsPersonPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Member->SetContentName(XMLString::transcode("Member"));
		}
	// Mp7JrsFactory::DeleteObject(m_MemberRef);
	m_MemberRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsPersonGroupType_LocalType::SetMemberRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonGroupType_LocalType::SetMemberRef().");
	}
	if (m_MemberRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MemberRef);
		m_MemberRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MemberRef) m_MemberRef->SetParent(m_myself.getPointer());
		if (m_MemberRef && m_MemberRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MemberRef->UseTypeAttribute = true;
		}
		if(m_MemberRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MemberRef->SetContentName(XMLString::transcode("MemberRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_Member);
	m_Member = Mp7JrsPersonPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: PersonGroupType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsPersonGroupType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsPersonGroupType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPersonGroupType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPersonGroupType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_Member != Mp7JrsPersonPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Member->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Member"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Member->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MemberRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MemberRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MemberRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MemberRef->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsPersonGroupType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Member"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Member")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePersonType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMember(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MemberRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MemberRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMemberRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPersonGroupType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Member")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePersonType;
    }
    this->SetMember(child);
  }
  if (XMLString::compareString(elementname, X("MemberRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetMemberRef(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPersonGroupType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMember() != Mp7JrsNodePtr())
		result.Insert(GetMember());
	if (GetMemberRef() != Mp7JrsNodePtr())
		result.Insert(GetMemberRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPersonGroupType_LocalType_ExtMethodImpl.h


