 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtImplInclude.h



#include "Mp7JrstimePointType.h"
#include "Mp7JrsPersonNameType_CollectionType.h"
#include "Mp7JrsPersonNameType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsPersonNameType_LocalType.h" // Choice collection GivenName
#include "Mp7JrsNameComponentType.h" // Choice collection element GivenName

#include <assert.h>
IMp7JrsPersonNameType::IMp7JrsPersonNameType()
{

// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtPropInit.h

}

IMp7JrsPersonNameType::~IMp7JrsPersonNameType()
{
// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtPropCleanup.h

}

Mp7JrsPersonNameType::Mp7JrsPersonNameType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPersonNameType::~Mp7JrsPersonNameType()
{
	Cleanup();
}

void Mp7JrsPersonNameType::Init()
{

	// Init attributes
	m_dateFrom = Mp7JrstimePointPtr(); // Pattern
	m_dateFrom_Exist = false;
	m_dateTo = Mp7JrstimePointPtr(); // Pattern
	m_dateTo_Exist = false;
	m_type = Mp7JrsPersonNameType_type_LocalType::UninitializedEnumeration;
	m_type_Exist = false;
	m_lang = NULL; // String
	m_lang_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_PersonNameType_LocalType = Mp7JrsPersonNameType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtMyPropInit.h

}

void Mp7JrsPersonNameType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_dateFrom); 
	// Mp7JrsFactory::DeleteObject(m_dateTo); 
	XMLString::release(&m_lang);
	// Mp7JrsFactory::DeleteObject(m_PersonNameType_LocalType);
}

void Mp7JrsPersonNameType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PersonNameTypePtr, since we
	// might need GetBase(), which isn't defined in IPersonNameType
    const Mp7JrsPtr< Mp7JrsPersonNameType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_dateFrom); 
	if (tmp->ExistdateFrom())
	{
		this->SetdateFrom(Mp7JrsFactory::CloneObject(tmp->GetdateFrom()));
	}
	else
	{
		InvalidatedateFrom();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_dateTo); 
	if (tmp->ExistdateTo())
	{
		this->SetdateTo(Mp7JrsFactory::CloneObject(tmp->GetdateTo()));
	}
	else
	{
		InvalidatedateTo();
	}
	}
	{
	if (tmp->Existtype())
	{
		this->Settype(tmp->Gettype());
	}
	else
	{
		Invalidatetype();
	}
	}
	{
	XMLString::release(&m_lang);
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_PersonNameType_LocalType);
		this->SetPersonNameType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetPersonNameType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsPersonNameType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsPersonNameType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrstimePointPtr Mp7JrsPersonNameType::GetdateFrom() const
{
	return m_dateFrom;
}

bool Mp7JrsPersonNameType::ExistdateFrom() const
{
	return m_dateFrom_Exist;
}
void Mp7JrsPersonNameType::SetdateFrom(const Mp7JrstimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType::SetdateFrom().");
	}
    m_dateFrom = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_dateFrom) m_dateFrom->SetParent(m_myself.getPointer());
	m_dateFrom_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType::InvalidatedateFrom()
{
	m_dateFrom_Exist = false;
}
Mp7JrstimePointPtr Mp7JrsPersonNameType::GetdateTo() const
{
	return m_dateTo;
}

bool Mp7JrsPersonNameType::ExistdateTo() const
{
	return m_dateTo_Exist;
}
void Mp7JrsPersonNameType::SetdateTo(const Mp7JrstimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType::SetdateTo().");
	}
    m_dateTo = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_dateTo) m_dateTo->SetParent(m_myself.getPointer());
	m_dateTo_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType::InvalidatedateTo()
{
	m_dateTo_Exist = false;
}
Mp7JrsPersonNameType_type_LocalType::Enumeration Mp7JrsPersonNameType::Gettype() const
{
	return m_type;
}

bool Mp7JrsPersonNameType::Existtype() const
{
	return m_type_Exist;
}
void Mp7JrsPersonNameType::Settype(Mp7JrsPersonNameType_type_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType::Settype().");
	}
    m_type = item;
	m_type_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType::Invalidatetype()
{
	m_type_Exist = false;
}
XMLCh * Mp7JrsPersonNameType::Getlang() const
{
	return m_lang;
}

bool Mp7JrsPersonNameType::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsPersonNameType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType::Setlang().");
	}
    m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsPersonNameType_CollectionPtr Mp7JrsPersonNameType::GetPersonNameType_LocalType() const
{
	return m_PersonNameType_LocalType;
}

void Mp7JrsPersonNameType::SetPersonNameType_LocalType(const Mp7JrsPersonNameType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType::SetPersonNameType_LocalType().");
	}
	if (m_PersonNameType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_PersonNameType_LocalType);
		m_PersonNameType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PersonNameType_LocalType) m_PersonNameType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsPersonNameType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("GivenName")) == 0)
	{
		// GivenName is contained in itemtype PersonNameType_LocalType
		// in choice collection PersonNameType_CollectionType
		
		context->Found = true;
		Mp7JrsPersonNameType_CollectionPtr coll = GetPersonNameType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePersonNameType_CollectionType;
				SetPersonNameType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->GetGivenName()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsPersonNameType_LocalPtr item = CreatePersonNameType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsNameComponentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->SetGivenName(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->GetGivenName()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FamilyName")) == 0)
	{
		// FamilyName is contained in itemtype PersonNameType_LocalType
		// in choice collection PersonNameType_CollectionType
		
		context->Found = true;
		Mp7JrsPersonNameType_CollectionPtr coll = GetPersonNameType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePersonNameType_CollectionType;
				SetPersonNameType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->GetFamilyName()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsPersonNameType_LocalPtr item = CreatePersonNameType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsNameComponentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->SetFamilyName(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->GetFamilyName()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Title")) == 0)
	{
		// Title is contained in itemtype PersonNameType_LocalType
		// in choice collection PersonNameType_CollectionType
		
		context->Found = true;
		Mp7JrsPersonNameType_CollectionPtr coll = GetPersonNameType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePersonNameType_CollectionType;
				SetPersonNameType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->GetTitle()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsPersonNameType_LocalPtr item = CreatePersonNameType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsNameComponentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->SetTitle(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsPersonNameType_LocalPtr)coll->elementAt(i))->GetTitle()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PersonNameType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PersonNameType");
	}		
	
	return result;
}

XMLCh * Mp7JrsPersonNameType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPersonNameType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPersonNameType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PersonNameType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_dateFrom_Exist)
	{
	// Pattern
	if (m_dateFrom != Mp7JrstimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_dateFrom->ToText();
		element->setAttributeNS(X(""), X("dateFrom"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_dateTo_Exist)
	{
	// Pattern
	if (m_dateTo != Mp7JrstimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_dateTo->ToText();
		element->setAttributeNS(X(""), X("dateTo"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_type_Exist)
	{
	// Enumeration
	if(m_type != Mp7JrsPersonNameType_type_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsPersonNameType_type_LocalType::ToText(m_type);
		element->setAttributeNS(X(""), X("type"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_PersonNameType_LocalType != Mp7JrsPersonNameType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_PersonNameType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsPersonNameType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("dateFrom")))
	{
        Mp7JrstimePointPtr tmp = CreatetimePointType;
        tmp->Parse(parent->getAttributeNS(X(""), X("dateFrom")));
        this->SetdateFrom(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("dateFrom")))
	{
        Mp7JrstimePointPtr tmp = CreatetimePointType;
        tmp->Parse(parent->getAttribute(X("dateFrom")));
        this->SetdateFrom(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("dateTo")))
	{
        Mp7JrstimePointPtr tmp = CreatetimePointType;
        tmp->Parse(parent->getAttributeNS(X(""), X("dateTo")));
        this->SetdateTo(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("dateTo")))
	{
        Mp7JrstimePointPtr tmp = CreatetimePointType;
        tmp->Parse(parent->getAttribute(X("dateTo")));
        this->SetdateTo(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("type")))
	{
        this->Settype(Mp7JrsPersonNameType_type_LocalType::Parse(parent->getAttributeNS(X(""), X("type"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("type")))
	{
        this->Settype(Mp7JrsPersonNameType_type_LocalType::Parse(parent->getAttribute(X("type"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttribute(X("lang"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:GivenName
            Mp7JrsUtil::HasNodeName(parent, X("GivenName"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:GivenName")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:FamilyName
            Mp7JrsUtil::HasNodeName(parent, X("FamilyName"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:FamilyName")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:Title
            Mp7JrsUtil::HasNodeName(parent, X("Title"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Title")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:Numeration
            Mp7JrsUtil::HasNodeName(parent, X("Numeration"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Numeration")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsPersonNameType_CollectionPtr tmp = CreatePersonNameType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetPersonNameType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPersonNameType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("GivenName"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("GivenName")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("FamilyName"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("FamilyName")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Title"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Title")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Numeration"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Numeration")) == 0)
    )
  {
    Mp7JrsPersonNameType_CollectionPtr tmp = CreatePersonNameType_CollectionType;
    this->SetPersonNameType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPersonNameType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetPersonNameType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetPersonNameType_LocalType());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPersonNameType_ExtMethodImpl.h


