 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtImplInclude.h



#include "Mp7JrsClassificationModelType.h"
#include "Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsProbabilityClassificationModelType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsProbabilityModelClassType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ProbabilityModelClass

#include <assert.h>
IMp7JrsProbabilityClassificationModelType::IMp7JrsProbabilityClassificationModelType()
{

// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtPropInit.h

}

IMp7JrsProbabilityClassificationModelType::~IMp7JrsProbabilityClassificationModelType()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtPropCleanup.h

}

Mp7JrsProbabilityClassificationModelType::Mp7JrsProbabilityClassificationModelType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsProbabilityClassificationModelType::~Mp7JrsProbabilityClassificationModelType()
{
	Cleanup();
}

void Mp7JrsProbabilityClassificationModelType::Init()
{
	// Init base
	m_Base = CreateClassificationModelType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_ProbabilityModelClass = Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtMyPropInit.h

}

void Mp7JrsProbabilityClassificationModelType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_ProbabilityModelClass);
}

void Mp7JrsProbabilityClassificationModelType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ProbabilityClassificationModelTypePtr, since we
	// might need GetBase(), which isn't defined in IProbabilityClassificationModelType
    const Mp7JrsPtr< Mp7JrsProbabilityClassificationModelType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsClassificationModelPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsProbabilityClassificationModelType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_ProbabilityModelClass);
		this->SetProbabilityModelClass(Mp7JrsFactory::CloneObject(tmp->GetProbabilityModelClass()));
}

Mp7JrsNodePtr Mp7JrsProbabilityClassificationModelType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsProbabilityClassificationModelType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsClassificationModelType > Mp7JrsProbabilityClassificationModelType::GetBase() const
{
	return m_Base;
}

Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr Mp7JrsProbabilityClassificationModelType::GetProbabilityModelClass() const
{
	return m_ProbabilityModelClass;
}

void Mp7JrsProbabilityClassificationModelType::SetProbabilityModelClass(const Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::SetProbabilityModelClass().");
	}
	if (m_ProbabilityModelClass != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ProbabilityModelClass);
		m_ProbabilityModelClass = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ProbabilityModelClass) m_ProbabilityModelClass->SetParent(m_myself.getPointer());
		if(m_ProbabilityModelClass != Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ProbabilityModelClass->SetContentName(XMLString::transcode("ProbabilityModelClass"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsProbabilityClassificationModelType::Getcomplete() const
{
	return GetBase()->Getcomplete();
}

bool Mp7JrsProbabilityClassificationModelType::Existcomplete() const
{
	return GetBase()->Existcomplete();
}
void Mp7JrsProbabilityClassificationModelType::Setcomplete(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::Setcomplete().");
	}
	GetBase()->Setcomplete(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::Invalidatecomplete()
{
	GetBase()->Invalidatecomplete();
}
bool Mp7JrsProbabilityClassificationModelType::Getredundant() const
{
	return GetBase()->Getredundant();
}

bool Mp7JrsProbabilityClassificationModelType::Existredundant() const
{
	return GetBase()->Existredundant();
}
void Mp7JrsProbabilityClassificationModelType::Setredundant(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::Setredundant().");
	}
	GetBase()->Setredundant(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::Invalidateredundant()
{
	GetBase()->Invalidateredundant();
}
Mp7JrszeroToOnePtr Mp7JrsProbabilityClassificationModelType::Getconfidence() const
{
	return GetBase()->GetBase()->Getconfidence();
}

bool Mp7JrsProbabilityClassificationModelType::Existconfidence() const
{
	return GetBase()->GetBase()->Existconfidence();
}
void Mp7JrsProbabilityClassificationModelType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::Setconfidence().");
	}
	GetBase()->GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::Invalidateconfidence()
{
	GetBase()->GetBase()->Invalidateconfidence();
}
Mp7JrszeroToOnePtr Mp7JrsProbabilityClassificationModelType::Getreliability() const
{
	return GetBase()->GetBase()->Getreliability();
}

bool Mp7JrsProbabilityClassificationModelType::Existreliability() const
{
	return GetBase()->GetBase()->Existreliability();
}
void Mp7JrsProbabilityClassificationModelType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::Setreliability().");
	}
	GetBase()->GetBase()->Setreliability(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::Invalidatereliability()
{
	GetBase()->GetBase()->Invalidatereliability();
}
XMLCh * Mp7JrsProbabilityClassificationModelType::Getid() const
{
	return GetBase()->GetBase()->GetBase()->Getid();
}

bool Mp7JrsProbabilityClassificationModelType::Existid() const
{
	return GetBase()->GetBase()->GetBase()->Existid();
}
void Mp7JrsProbabilityClassificationModelType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::Setid().");
	}
	GetBase()->GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::Invalidateid()
{
	GetBase()->GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsProbabilityClassificationModelType::GettimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsProbabilityClassificationModelType::ExisttimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsProbabilityClassificationModelType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::SettimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::InvalidatetimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsProbabilityClassificationModelType::GettimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsProbabilityClassificationModelType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsProbabilityClassificationModelType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::SettimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsProbabilityClassificationModelType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsProbabilityClassificationModelType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsProbabilityClassificationModelType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsProbabilityClassificationModelType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsProbabilityClassificationModelType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsProbabilityClassificationModelType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityClassificationModelType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsProbabilityClassificationModelType::GetHeader() const
{
	return GetBase()->GetBase()->GetBase()->GetHeader();
}

void Mp7JrsProbabilityClassificationModelType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityClassificationModelType::SetHeader().");
	}
	GetBase()->GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsProbabilityClassificationModelType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ProbabilityModelClass")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ProbabilityModelClass is item of type ProbabilityModelClassType
		// in element collection ProbabilityClassificationModelType_ProbabilityModelClass_CollectionType
		
		context->Found = true;
		Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr coll = GetProbabilityModelClass();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateProbabilityClassificationModelType_ProbabilityModelClass_CollectionType;
				SetProbabilityModelClass(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ProbabilityModelClassType")))) != empty)
			{
				// Is type allowed
				Mp7JrsProbabilityModelClassPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ProbabilityClassificationModelType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ProbabilityClassificationModelType");
	}		
	
	return result;
}

XMLCh * Mp7JrsProbabilityClassificationModelType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsProbabilityClassificationModelType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsProbabilityClassificationModelType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ProbabilityClassificationModelType"));
	// Element serialization:
	if (m_ProbabilityModelClass != Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ProbabilityModelClass->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsProbabilityClassificationModelType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsClassificationModelType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("ProbabilityModelClass"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("ProbabilityModelClass")) == 0))
        {
            // Deserialize factory type
            Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr tmp = CreateProbabilityClassificationModelType_ProbabilityModelClass_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetProbabilityModelClass(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsProbabilityClassificationModelType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ProbabilityModelClass")) == 0))
  {
    Mp7JrsProbabilityClassificationModelType_ProbabilityModelClass_CollectionPtr tmp = CreateProbabilityClassificationModelType_ProbabilityModelClass_CollectionType;
    this->SetProbabilityModelClass(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsProbabilityClassificationModelType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetProbabilityModelClass() != Mp7JrsNodePtr())
		result.Insert(GetProbabilityModelClass());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsProbabilityClassificationModelType_ExtMethodImpl.h


