 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtImplInclude.h



#include "Mp7JrsDescriptorModelType.h"
#include "Mp7JrsProbabilityModelType.h"
#include "Mp7JrsProbabilityModelClassType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsProbabilityModelClassType_LocalType::IMp7JrsProbabilityModelClassType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtPropInit.h

}

IMp7JrsProbabilityModelClassType_LocalType::~IMp7JrsProbabilityModelClassType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtPropCleanup.h

}

Mp7JrsProbabilityModelClassType_LocalType::Mp7JrsProbabilityModelClassType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsProbabilityModelClassType_LocalType::~Mp7JrsProbabilityModelClassType_LocalType()
{
	Cleanup();
}

void Mp7JrsProbabilityModelClassType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_DescriptorModel = Mp7JrsDescriptorModelPtr(); // Class
	m_ProbabilityModel = Mp7JrsPtr< Mp7JrsNode >(); // Class


// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtMyPropInit.h

}

void Mp7JrsProbabilityModelClassType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_DescriptorModel);
	// Mp7JrsFactory::DeleteObject(m_ProbabilityModel);
}

void Mp7JrsProbabilityModelClassType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ProbabilityModelClassType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IProbabilityModelClassType_LocalType
    const Mp7JrsPtr< Mp7JrsProbabilityModelClassType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_DescriptorModel);
		this->SetDescriptorModel(Mp7JrsFactory::CloneObject(tmp->GetDescriptorModel()));
		// Mp7JrsFactory::DeleteObject(m_ProbabilityModel);
		this->SetProbabilityModel(Mp7JrsFactory::CloneObject(tmp->GetProbabilityModel()));
}

Mp7JrsNodePtr Mp7JrsProbabilityModelClassType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsProbabilityModelClassType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsDescriptorModelPtr Mp7JrsProbabilityModelClassType_LocalType::GetDescriptorModel() const
{
	return m_DescriptorModel;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsProbabilityModelClassType_LocalType::GetProbabilityModel() const
{
	return m_ProbabilityModel;
}

void Mp7JrsProbabilityModelClassType_LocalType::SetDescriptorModel(const Mp7JrsDescriptorModelPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType_LocalType::SetDescriptorModel().");
	}
	if (m_DescriptorModel != item)
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptorModel);
		m_DescriptorModel = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DescriptorModel) m_DescriptorModel->SetParent(m_myself.getPointer());
		if (m_DescriptorModel && m_DescriptorModel->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptorModelType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DescriptorModel->UseTypeAttribute = true;
		}
		if(m_DescriptorModel != Mp7JrsDescriptorModelPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DescriptorModel->SetContentName(XMLString::transcode("DescriptorModel"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsProbabilityModelClassType_LocalType::SetProbabilityModel(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsProbabilityModelClassType_LocalType::SetProbabilityModel().");
	}
	if (m_ProbabilityModel != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ProbabilityModel);
		m_ProbabilityModel = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ProbabilityModel) m_ProbabilityModel->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_ProbabilityModel) m_ProbabilityModel->UseTypeAttribute = true;
		if(m_ProbabilityModel != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ProbabilityModel->SetContentName(XMLString::transcode("ProbabilityModel"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: ProbabilityModelClassType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsProbabilityModelClassType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsProbabilityModelClassType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsProbabilityModelClassType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsProbabilityModelClassType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ProbabilityModelClassType_LocalType"));
	// Element serialization:
	// Class
	
	if (m_DescriptorModel != Mp7JrsDescriptorModelPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DescriptorModel->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DescriptorModel"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DescriptorModel->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ProbabilityModel != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ProbabilityModel->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ProbabilityModel"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_ProbabilityModel->UseTypeAttribute = true;
		}
		m_ProbabilityModel->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsProbabilityModelClassType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DescriptorModel"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DescriptorModel")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateDescriptorModelType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDescriptorModel(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ProbabilityModel"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ProbabilityModel")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateProbabilityModelType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetProbabilityModel(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsProbabilityModelClassType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("DescriptorModel")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateDescriptorModelType;
    }
    this->SetDescriptorModel(child);
  }
  if (XMLString::compareString(elementname, X("ProbabilityModel")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateProbabilityModelType;
    }
    this->SetProbabilityModel(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsProbabilityModelClassType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptorModel() != Mp7JrsNodePtr())
		result.Insert(GetDescriptorModel());
	if (GetProbabilityModel() != Mp7JrsNodePtr())
		result.Insert(GetProbabilityModel());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsProbabilityModelClassType_LocalType_ExtMethodImpl.h


