 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtImplInclude.h



#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsReferenceType::IMp7JrsReferenceType()
{

// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtPropInit.h

}

IMp7JrsReferenceType::~IMp7JrsReferenceType()
{
// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtPropCleanup.h

}

Mp7JrsReferenceType::Mp7JrsReferenceType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsReferenceType::~Mp7JrsReferenceType()
{
	Cleanup();
}

void Mp7JrsReferenceType::Init()
{

	// Init attributes
	m_idref = NULL; // String
	m_idref_Exist = false;
	m_xpath = Mp7JrsxPathRefPtr(); // Pattern
	m_xpath_Exist = false;
	m_href = NULL; // String
	m_href_Exist = false;



// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtMyPropInit.h

}

void Mp7JrsReferenceType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtMyPropCleanup.h


	XMLString::release(&m_idref);
	// Mp7JrsFactory::DeleteObject(m_xpath); 
	XMLString::release(&m_href);
}

void Mp7JrsReferenceType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ReferenceTypePtr, since we
	// might need GetBase(), which isn't defined in IReferenceType
    const Mp7JrsPtr< Mp7JrsReferenceType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	XMLString::release(&m_idref);
	if (tmp->Existidref())
	{
		this->Setidref(XMLString::replicate(tmp->Getidref()));
	}
	else
	{
		Invalidateidref();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_xpath); 
	if (tmp->Existxpath())
	{
		this->Setxpath(Mp7JrsFactory::CloneObject(tmp->Getxpath()));
	}
	else
	{
		Invalidatexpath();
	}
	}
	{
	XMLString::release(&m_href);
	if (tmp->Existhref())
	{
		this->Sethref(XMLString::replicate(tmp->Gethref()));
	}
	else
	{
		Invalidatehref();
	}
	}
}

Mp7JrsNodePtr Mp7JrsReferenceType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsReferenceType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


XMLCh * Mp7JrsReferenceType::Getidref() const
{
	return m_idref;
}

bool Mp7JrsReferenceType::Existidref() const
{
	return m_idref_Exist;
}
void Mp7JrsReferenceType::Setidref(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsReferenceType::Setidref().");
	}
    m_idref = item;
	m_idref_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsReferenceType::Invalidateidref()
{
	m_idref_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsReferenceType::Getxpath() const
{
	return m_xpath;
}

bool Mp7JrsReferenceType::Existxpath() const
{
	return m_xpath_Exist;
}
void Mp7JrsReferenceType::Setxpath(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsReferenceType::Setxpath().");
	}
    m_xpath = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_xpath) m_xpath->SetParent(m_myself.getPointer());
	m_xpath_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsReferenceType::Invalidatexpath()
{
	m_xpath_Exist = false;
}
XMLCh * Mp7JrsReferenceType::Gethref() const
{
	return m_href;
}

bool Mp7JrsReferenceType::Existhref() const
{
	return m_href_Exist;
}
void Mp7JrsReferenceType::Sethref(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsReferenceType::Sethref().");
	}
    m_href = item;
	m_href_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsReferenceType::Invalidatehref()
{
	m_href_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsReferenceType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsReferenceType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsReferenceType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsReferenceType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ReferenceType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_idref_Exist)
	{
	// String
	if(m_idref != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("idref"), m_idref);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_xpath_Exist)
	{
	// Pattern
	if (m_xpath != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_xpath->ToText();
		element->setAttributeNS(X(""), X("xpath"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_href_Exist)
	{
	// String
	if(m_href != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("href"), m_href);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsReferenceType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("idref")))
	{
        // Deserialize string type
        this->Setidref(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("idref"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("idref")))
	{
        // Deserialize string type
        this->Setidref(Mp7JrsConvert::TextToString(parent->getAttribute(X("idref"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("xpath")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("xpath")));
        this->Setxpath(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("xpath")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("xpath")));
        this->Setxpath(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("href")))
	{
        // Deserialize string type
        this->Sethref(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("href"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("href")))
	{
        // Deserialize string type
        this->Sethref(Mp7JrsConvert::TextToString(parent->getAttribute(X("href"))));
        * current = parent;
    }

// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsReferenceType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsReferenceType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsReferenceType_ExtMethodImpl.h


