 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsScalableColorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsScalableColorType::IMp7JrsScalableColorType()
{

// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtPropInit.h

}

IMp7JrsScalableColorType::~IMp7JrsScalableColorType()
{
// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtPropCleanup.h

}

Mp7JrsScalableColorType::Mp7JrsScalableColorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsScalableColorType::~Mp7JrsScalableColorType()
{
	Cleanup();
}

void Mp7JrsScalableColorType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_numOfCoeff = Mp7JrsScalableColorType_numOfCoeff_LocalType::UninitializedEnumeration;
	m_numOfBitplanesDiscarded = Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::UninitializedEnumeration;

	// Init elements (element, union sequence choice all any)
	
	m_Coeff = Mp7JrsintegerVectorPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtMyPropInit.h

}

void Mp7JrsScalableColorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Coeff);
}

void Mp7JrsScalableColorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ScalableColorTypePtr, since we
	// might need GetBase(), which isn't defined in IScalableColorType
    const Mp7JrsPtr< Mp7JrsScalableColorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsScalableColorType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
		this->SetnumOfCoeff(tmp->GetnumOfCoeff());
	}
	{
		this->SetnumOfBitplanesDiscarded(tmp->GetnumOfBitplanesDiscarded());
	}
		// Mp7JrsFactory::DeleteObject(m_Coeff);
		this->SetCoeff(Mp7JrsFactory::CloneObject(tmp->GetCoeff()));
}

Mp7JrsNodePtr Mp7JrsScalableColorType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsScalableColorType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsScalableColorType::GetBase() const
{
	return m_Base;
}

Mp7JrsScalableColorType_numOfCoeff_LocalType::Enumeration Mp7JrsScalableColorType::GetnumOfCoeff() const
{
	return m_numOfCoeff;
}

void Mp7JrsScalableColorType::SetnumOfCoeff(Mp7JrsScalableColorType_numOfCoeff_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsScalableColorType::SetnumOfCoeff().");
	}
    m_numOfCoeff = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::Enumeration Mp7JrsScalableColorType::GetnumOfBitplanesDiscarded() const
{
	return m_numOfBitplanesDiscarded;
}

void Mp7JrsScalableColorType::SetnumOfBitplanesDiscarded(Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsScalableColorType::SetnumOfBitplanesDiscarded().");
	}
    m_numOfBitplanesDiscarded = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsScalableColorType::GetCoeff() const
{
	return m_Coeff;
}

void Mp7JrsScalableColorType::SetCoeff(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsScalableColorType::SetCoeff().");
	}
	if (m_Coeff != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Coeff);
		m_Coeff = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Coeff) m_Coeff->SetParent(m_myself.getPointer());
		if(m_Coeff != Mp7JrsintegerVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Coeff->SetContentName(XMLString::transcode("Coeff"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsScalableColorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Coeff")) == 0)
	{
		// Coeff is simple element integerVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCoeff()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:integerVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsintegerVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCoeff(p, client);
					if((p = GetCoeff()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ScalableColorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ScalableColorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsScalableColorType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsScalableColorType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsScalableColorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ScalableColorType"));
    // Attribute Serialization:
	// Enumeration
	if(m_numOfCoeff != Mp7JrsScalableColorType_numOfCoeff_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsScalableColorType_numOfCoeff_LocalType::ToText(m_numOfCoeff);
		element->setAttributeNS(X(""), X("numOfCoeff"), tmp);
		XMLString::release(&tmp);
	}
    // Attribute Serialization:
	// Enumeration
	if(m_numOfBitplanesDiscarded != Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::ToText(m_numOfBitplanesDiscarded);
		element->setAttributeNS(X(""), X("numOfBitplanesDiscarded"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	if (m_Coeff != Mp7JrsintegerVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Coeff->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsScalableColorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("numOfCoeff")))
	{
        this->SetnumOfCoeff(Mp7JrsScalableColorType_numOfCoeff_LocalType::Parse(parent->getAttributeNS(X(""), X("numOfCoeff"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("numOfCoeff")))
	{
        this->SetnumOfCoeff(Mp7JrsScalableColorType_numOfCoeff_LocalType::Parse(parent->getAttribute(X("numOfCoeff"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("numOfBitplanesDiscarded")))
	{
        this->SetnumOfBitplanesDiscarded(Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::Parse(parent->getAttributeNS(X(""), X("numOfBitplanesDiscarded"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("numOfBitplanesDiscarded")))
	{
        this->SetnumOfBitplanesDiscarded(Mp7JrsScalableColorType_numOfBitplanesDiscarded_LocalType::Parse(parent->getAttribute(X("numOfBitplanesDiscarded"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialise Element ValCollectionType
        if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Coeff")) == 0))
        {
            // Deserialize factory type
            Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
            tmp->Deserialize(doc, parent, current);
            this->SetCoeff(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsScalableColorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Coeff")) == 0))
  {
    Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
    this->SetCoeff(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsScalableColorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetCoeff() != Mp7JrsNodePtr())
		result.Insert(GetCoeff());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsScalableColorType_ExtMethodImpl.h


