 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsMediaInformationType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsControlledTermUseType.h"
#include "Mp7JrsCreationInformationType.h"
#include "Mp7JrsUsageInformationType.h"
#include "Mp7JrsSegmentType_TextAnnotation_CollectionType.h"
#include "Mp7JrsSegmentType_CollectionType.h"
#include "Mp7JrsSegmentType_MatchingHint_CollectionType.h"
#include "Mp7JrsSegmentType_PointOfView_CollectionType.h"
#include "Mp7JrsSegmentType_Relation_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSegmentType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsSegmentType_TextAnnotation_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:TextAnnotation
#include "Mp7JrsSegmentType_LocalType.h" // Choice collection Semantic
#include "Mp7JrsSemanticType.h" // Choice collection element Semantic
#include "Mp7JrsMatchingHintType.h" // Element collection urn:mpeg:mpeg7:schema:2004:MatchingHint
#include "Mp7JrsPointOfViewType.h" // Element collection urn:mpeg:mpeg7:schema:2004:PointOfView
#include "Mp7JrsRelationType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relation

#include <assert.h>
IMp7JrsSegmentType::IMp7JrsSegmentType()
{

// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtPropInit.h

}

IMp7JrsSegmentType::~IMp7JrsSegmentType()
{
// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtPropCleanup.h

}

Mp7JrsSegmentType::Mp7JrsSegmentType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSegmentType::~Mp7JrsSegmentType()
{
	Cleanup();
}

void Mp7JrsSegmentType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_MediaInformation = Mp7JrsMediaInformationPtr(); // Class
	m_MediaInformationRef = Mp7JrsReferencePtr(); // Class
	m_MediaLocator = Mp7JrsMediaLocatorPtr(); // Class
	m_StructuralUnit = Mp7JrsControlledTermUsePtr(); // Class
	m_StructuralUnit_Exist = false;
	m_CreationInformation = Mp7JrsCreationInformationPtr(); // Class
	m_CreationInformationRef = Mp7JrsReferencePtr(); // Class
	m_UsageInformation = Mp7JrsUsageInformationPtr(); // Class
	m_UsageInformationRef = Mp7JrsReferencePtr(); // Class
	m_TextAnnotation = Mp7JrsSegmentType_TextAnnotation_CollectionPtr(); // Collection
	m_SegmentType_LocalType = Mp7JrsSegmentType_CollectionPtr(); // Collection
	m_MatchingHint = Mp7JrsSegmentType_MatchingHint_CollectionPtr(); // Collection
	m_PointOfView = Mp7JrsSegmentType_PointOfView_CollectionPtr(); // Collection
	m_Relation = Mp7JrsSegmentType_Relation_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtMyPropInit.h

}

void Mp7JrsSegmentType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_MediaInformation);
	// Mp7JrsFactory::DeleteObject(m_MediaInformationRef);
	// Mp7JrsFactory::DeleteObject(m_MediaLocator);
	// Mp7JrsFactory::DeleteObject(m_StructuralUnit);
	// Mp7JrsFactory::DeleteObject(m_CreationInformation);
	// Mp7JrsFactory::DeleteObject(m_CreationInformationRef);
	// Mp7JrsFactory::DeleteObject(m_UsageInformation);
	// Mp7JrsFactory::DeleteObject(m_UsageInformationRef);
	// Mp7JrsFactory::DeleteObject(m_TextAnnotation);
	// Mp7JrsFactory::DeleteObject(m_SegmentType_LocalType);
	// Mp7JrsFactory::DeleteObject(m_MatchingHint);
	// Mp7JrsFactory::DeleteObject(m_PointOfView);
	// Mp7JrsFactory::DeleteObject(m_Relation);
}

void Mp7JrsSegmentType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SegmentTypePtr, since we
	// might need GetBase(), which isn't defined in ISegmentType
    const Mp7JrsPtr< Mp7JrsSegmentType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSegmentType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_MediaInformation);
		this->SetMediaInformation(Mp7JrsFactory::CloneObject(tmp->GetMediaInformation()));
		// Mp7JrsFactory::DeleteObject(m_MediaInformationRef);
		this->SetMediaInformationRef(Mp7JrsFactory::CloneObject(tmp->GetMediaInformationRef()));
		// Mp7JrsFactory::DeleteObject(m_MediaLocator);
		this->SetMediaLocator(Mp7JrsFactory::CloneObject(tmp->GetMediaLocator()));
	if (tmp->IsValidStructuralUnit())
	{
		// Mp7JrsFactory::DeleteObject(m_StructuralUnit);
		this->SetStructuralUnit(Mp7JrsFactory::CloneObject(tmp->GetStructuralUnit()));
	}
	else
	{
		InvalidateStructuralUnit();
	}
		// Mp7JrsFactory::DeleteObject(m_CreationInformation);
		this->SetCreationInformation(Mp7JrsFactory::CloneObject(tmp->GetCreationInformation()));
		// Mp7JrsFactory::DeleteObject(m_CreationInformationRef);
		this->SetCreationInformationRef(Mp7JrsFactory::CloneObject(tmp->GetCreationInformationRef()));
		// Mp7JrsFactory::DeleteObject(m_UsageInformation);
		this->SetUsageInformation(Mp7JrsFactory::CloneObject(tmp->GetUsageInformation()));
		// Mp7JrsFactory::DeleteObject(m_UsageInformationRef);
		this->SetUsageInformationRef(Mp7JrsFactory::CloneObject(tmp->GetUsageInformationRef()));
		// Mp7JrsFactory::DeleteObject(m_TextAnnotation);
		this->SetTextAnnotation(Mp7JrsFactory::CloneObject(tmp->GetTextAnnotation()));
		// Mp7JrsFactory::DeleteObject(m_SegmentType_LocalType);
		this->SetSegmentType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetSegmentType_LocalType()));
		// Mp7JrsFactory::DeleteObject(m_MatchingHint);
		this->SetMatchingHint(Mp7JrsFactory::CloneObject(tmp->GetMatchingHint()));
		// Mp7JrsFactory::DeleteObject(m_PointOfView);
		this->SetPointOfView(Mp7JrsFactory::CloneObject(tmp->GetPointOfView()));
		// Mp7JrsFactory::DeleteObject(m_Relation);
		this->SetRelation(Mp7JrsFactory::CloneObject(tmp->GetRelation()));
}

Mp7JrsNodePtr Mp7JrsSegmentType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSegmentType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsSegmentType::GetBase() const
{
	return m_Base;
}

Mp7JrsMediaInformationPtr Mp7JrsSegmentType::GetMediaInformation() const
{
	return m_MediaInformation;
}

Mp7JrsReferencePtr Mp7JrsSegmentType::GetMediaInformationRef() const
{
	return m_MediaInformationRef;
}

Mp7JrsMediaLocatorPtr Mp7JrsSegmentType::GetMediaLocator() const
{
	return m_MediaLocator;
}

Mp7JrsControlledTermUsePtr Mp7JrsSegmentType::GetStructuralUnit() const
{
	return m_StructuralUnit;
}

// Element is optional
bool Mp7JrsSegmentType::IsValidStructuralUnit() const
{
	return m_StructuralUnit_Exist;
}

Mp7JrsCreationInformationPtr Mp7JrsSegmentType::GetCreationInformation() const
{
	return m_CreationInformation;
}

Mp7JrsReferencePtr Mp7JrsSegmentType::GetCreationInformationRef() const
{
	return m_CreationInformationRef;
}

Mp7JrsUsageInformationPtr Mp7JrsSegmentType::GetUsageInformation() const
{
	return m_UsageInformation;
}

Mp7JrsReferencePtr Mp7JrsSegmentType::GetUsageInformationRef() const
{
	return m_UsageInformationRef;
}

Mp7JrsSegmentType_TextAnnotation_CollectionPtr Mp7JrsSegmentType::GetTextAnnotation() const
{
	return m_TextAnnotation;
}

Mp7JrsSegmentType_CollectionPtr Mp7JrsSegmentType::GetSegmentType_LocalType() const
{
	return m_SegmentType_LocalType;
}

Mp7JrsSegmentType_MatchingHint_CollectionPtr Mp7JrsSegmentType::GetMatchingHint() const
{
	return m_MatchingHint;
}

Mp7JrsSegmentType_PointOfView_CollectionPtr Mp7JrsSegmentType::GetPointOfView() const
{
	return m_PointOfView;
}

Mp7JrsSegmentType_Relation_CollectionPtr Mp7JrsSegmentType::GetRelation() const
{
	return m_Relation;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsSegmentType::SetMediaInformation(const Mp7JrsMediaInformationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetMediaInformation().");
	}
	if (m_MediaInformation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaInformation);
		m_MediaInformation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaInformation) m_MediaInformation->SetParent(m_myself.getPointer());
		if (m_MediaInformation && m_MediaInformation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaInformationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaInformation->UseTypeAttribute = true;
		}
		if(m_MediaInformation != Mp7JrsMediaInformationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaInformation->SetContentName(XMLString::transcode("MediaInformation"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaInformationRef);
	m_MediaInformationRef = Mp7JrsReferencePtr();
	// Mp7JrsFactory::DeleteObject(m_MediaLocator);
	m_MediaLocator = Mp7JrsMediaLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSegmentType::SetMediaInformationRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetMediaInformationRef().");
	}
	if (m_MediaInformationRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaInformationRef);
		m_MediaInformationRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaInformationRef) m_MediaInformationRef->SetParent(m_myself.getPointer());
		if (m_MediaInformationRef && m_MediaInformationRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaInformationRef->UseTypeAttribute = true;
		}
		if(m_MediaInformationRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaInformationRef->SetContentName(XMLString::transcode("MediaInformationRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaInformation);
	m_MediaInformation = Mp7JrsMediaInformationPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaLocator);
	m_MediaLocator = Mp7JrsMediaLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSegmentType::SetMediaLocator(const Mp7JrsMediaLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetMediaLocator().");
	}
	if (m_MediaLocator != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaLocator);
		m_MediaLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaLocator) m_MediaLocator->SetParent(m_myself.getPointer());
		if (m_MediaLocator && m_MediaLocator->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaLocator->UseTypeAttribute = true;
		}
		if(m_MediaLocator != Mp7JrsMediaLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaLocator->SetContentName(XMLString::transcode("MediaLocator"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaInformation);
	m_MediaInformation = Mp7JrsMediaInformationPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaInformationRef);
	m_MediaInformationRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::SetStructuralUnit(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetStructuralUnit().");
	}
	if (m_StructuralUnit != item || m_StructuralUnit_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_StructuralUnit);
		m_StructuralUnit = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StructuralUnit) m_StructuralUnit->SetParent(m_myself.getPointer());
		if (m_StructuralUnit && m_StructuralUnit->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_StructuralUnit->UseTypeAttribute = true;
		}
		m_StructuralUnit_Exist = true;
		if(m_StructuralUnit != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StructuralUnit->SetContentName(XMLString::transcode("StructuralUnit"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::InvalidateStructuralUnit()
{
	m_StructuralUnit_Exist = false;
}
	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsSegmentType::SetCreationInformation(const Mp7JrsCreationInformationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetCreationInformation().");
	}
	if (m_CreationInformation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_CreationInformation);
		m_CreationInformation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CreationInformation) m_CreationInformation->SetParent(m_myself.getPointer());
		if (m_CreationInformation && m_CreationInformation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationInformationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_CreationInformation->UseTypeAttribute = true;
		}
		if(m_CreationInformation != Mp7JrsCreationInformationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CreationInformation->SetContentName(XMLString::transcode("CreationInformation"));
		}
	// Mp7JrsFactory::DeleteObject(m_CreationInformationRef);
	m_CreationInformationRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSegmentType::SetCreationInformationRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetCreationInformationRef().");
	}
	if (m_CreationInformationRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_CreationInformationRef);
		m_CreationInformationRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CreationInformationRef) m_CreationInformationRef->SetParent(m_myself.getPointer());
		if (m_CreationInformationRef && m_CreationInformationRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_CreationInformationRef->UseTypeAttribute = true;
		}
		if(m_CreationInformationRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CreationInformationRef->SetContentName(XMLString::transcode("CreationInformationRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_CreationInformation);
	m_CreationInformation = Mp7JrsCreationInformationPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsSegmentType::SetUsageInformation(const Mp7JrsUsageInformationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetUsageInformation().");
	}
	if (m_UsageInformation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UsageInformation);
		m_UsageInformation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UsageInformation) m_UsageInformation->SetParent(m_myself.getPointer());
		if (m_UsageInformation && m_UsageInformation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UsageInformationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_UsageInformation->UseTypeAttribute = true;
		}
		if(m_UsageInformation != Mp7JrsUsageInformationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UsageInformation->SetContentName(XMLString::transcode("UsageInformation"));
		}
	// Mp7JrsFactory::DeleteObject(m_UsageInformationRef);
	m_UsageInformationRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSegmentType::SetUsageInformationRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetUsageInformationRef().");
	}
	if (m_UsageInformationRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UsageInformationRef);
		m_UsageInformationRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UsageInformationRef) m_UsageInformationRef->SetParent(m_myself.getPointer());
		if (m_UsageInformationRef && m_UsageInformationRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_UsageInformationRef->UseTypeAttribute = true;
		}
		if(m_UsageInformationRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UsageInformationRef->SetContentName(XMLString::transcode("UsageInformationRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_UsageInformation);
	m_UsageInformation = Mp7JrsUsageInformationPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::SetTextAnnotation(const Mp7JrsSegmentType_TextAnnotation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetTextAnnotation().");
	}
	if (m_TextAnnotation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextAnnotation);
		m_TextAnnotation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextAnnotation) m_TextAnnotation->SetParent(m_myself.getPointer());
		if(m_TextAnnotation != Mp7JrsSegmentType_TextAnnotation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TextAnnotation->SetContentName(XMLString::transcode("TextAnnotation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::SetSegmentType_LocalType(const Mp7JrsSegmentType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetSegmentType_LocalType().");
	}
	if (m_SegmentType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SegmentType_LocalType);
		m_SegmentType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SegmentType_LocalType) m_SegmentType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::SetMatchingHint(const Mp7JrsSegmentType_MatchingHint_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetMatchingHint().");
	}
	if (m_MatchingHint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MatchingHint);
		m_MatchingHint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MatchingHint) m_MatchingHint->SetParent(m_myself.getPointer());
		if(m_MatchingHint != Mp7JrsSegmentType_MatchingHint_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MatchingHint->SetContentName(XMLString::transcode("MatchingHint"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::SetPointOfView(const Mp7JrsSegmentType_PointOfView_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetPointOfView().");
	}
	if (m_PointOfView != item)
	{
		// Mp7JrsFactory::DeleteObject(m_PointOfView);
		m_PointOfView = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PointOfView) m_PointOfView->SetParent(m_myself.getPointer());
		if(m_PointOfView != Mp7JrsSegmentType_PointOfView_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PointOfView->SetContentName(XMLString::transcode("PointOfView"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::SetRelation(const Mp7JrsSegmentType_Relation_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetRelation().");
	}
	if (m_Relation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Relation);
		m_Relation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Relation) m_Relation->SetParent(m_myself.getPointer());
		if(m_Relation != Mp7JrsSegmentType_Relation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Relation->SetContentName(XMLString::transcode("Relation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsSegmentType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsSegmentType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsSegmentType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSegmentType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsSegmentType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsSegmentType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSegmentType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsSegmentType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsSegmentType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSegmentType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSegmentType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSegmentType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSegmentType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSegmentType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSegmentType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSegmentType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsSegmentType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSegmentType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("MediaInformation")) == 0)
	{
		// MediaInformation is simple element MediaInformationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaInformation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaInformationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaInformationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaInformation(p, client);
					if((p = GetMediaInformation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaInformationRef")) == 0)
	{
		// MediaInformationRef is simple element ReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaInformationRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaInformationRef(p, client);
					if((p = GetMediaInformationRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaLocator")) == 0)
	{
		// MediaLocator is simple element MediaLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaLocator(p, client);
					if((p = GetMediaLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("StructuralUnit")) == 0)
	{
		// StructuralUnit is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetStructuralUnit()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetStructuralUnit(p, client);
					if((p = GetStructuralUnit()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CreationInformation")) == 0)
	{
		// CreationInformation is simple element CreationInformationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCreationInformation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationInformationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreationInformationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCreationInformation(p, client);
					if((p = GetCreationInformation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CreationInformationRef")) == 0)
	{
		// CreationInformationRef is simple element ReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCreationInformationRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCreationInformationRef(p, client);
					if((p = GetCreationInformationRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UsageInformation")) == 0)
	{
		// UsageInformation is simple element UsageInformationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetUsageInformation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UsageInformationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUsageInformationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetUsageInformation(p, client);
					if((p = GetUsageInformation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UsageInformationRef")) == 0)
	{
		// UsageInformationRef is simple element ReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetUsageInformationRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetUsageInformationRef(p, client);
					if((p = GetUsageInformationRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TextAnnotation")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:TextAnnotation is item of type SegmentType_TextAnnotation_LocalType
		// in element collection SegmentType_TextAnnotation_CollectionType
		
		context->Found = true;
		Mp7JrsSegmentType_TextAnnotation_CollectionPtr coll = GetTextAnnotation();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSegmentType_TextAnnotation_CollectionType;
				SetTextAnnotation(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SegmentType_TextAnnotation_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSegmentType_TextAnnotation_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Semantic")) == 0)
	{
		// Semantic is contained in itemtype SegmentType_LocalType
		// in choice collection SegmentType_CollectionType
		
		context->Found = true;
		Mp7JrsSegmentType_CollectionPtr coll = GetSegmentType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSegmentType_CollectionType;
				SetSegmentType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsSegmentType_LocalPtr)coll->elementAt(i))->GetSemantic()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsSegmentType_LocalPtr item = CreateSegmentType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsSegmentType_LocalPtr)coll->elementAt(i))->SetSemantic(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsSegmentType_LocalPtr)coll->elementAt(i))->GetSemantic()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SemanticRef")) == 0)
	{
		// SemanticRef is contained in itemtype SegmentType_LocalType
		// in choice collection SegmentType_CollectionType
		
		context->Found = true;
		Mp7JrsSegmentType_CollectionPtr coll = GetSegmentType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSegmentType_CollectionType;
				SetSegmentType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsSegmentType_LocalPtr)coll->elementAt(i))->GetSemanticRef()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsSegmentType_LocalPtr item = CreateSegmentType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsSegmentType_LocalPtr)coll->elementAt(i))->SetSemanticRef(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsSegmentType_LocalPtr)coll->elementAt(i))->GetSemanticRef()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MatchingHint")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:MatchingHint is item of type MatchingHintType
		// in element collection SegmentType_MatchingHint_CollectionType
		
		context->Found = true;
		Mp7JrsSegmentType_MatchingHint_CollectionPtr coll = GetMatchingHint();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSegmentType_MatchingHint_CollectionType;
				SetMatchingHint(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MatchingHintType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMatchingHintPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PointOfView")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:PointOfView is item of type PointOfViewType
		// in element collection SegmentType_PointOfView_CollectionType
		
		context->Found = true;
		Mp7JrsSegmentType_PointOfView_CollectionPtr coll = GetPointOfView();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSegmentType_PointOfView_CollectionType;
				SetPointOfView(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PointOfViewType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPointOfViewPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Relation")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Relation is item of type RelationType
		// in element collection SegmentType_Relation_CollectionType
		
		context->Found = true;
		Mp7JrsSegmentType_Relation_CollectionPtr coll = GetRelation();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSegmentType_Relation_CollectionType;
				SetRelation(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRelationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SegmentType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SegmentType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSegmentType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsSegmentType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsSegmentType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SegmentType"));
	// Element serialization:
	// Class
	
	if (m_MediaInformation != Mp7JrsMediaInformationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaInformation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaInformation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaInformation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MediaInformationRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaInformationRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaInformationRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaInformationRef->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_MediaLocator != Mp7JrsMediaLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaLocator->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_StructuralUnit != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_StructuralUnit->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("StructuralUnit"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_StructuralUnit->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_CreationInformation != Mp7JrsCreationInformationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_CreationInformation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("CreationInformation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_CreationInformation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_CreationInformationRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_CreationInformationRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("CreationInformationRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_CreationInformationRef->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_UsageInformation != Mp7JrsUsageInformationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_UsageInformation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("UsageInformation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_UsageInformation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_UsageInformationRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_UsageInformationRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("UsageInformationRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_UsageInformationRef->Serialize(doc, element, &elem);
	}
	if (m_TextAnnotation != Mp7JrsSegmentType_TextAnnotation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_TextAnnotation->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_SegmentType_LocalType != Mp7JrsSegmentType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SegmentType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_MatchingHint != Mp7JrsSegmentType_MatchingHint_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_MatchingHint->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_PointOfView != Mp7JrsSegmentType_PointOfView_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_PointOfView->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Relation != Mp7JrsSegmentType_Relation_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Relation->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSegmentType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaInformation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaInformation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaInformationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaInformation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaInformationRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaInformationRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaInformationRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaLocator")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMediaLocator(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("StructuralUnit"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("StructuralUnit")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetStructuralUnit(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("CreationInformation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("CreationInformation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateCreationInformationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetCreationInformation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("CreationInformationRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("CreationInformationRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetCreationInformationRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("UsageInformation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("UsageInformation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateUsageInformationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetUsageInformation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("UsageInformationRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("UsageInformationRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetUsageInformationRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("TextAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("TextAnnotation")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSegmentType_TextAnnotation_CollectionPtr tmp = CreateSegmentType_TextAnnotation_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTextAnnotation(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:Semantic
            Mp7JrsUtil::HasNodeName(parent, X("Semantic"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Semantic")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:SemanticRef
            Mp7JrsUtil::HasNodeName(parent, X("SemanticRef"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:SemanticRef")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsSegmentType_CollectionPtr tmp = CreateSegmentType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSegmentType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("MatchingHint"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("MatchingHint")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSegmentType_MatchingHint_CollectionPtr tmp = CreateSegmentType_MatchingHint_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetMatchingHint(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("PointOfView"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("PointOfView")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSegmentType_PointOfView_CollectionPtr tmp = CreateSegmentType_PointOfView_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetPointOfView(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Relation"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Relation")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSegmentType_Relation_CollectionPtr tmp = CreateSegmentType_Relation_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetRelation(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSegmentType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("MediaInformation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaInformationType;
    }
    this->SetMediaInformation(child);
  }
  if (XMLString::compareString(elementname, X("MediaInformationRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetMediaInformationRef(child);
  }
  if (XMLString::compareString(elementname, X("MediaLocator")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaLocatorType;
    }
    this->SetMediaLocator(child);
  }
  if (XMLString::compareString(elementname, X("StructuralUnit")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetStructuralUnit(child);
  }
  if (XMLString::compareString(elementname, X("CreationInformation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateCreationInformationType;
    }
    this->SetCreationInformation(child);
  }
  if (XMLString::compareString(elementname, X("CreationInformationRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetCreationInformationRef(child);
  }
  if (XMLString::compareString(elementname, X("UsageInformation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateUsageInformationType;
    }
    this->SetUsageInformation(child);
  }
  if (XMLString::compareString(elementname, X("UsageInformationRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetUsageInformationRef(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("TextAnnotation")) == 0))
  {
    Mp7JrsSegmentType_TextAnnotation_CollectionPtr tmp = CreateSegmentType_TextAnnotation_CollectionType;
    this->SetTextAnnotation(tmp);
    child = tmp;
  }
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("Semantic"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("Semantic")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("SemanticRef"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("SemanticRef")) == 0)
    )
  {
    Mp7JrsSegmentType_CollectionPtr tmp = CreateSegmentType_CollectionType;
    this->SetSegmentType_LocalType(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("MatchingHint")) == 0))
  {
    Mp7JrsSegmentType_MatchingHint_CollectionPtr tmp = CreateSegmentType_MatchingHint_CollectionType;
    this->SetMatchingHint(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("PointOfView")) == 0))
  {
    Mp7JrsSegmentType_PointOfView_CollectionPtr tmp = CreateSegmentType_PointOfView_CollectionType;
    this->SetPointOfView(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Relation")) == 0))
  {
    Mp7JrsSegmentType_Relation_CollectionPtr tmp = CreateSegmentType_Relation_CollectionType;
    this->SetRelation(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSegmentType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetStructuralUnit() != Mp7JrsNodePtr())
		result.Insert(GetStructuralUnit());
	if (GetTextAnnotation() != Mp7JrsNodePtr())
		result.Insert(GetTextAnnotation());
	if (GetSegmentType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetSegmentType_LocalType());
	if (GetMatchingHint() != Mp7JrsNodePtr())
		result.Insert(GetMatchingHint());
	if (GetPointOfView() != Mp7JrsNodePtr())
		result.Insert(GetPointOfView());
	if (GetRelation() != Mp7JrsNodePtr())
		result.Insert(GetRelation());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetMediaInformation() != Mp7JrsNodePtr())
		result.Insert(GetMediaInformation());
	if (GetMediaInformationRef() != Mp7JrsNodePtr())
		result.Insert(GetMediaInformationRef());
	if (GetMediaLocator() != Mp7JrsNodePtr())
		result.Insert(GetMediaLocator());
	if (GetCreationInformation() != Mp7JrsNodePtr())
		result.Insert(GetCreationInformation());
	if (GetCreationInformationRef() != Mp7JrsNodePtr())
		result.Insert(GetCreationInformationRef());
	if (GetUsageInformation() != Mp7JrsNodePtr())
		result.Insert(GetUsageInformation());
	if (GetUsageInformationRef() != Mp7JrsNodePtr())
		result.Insert(GetUsageInformationRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSegmentType_ExtMethodImpl.h


