 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtImplInclude.h



#include "Mp7JrsContentAbstractionType.h"
#include "Mp7JrsSemanticDescriptionType_Semantics_CollectionType.h"
#include "Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionType.h"
#include "Mp7JrsContentDescriptionType_Affective_CollectionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsSemanticDescriptionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey
#include "Mp7JrsAffectiveType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Affective
#include "Mp7JrsSemanticType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Semantics
#include "Mp7JrsConceptCollectionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ConceptCollection

#include <assert.h>
IMp7JrsSemanticDescriptionType::IMp7JrsSemanticDescriptionType()
{

// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtPropInit.h

}

IMp7JrsSemanticDescriptionType::~IMp7JrsSemanticDescriptionType()
{
// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtPropCleanup.h

}

Mp7JrsSemanticDescriptionType::Mp7JrsSemanticDescriptionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSemanticDescriptionType::~Mp7JrsSemanticDescriptionType()
{
	Cleanup();
}

void Mp7JrsSemanticDescriptionType::Init()
{
	// Init base
	m_Base = CreateContentAbstractionType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Semantics = Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr(); // Collection
	m_ConceptCollection = Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtMyPropInit.h

}

void Mp7JrsSemanticDescriptionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Semantics);
	// Mp7JrsFactory::DeleteObject(m_ConceptCollection);
}

void Mp7JrsSemanticDescriptionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SemanticDescriptionTypePtr, since we
	// might need GetBase(), which isn't defined in ISemanticDescriptionType
    const Mp7JrsPtr< Mp7JrsSemanticDescriptionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsContentAbstractionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSemanticDescriptionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Semantics);
		this->SetSemantics(Mp7JrsFactory::CloneObject(tmp->GetSemantics()));
		// Mp7JrsFactory::DeleteObject(m_ConceptCollection);
		this->SetConceptCollection(Mp7JrsFactory::CloneObject(tmp->GetConceptCollection()));
}

Mp7JrsNodePtr Mp7JrsSemanticDescriptionType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSemanticDescriptionType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsContentAbstractionType > Mp7JrsSemanticDescriptionType::GetBase() const
{
	return m_Base;
}

Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr Mp7JrsSemanticDescriptionType::GetSemantics() const
{
	return m_Semantics;
}

Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr Mp7JrsSemanticDescriptionType::GetConceptCollection() const
{
	return m_ConceptCollection;
}

// implementing setter for choice 
/* element */
void Mp7JrsSemanticDescriptionType::SetSemantics(const Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticDescriptionType::SetSemantics().");
	}
	if (m_Semantics != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Semantics);
		m_Semantics = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Semantics) m_Semantics->SetParent(m_myself.getPointer());
		if(m_Semantics != Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Semantics->SetContentName(XMLString::transcode("Semantics"));
		}
	// Mp7JrsFactory::DeleteObject(m_ConceptCollection);
	m_ConceptCollection = Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSemanticDescriptionType::SetConceptCollection(const Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticDescriptionType::SetConceptCollection().");
	}
	if (m_ConceptCollection != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ConceptCollection);
		m_ConceptCollection = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ConceptCollection) m_ConceptCollection->SetParent(m_myself.getPointer());
		if(m_ConceptCollection != Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ConceptCollection->SetContentName(XMLString::transcode("ConceptCollection"));
		}
	// Mp7JrsFactory::DeleteObject(m_Semantics);
	m_Semantics = Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsContentDescriptionType_Affective_CollectionPtr Mp7JrsSemanticDescriptionType::GetAffective() const
{
	return GetBase()->GetBase()->GetAffective();
}

void Mp7JrsSemanticDescriptionType::SetAffective(const Mp7JrsContentDescriptionType_Affective_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticDescriptionType::SetAffective().");
	}
	GetBase()->GetBase()->SetAffective(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsDescriptionMetadataPtr Mp7JrsSemanticDescriptionType::GetDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsSemanticDescriptionType::IsValidDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetBase()->IsValidDescriptionMetadata();
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsSemanticDescriptionType::GetRelationships() const
{
	return GetBase()->GetBase()->GetBase()->GetRelationships();
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsSemanticDescriptionType::GetOrderingKey() const
{
	return GetBase()->GetBase()->GetBase()->GetOrderingKey();
}

void Mp7JrsSemanticDescriptionType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticDescriptionType::SetDescriptionMetadata().");
	}
	GetBase()->GetBase()->GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticDescriptionType::InvalidateDescriptionMetadata()
{
	GetBase()->GetBase()->GetBase()->InvalidateDescriptionMetadata();
}
void Mp7JrsSemanticDescriptionType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticDescriptionType::SetRelationships().");
	}
	GetBase()->GetBase()->GetBase()->SetRelationships(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSemanticDescriptionType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSemanticDescriptionType::SetOrderingKey().");
	}
	GetBase()->GetBase()->GetBase()->SetOrderingKey(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSemanticDescriptionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Semantics")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Semantics is item of type SemanticType
		// in element collection SemanticDescriptionType_Semantics_CollectionType
		
		context->Found = true;
		Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr coll = GetSemantics();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSemanticDescriptionType_Semantics_CollectionType;
				SetSemantics(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SemanticType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSemanticPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ConceptCollection")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ConceptCollection is item of type ConceptCollectionType
		// in element collection SemanticDescriptionType_ConceptCollection_CollectionType
		
		context->Found = true;
		Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr coll = GetConceptCollection();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSemanticDescriptionType_ConceptCollection_CollectionType;
				SetConceptCollection(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ConceptCollectionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsConceptCollectionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SemanticDescriptionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SemanticDescriptionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSemanticDescriptionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsSemanticDescriptionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsSemanticDescriptionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SemanticDescriptionType"));
	// Element serialization:
	if (m_Semantics != Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Semantics->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_ConceptCollection != Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ConceptCollection->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSemanticDescriptionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsContentAbstractionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Semantics"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Semantics")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr tmp = CreateSemanticDescriptionType_Semantics_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetSemantics(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("ConceptCollection"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("ConceptCollection")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr tmp = CreateSemanticDescriptionType_ConceptCollection_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetConceptCollection(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSemanticDescriptionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Semantics")) == 0))
  {
    Mp7JrsSemanticDescriptionType_Semantics_CollectionPtr tmp = CreateSemanticDescriptionType_Semantics_CollectionType;
    this->SetSemantics(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ConceptCollection")) == 0))
  {
    Mp7JrsSemanticDescriptionType_ConceptCollection_CollectionPtr tmp = CreateSemanticDescriptionType_ConceptCollection_CollectionType;
    this->SetConceptCollection(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSemanticDescriptionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSemantics() != Mp7JrsNodePtr())
		result.Insert(GetSemantics());
	if (GetConceptCollection() != Mp7JrsNodePtr())
		result.Insert(GetConceptCollection());
	if (GetAffective() != Mp7JrsNodePtr())
		result.Insert(GetAffective());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSemanticDescriptionType_ExtMethodImpl.h


