 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtImplInclude.h



#include "Mp7JrsSummaryType.h"
#include "Mp7JrsSequentialSummaryType_components_CollectionType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h"
#include "Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionType.h"
#include "Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionType.h"
#include "Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionType.h"
#include "Mp7JrsSummaryType_Name_CollectionType.h"
#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSequentialSummaryType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Name
#include "Mp7JrsVisualSummaryComponentType.h" // Element collection urn:mpeg:mpeg7:schema:2004:VisualSummaryComponent
#include "Mp7JrsAudioSummaryComponentType.h" // Element collection urn:mpeg:mpeg7:schema:2004:AudioSummaryComponent
#include "Mp7JrsTextualSummaryComponentType.h" // Element collection urn:mpeg:mpeg7:schema:2004:TextualSummaryComponent

#include <assert.h>
IMp7JrsSequentialSummaryType::IMp7JrsSequentialSummaryType()
{

// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtPropInit.h

}

IMp7JrsSequentialSummaryType::~IMp7JrsSequentialSummaryType()
{
// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtPropCleanup.h

}

Mp7JrsSequentialSummaryType::Mp7JrsSequentialSummaryType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSequentialSummaryType::~Mp7JrsSequentialSummaryType()
{
	Cleanup();
}

void Mp7JrsSequentialSummaryType::Init()
{
	// Init base
	m_Base = CreateSummaryType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_components = Mp7JrsSequentialSummaryType_components_CollectionPtr(); // Collection
	m_components_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_AudioVisualSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_AudioVisualSummaryLocator_Exist = false;
	m_VideoSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_VideoSummaryLocator_Exist = false;
	m_AudioSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_AudioSummaryLocator_Exist = false;
	m_VisualSummaryComponent = Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr(); // Collection
	m_AudioSummaryComponent = Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr(); // Collection
	m_TextualSummaryComponent = Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtMyPropInit.h

}

void Mp7JrsSequentialSummaryType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_components);
	// Mp7JrsFactory::DeleteObject(m_AudioVisualSummaryLocator);
	// Mp7JrsFactory::DeleteObject(m_VideoSummaryLocator);
	// Mp7JrsFactory::DeleteObject(m_AudioSummaryLocator);
	// Mp7JrsFactory::DeleteObject(m_VisualSummaryComponent);
	// Mp7JrsFactory::DeleteObject(m_AudioSummaryComponent);
	// Mp7JrsFactory::DeleteObject(m_TextualSummaryComponent);
}

void Mp7JrsSequentialSummaryType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SequentialSummaryTypePtr, since we
	// might need GetBase(), which isn't defined in ISequentialSummaryType
    const Mp7JrsPtr< Mp7JrsSequentialSummaryType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsSummaryPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSequentialSummaryType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_components);
	if (tmp->Existcomponents())
	{
		this->Setcomponents(Mp7JrsFactory::CloneObject(tmp->Getcomponents()));
	}
	else
	{
		Invalidatecomponents();
	}
	}
	if (tmp->IsValidAudioVisualSummaryLocator())
	{
		// Mp7JrsFactory::DeleteObject(m_AudioVisualSummaryLocator);
		this->SetAudioVisualSummaryLocator(Mp7JrsFactory::CloneObject(tmp->GetAudioVisualSummaryLocator()));
	}
	else
	{
		InvalidateAudioVisualSummaryLocator();
	}
	if (tmp->IsValidVideoSummaryLocator())
	{
		// Mp7JrsFactory::DeleteObject(m_VideoSummaryLocator);
		this->SetVideoSummaryLocator(Mp7JrsFactory::CloneObject(tmp->GetVideoSummaryLocator()));
	}
	else
	{
		InvalidateVideoSummaryLocator();
	}
	if (tmp->IsValidAudioSummaryLocator())
	{
		// Mp7JrsFactory::DeleteObject(m_AudioSummaryLocator);
		this->SetAudioSummaryLocator(Mp7JrsFactory::CloneObject(tmp->GetAudioSummaryLocator()));
	}
	else
	{
		InvalidateAudioSummaryLocator();
	}
		// Mp7JrsFactory::DeleteObject(m_VisualSummaryComponent);
		this->SetVisualSummaryComponent(Mp7JrsFactory::CloneObject(tmp->GetVisualSummaryComponent()));
		// Mp7JrsFactory::DeleteObject(m_AudioSummaryComponent);
		this->SetAudioSummaryComponent(Mp7JrsFactory::CloneObject(tmp->GetAudioSummaryComponent()));
		// Mp7JrsFactory::DeleteObject(m_TextualSummaryComponent);
		this->SetTextualSummaryComponent(Mp7JrsFactory::CloneObject(tmp->GetTextualSummaryComponent()));
}

Mp7JrsNodePtr Mp7JrsSequentialSummaryType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSequentialSummaryType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsSummaryType > Mp7JrsSequentialSummaryType::GetBase() const
{
	return m_Base;
}

Mp7JrsSequentialSummaryType_components_CollectionPtr Mp7JrsSequentialSummaryType::Getcomponents() const
{
	return m_components;
}

bool Mp7JrsSequentialSummaryType::Existcomponents() const
{
	return m_components_Exist;
}
void Mp7JrsSequentialSummaryType::Setcomponents(const Mp7JrsSequentialSummaryType_components_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::Setcomponents().");
	}
    m_components = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_components) m_components->SetParent(m_myself.getPointer());
	m_components_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::Invalidatecomponents()
{
	m_components_Exist = false;
}
Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSequentialSummaryType::GetAudioVisualSummaryLocator() const
{
	return m_AudioVisualSummaryLocator;
}

// Element is optional
bool Mp7JrsSequentialSummaryType::IsValidAudioVisualSummaryLocator() const
{
	return m_AudioVisualSummaryLocator_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSequentialSummaryType::GetVideoSummaryLocator() const
{
	return m_VideoSummaryLocator;
}

// Element is optional
bool Mp7JrsSequentialSummaryType::IsValidVideoSummaryLocator() const
{
	return m_VideoSummaryLocator_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSequentialSummaryType::GetAudioSummaryLocator() const
{
	return m_AudioSummaryLocator;
}

// Element is optional
bool Mp7JrsSequentialSummaryType::IsValidAudioSummaryLocator() const
{
	return m_AudioSummaryLocator_Exist;
}

Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr Mp7JrsSequentialSummaryType::GetVisualSummaryComponent() const
{
	return m_VisualSummaryComponent;
}

Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr Mp7JrsSequentialSummaryType::GetAudioSummaryComponent() const
{
	return m_AudioSummaryComponent;
}

Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr Mp7JrsSequentialSummaryType::GetTextualSummaryComponent() const
{
	return m_TextualSummaryComponent;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsSequentialSummaryType::SetAudioVisualSummaryLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetAudioVisualSummaryLocator().");
	}
	if (m_AudioVisualSummaryLocator != item || m_AudioVisualSummaryLocator_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioVisualSummaryLocator);
		m_AudioVisualSummaryLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioVisualSummaryLocator) m_AudioVisualSummaryLocator->SetParent(m_myself.getPointer());
		if (m_AudioVisualSummaryLocator && m_AudioVisualSummaryLocator->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_AudioVisualSummaryLocator->UseTypeAttribute = true;
		}
		m_AudioVisualSummaryLocator_Exist = true;
		if(m_AudioVisualSummaryLocator != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioVisualSummaryLocator->SetContentName(XMLString::transcode("AudioVisualSummaryLocator"));
		}
	// Mp7JrsFactory::DeleteObject(m_VideoSummaryLocator);
	m_VideoSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr();
	// Mp7JrsFactory::DeleteObject(m_AudioSummaryLocator);
	m_AudioSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidateAudioVisualSummaryLocator()
{
	m_AudioVisualSummaryLocator_Exist = false;
}
/* sequence */
void Mp7JrsSequentialSummaryType::SetVideoSummaryLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetVideoSummaryLocator().");
	}
	if (m_VideoSummaryLocator != item || m_VideoSummaryLocator_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_VideoSummaryLocator);
		m_VideoSummaryLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VideoSummaryLocator) m_VideoSummaryLocator->SetParent(m_myself.getPointer());
		if (m_VideoSummaryLocator && m_VideoSummaryLocator->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_VideoSummaryLocator->UseTypeAttribute = true;
		}
		m_VideoSummaryLocator_Exist = true;
		if(m_VideoSummaryLocator != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VideoSummaryLocator->SetContentName(XMLString::transcode("VideoSummaryLocator"));
		}
	// Mp7JrsFactory::DeleteObject(m_AudioVisualSummaryLocator);
	m_AudioVisualSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidateVideoSummaryLocator()
{
	m_VideoSummaryLocator_Exist = false;
}
void Mp7JrsSequentialSummaryType::SetAudioSummaryLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetAudioSummaryLocator().");
	}
	if (m_AudioSummaryLocator != item || m_AudioSummaryLocator_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioSummaryLocator);
		m_AudioSummaryLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioSummaryLocator) m_AudioSummaryLocator->SetParent(m_myself.getPointer());
		if (m_AudioSummaryLocator && m_AudioSummaryLocator->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_AudioSummaryLocator->UseTypeAttribute = true;
		}
		m_AudioSummaryLocator_Exist = true;
		if(m_AudioSummaryLocator != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioSummaryLocator->SetContentName(XMLString::transcode("AudioSummaryLocator"));
		}
	// Mp7JrsFactory::DeleteObject(m_AudioVisualSummaryLocator);
	m_AudioVisualSummaryLocator = Mp7JrsTemporalSegmentLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidateAudioSummaryLocator()
{
	m_AudioSummaryLocator_Exist = false;
}
void Mp7JrsSequentialSummaryType::SetVisualSummaryComponent(const Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetVisualSummaryComponent().");
	}
	if (m_VisualSummaryComponent != item)
	{
		// Mp7JrsFactory::DeleteObject(m_VisualSummaryComponent);
		m_VisualSummaryComponent = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VisualSummaryComponent) m_VisualSummaryComponent->SetParent(m_myself.getPointer());
		if(m_VisualSummaryComponent != Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VisualSummaryComponent->SetContentName(XMLString::transcode("VisualSummaryComponent"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::SetAudioSummaryComponent(const Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetAudioSummaryComponent().");
	}
	if (m_AudioSummaryComponent != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioSummaryComponent);
		m_AudioSummaryComponent = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioSummaryComponent) m_AudioSummaryComponent->SetParent(m_myself.getPointer());
		if(m_AudioSummaryComponent != Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioSummaryComponent->SetContentName(XMLString::transcode("AudioSummaryComponent"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::SetTextualSummaryComponent(const Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetTextualSummaryComponent().");
	}
	if (m_TextualSummaryComponent != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextualSummaryComponent);
		m_TextualSummaryComponent = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextualSummaryComponent) m_TextualSummaryComponent->SetParent(m_myself.getPointer());
		if(m_TextualSummaryComponent != Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TextualSummaryComponent->SetContentName(XMLString::transcode("TextualSummaryComponent"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsSummaryType_Name_CollectionPtr Mp7JrsSequentialSummaryType::GetName() const
{
	return GetBase()->GetName();
}

Mp7JrsUniqueIDPtr Mp7JrsSequentialSummaryType::GetSourceID() const
{
	return GetBase()->GetSourceID();
}

// Element is optional
bool Mp7JrsSequentialSummaryType::IsValidSourceID() const
{
	return GetBase()->IsValidSourceID();
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsSequentialSummaryType::GetSourceLocator() const
{
	return GetBase()->GetSourceLocator();
}

// Element is optional
bool Mp7JrsSequentialSummaryType::IsValidSourceLocator() const
{
	return GetBase()->IsValidSourceLocator();
}

Mp7JrsReferencePtr Mp7JrsSequentialSummaryType::GetSourceInformation() const
{
	return GetBase()->GetSourceInformation();
}

// Element is optional
bool Mp7JrsSequentialSummaryType::IsValidSourceInformation() const
{
	return GetBase()->IsValidSourceInformation();
}

void Mp7JrsSequentialSummaryType::SetName(const Mp7JrsSummaryType_Name_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetName().");
	}
	GetBase()->SetName(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::SetSourceID(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetSourceID().");
	}
	GetBase()->SetSourceID(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidateSourceID()
{
	GetBase()->InvalidateSourceID();
}
void Mp7JrsSequentialSummaryType::SetSourceLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetSourceLocator().");
	}
	GetBase()->SetSourceLocator(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidateSourceLocator()
{
	GetBase()->InvalidateSourceLocator();
}
void Mp7JrsSequentialSummaryType::SetSourceInformation(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetSourceInformation().");
	}
	GetBase()->SetSourceInformation(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidateSourceInformation()
{
	GetBase()->InvalidateSourceInformation();
}
XMLCh * Mp7JrsSequentialSummaryType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsSequentialSummaryType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsSequentialSummaryType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSequentialSummaryType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsSequentialSummaryType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsSequentialSummaryType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSequentialSummaryType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsSequentialSummaryType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsSequentialSummaryType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSequentialSummaryType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSequentialSummaryType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSequentialSummaryType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSequentialSummaryType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSequentialSummaryType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSequentialSummaryType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSequentialSummaryType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSequentialSummaryType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsSequentialSummaryType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSequentialSummaryType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSequentialSummaryType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("AudioVisualSummaryLocator")) == 0)
	{
		// AudioVisualSummaryLocator is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetAudioVisualSummaryLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetAudioVisualSummaryLocator(p, client);
					if((p = GetAudioVisualSummaryLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VideoSummaryLocator")) == 0)
	{
		// VideoSummaryLocator is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVideoSummaryLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVideoSummaryLocator(p, client);
					if((p = GetVideoSummaryLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AudioSummaryLocator")) == 0)
	{
		// AudioSummaryLocator is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetAudioSummaryLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetAudioSummaryLocator(p, client);
					if((p = GetAudioSummaryLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VisualSummaryComponent")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:VisualSummaryComponent is item of type VisualSummaryComponentType
		// in element collection SequentialSummaryType_VisualSummaryComponent_CollectionType
		
		context->Found = true;
		Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr coll = GetVisualSummaryComponent();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSequentialSummaryType_VisualSummaryComponent_CollectionType;
				SetVisualSummaryComponent(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:VisualSummaryComponentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsVisualSummaryComponentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AudioSummaryComponent")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:AudioSummaryComponent is item of type AudioSummaryComponentType
		// in element collection SequentialSummaryType_AudioSummaryComponent_CollectionType
		
		context->Found = true;
		Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr coll = GetAudioSummaryComponent();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSequentialSummaryType_AudioSummaryComponent_CollectionType;
				SetAudioSummaryComponent(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioSummaryComponentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsAudioSummaryComponentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TextualSummaryComponent")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:TextualSummaryComponent is item of type TextualSummaryComponentType
		// in element collection SequentialSummaryType_TextualSummaryComponent_CollectionType
		
		context->Found = true;
		Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr coll = GetTextualSummaryComponent();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSequentialSummaryType_TextualSummaryComponent_CollectionType;
				SetTextualSummaryComponent(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualSummaryComponentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualSummaryComponentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SequentialSummaryType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SequentialSummaryType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSequentialSummaryType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSequentialSummaryType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSequentialSummaryType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SequentialSummaryType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_components_Exist)
	{
	// Collection
	if(m_components != Mp7JrsSequentialSummaryType_components_CollectionPtr())
	{
		XMLCh * tmp = m_components->ToText();
		element->setAttributeNS(X(""), X("components"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_AudioVisualSummaryLocator != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioVisualSummaryLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioVisualSummaryLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_AudioVisualSummaryLocator->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_VideoSummaryLocator != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_VideoSummaryLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("VideoSummaryLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_VideoSummaryLocator->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_AudioSummaryLocator != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioSummaryLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioSummaryLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_AudioSummaryLocator->Serialize(doc, element, &elem);
	}
	if (m_VisualSummaryComponent != Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_VisualSummaryComponent->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_AudioSummaryComponent != Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_AudioSummaryComponent->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_TextualSummaryComponent != Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_TextualSummaryComponent->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSequentialSummaryType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("components")))
	{
        // Deserialize collection type
		Mp7JrsSequentialSummaryType_components_CollectionPtr tmp = CreateSequentialSummaryType_components_CollectionType;
		tmp->Parse(parent->getAttributeNS(X(""), X("components")));
        this->Setcomponents(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("components")))
	{
     
		// Deserialize collection type
        Mp7JrsSequentialSummaryType_components_CollectionPtr tmp = CreateSequentialSummaryType_components_CollectionType;
        tmp->Parse(parent->getAttribute(X("components")));
        this->Setcomponents(tmp);
        * current = parent;
    }

	// Extensionbase is Mp7JrsSummaryType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioVisualSummaryLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioVisualSummaryLocator")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioVisualSummaryLocator(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("VideoSummaryLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("VideoSummaryLocator")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetVideoSummaryLocator(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioSummaryLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioSummaryLocator")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioSummaryLocator(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }

                // FIXME: sat 2004-04-15: `continue' would be good here; we
                //finished deserializing the sequence but don't know if it
                //succeeded, so we can't.

                // continue;
	} while(false);
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("VisualSummaryComponent"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("VisualSummaryComponent")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr tmp = CreateSequentialSummaryType_VisualSummaryComponent_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetVisualSummaryComponent(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("AudioSummaryComponent"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("AudioSummaryComponent")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr tmp = CreateSequentialSummaryType_AudioSummaryComponent_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetAudioSummaryComponent(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("TextualSummaryComponent"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("TextualSummaryComponent")) == 0))
        {
            // Deserialize factory type
            Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr tmp = CreateSequentialSummaryType_TextualSummaryComponent_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTextualSummaryComponent(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSequentialSummaryType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("AudioVisualSummaryLocator")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetAudioVisualSummaryLocator(child);
  }
  if (XMLString::compareString(elementname, X("VideoSummaryLocator")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetVideoSummaryLocator(child);
  }
  if (XMLString::compareString(elementname, X("AudioSummaryLocator")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetAudioSummaryLocator(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("VisualSummaryComponent")) == 0))
  {
    Mp7JrsSequentialSummaryType_VisualSummaryComponent_CollectionPtr tmp = CreateSequentialSummaryType_VisualSummaryComponent_CollectionType;
    this->SetVisualSummaryComponent(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("AudioSummaryComponent")) == 0))
  {
    Mp7JrsSequentialSummaryType_AudioSummaryComponent_CollectionPtr tmp = CreateSequentialSummaryType_AudioSummaryComponent_CollectionType;
    this->SetAudioSummaryComponent(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("TextualSummaryComponent")) == 0))
  {
    Mp7JrsSequentialSummaryType_TextualSummaryComponent_CollectionPtr tmp = CreateSequentialSummaryType_TextualSummaryComponent_CollectionType;
    this->SetTextualSummaryComponent(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSequentialSummaryType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVisualSummaryComponent() != Mp7JrsNodePtr())
		result.Insert(GetVisualSummaryComponent());
	if (GetAudioSummaryComponent() != Mp7JrsNodePtr())
		result.Insert(GetAudioSummaryComponent());
	if (GetTextualSummaryComponent() != Mp7JrsNodePtr())
		result.Insert(GetTextualSummaryComponent());
	if (GetName() != Mp7JrsNodePtr())
		result.Insert(GetName());
	if (GetSourceID() != Mp7JrsNodePtr())
		result.Insert(GetSourceID());
	if (GetSourceLocator() != Mp7JrsNodePtr())
		result.Insert(GetSourceLocator());
	if (GetSourceInformation() != Mp7JrsNodePtr())
		result.Insert(GetSourceInformation());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetAudioVisualSummaryLocator() != Mp7JrsNodePtr())
		result.Insert(GetAudioVisualSummaryLocator());
	if (GetVideoSummaryLocator() != Mp7JrsNodePtr())
		result.Insert(GetVideoSummaryLocator());
	if (GetAudioSummaryLocator() != Mp7JrsNodePtr())
		result.Insert(GetAudioSummaryLocator());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSequentialSummaryType_ExtMethodImpl.h


