 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtImplInclude.h



#include "Mp7JrsSignalPlaneType_dim_LocalType.h"
#include "Mp7JrsSignalPlaneOriginType.h"
#include "Mp7JrsSignalPlaneType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsSignalPlaneType::IMp7JrsSignalPlaneType()
{

// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtPropInit.h

}

IMp7JrsSignalPlaneType::~IMp7JrsSignalPlaneType()
{
// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtPropCleanup.h

}

Mp7JrsSignalPlaneType::Mp7JrsSignalPlaneType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSignalPlaneType::~Mp7JrsSignalPlaneType()
{
	Cleanup();
}

void Mp7JrsSignalPlaneType::Init()
{

	// Init attributes
	m_dim = Mp7JrsSignalPlaneType_dim_LocalPtr(); // Create emtpy class ptr
	m_dim_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Origin = Mp7JrsSignalPlaneOriginPtr(); // Class
	m_Origin_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtMyPropInit.h

}

void Mp7JrsSignalPlaneType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_dim);
	// Mp7JrsFactory::DeleteObject(m_Origin);
}

void Mp7JrsSignalPlaneType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SignalPlaneTypePtr, since we
	// might need GetBase(), which isn't defined in ISignalPlaneType
    const Mp7JrsPtr< Mp7JrsSignalPlaneType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_dim);
	if (tmp->Existdim())
	{
		this->Setdim(Mp7JrsFactory::CloneObject(tmp->Getdim()));
	}
	else
	{
		Invalidatedim();
	}
	}
	if (tmp->IsValidOrigin())
	{
		// Mp7JrsFactory::DeleteObject(m_Origin);
		this->SetOrigin(Mp7JrsFactory::CloneObject(tmp->GetOrigin()));
	}
	else
	{
		InvalidateOrigin();
	}
}

Mp7JrsNodePtr Mp7JrsSignalPlaneType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsSignalPlaneType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsSignalPlaneType_dim_LocalPtr Mp7JrsSignalPlaneType::Getdim() const
{
	return m_dim;
}

bool Mp7JrsSignalPlaneType::Existdim() const
{
	return m_dim_Exist;
}
void Mp7JrsSignalPlaneType::Setdim(const Mp7JrsSignalPlaneType_dim_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSignalPlaneType::Setdim().");
	}
    m_dim = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_dim) m_dim->SetParent(m_myself.getPointer());
	m_dim_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSignalPlaneType::Invalidatedim()
{
	m_dim_Exist = false;
}
Mp7JrsSignalPlaneOriginPtr Mp7JrsSignalPlaneType::GetOrigin() const
{
	return m_Origin;
}

// Element is optional
bool Mp7JrsSignalPlaneType::IsValidOrigin() const
{
	return m_Origin_Exist;
}

void Mp7JrsSignalPlaneType::SetOrigin(const Mp7JrsSignalPlaneOriginPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSignalPlaneType::SetOrigin().");
	}
	if (m_Origin != item || m_Origin_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Origin);
		m_Origin = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Origin) m_Origin->SetParent(m_myself.getPointer());
		if (m_Origin && m_Origin->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneOriginType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Origin->UseTypeAttribute = true;
		}
		m_Origin_Exist = true;
		if(m_Origin != Mp7JrsSignalPlaneOriginPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Origin->SetContentName(XMLString::transcode("Origin"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSignalPlaneType::InvalidateOrigin()
{
	m_Origin_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsSignalPlaneType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Origin")) == 0)
	{
		// Origin is simple element SignalPlaneOriginType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetOrigin()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneOriginType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneOriginPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetOrigin(p, client);
					if((p = GetOrigin()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SignalPlaneType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SignalPlaneType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSignalPlaneType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSignalPlaneType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSignalPlaneType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SignalPlaneType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_dim_Exist)
	{
	// Class
	if (m_dim != Mp7JrsSignalPlaneType_dim_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_dim->ToText();
		element->setAttributeNS(X(""), X("dim"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Origin != Mp7JrsSignalPlaneOriginPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Origin->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Origin"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Origin->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSignalPlaneType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("dim")))
	{
        // Deserialize class type
		Mp7JrsSignalPlaneType_dim_LocalPtr tmp = CreateSignalPlaneType_dim_LocalType;
		tmp->Parse(parent->getAttributeNS(X(""), X("dim")));
        this->Setdim(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("dim")))
	{
		// Deserialize class type
        Mp7JrsSignalPlaneType_dim_LocalPtr tmp = CreateSignalPlaneType_dim_LocalType;
        tmp->Parse(parent->getAttribute(X("dim")));
        this->Setdim(tmp);
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Origin"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Origin")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneOriginType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetOrigin(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSignalPlaneType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Origin")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneOriginType;
    }
    this->SetOrigin(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSignalPlaneType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetOrigin() != Mp7JrsNodePtr())
		result.Insert(GetOrigin());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSignalPlaneType_ExtMethodImpl.h


