 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtImplInclude.h



#include "Mp7JrsAudioDType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsSilenceType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsSilenceType::IMp7JrsSilenceType()
{

// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtPropInit.h

}

IMp7JrsSilenceType::~IMp7JrsSilenceType()
{
// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtPropCleanup.h

}

Mp7JrsSilenceType::Mp7JrsSilenceType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSilenceType::~Mp7JrsSilenceType()
{
	Cleanup();
}

void Mp7JrsSilenceType::Init()
{
	// Init base
	m_Base = CreateAudioDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_confidence = CreatezeroToOneType; // Create a valid class
	m_confidence->SetContent(0.0); // Use Value initialiser (xxx2)
	m_confidence_Exist = false;
	m_minDurationRef = NULL; // String
	m_minDurationRef_Exist = false;



// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtMyPropInit.h

}

void Mp7JrsSilenceType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_confidence);
	XMLString::release(&m_minDurationRef);
}

void Mp7JrsSilenceType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SilenceTypePtr, since we
	// might need GetBase(), which isn't defined in ISilenceType
    const Mp7JrsPtr< Mp7JrsSilenceType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSilenceType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_confidence);
	if (tmp->Existconfidence())
	{
		this->Setconfidence(Mp7JrsFactory::CloneObject(tmp->Getconfidence()));
	}
	else
	{
		Invalidateconfidence();
	}
	}
	{
	XMLString::release(&m_minDurationRef);
	if (tmp->ExistminDurationRef())
	{
		this->SetminDurationRef(XMLString::replicate(tmp->GetminDurationRef()));
	}
	else
	{
		InvalidateminDurationRef();
	}
	}
}

Mp7JrsNodePtr Mp7JrsSilenceType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSilenceType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDType > Mp7JrsSilenceType::GetBase() const
{
	return m_Base;
}

Mp7JrszeroToOnePtr Mp7JrsSilenceType::Getconfidence() const
{
	if (this->Existconfidence()) {
		return m_confidence;
	} else {
		return m_confidence_Default;
	}
}

bool Mp7JrsSilenceType::Existconfidence() const
{
	return m_confidence_Exist;
}
void Mp7JrsSilenceType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSilenceType::Setconfidence().");
	}
    m_confidence = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_confidence) m_confidence->SetParent(m_myself.getPointer());
	m_confidence_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSilenceType::Invalidateconfidence()
{
	m_confidence_Exist = false;
}
XMLCh * Mp7JrsSilenceType::GetminDurationRef() const
{
	return m_minDurationRef;
}

bool Mp7JrsSilenceType::ExistminDurationRef() const
{
	return m_minDurationRef_Exist;
}
void Mp7JrsSilenceType::SetminDurationRef(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSilenceType::SetminDurationRef().");
	}
    m_minDurationRef = item;
	m_minDurationRef_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSilenceType::InvalidateminDurationRef()
{
	m_minDurationRef_Exist = false;
}
Mp7JrsintegerVectorPtr Mp7JrsSilenceType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsSilenceType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsSilenceType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSilenceType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSilenceType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsSilenceType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for SilenceType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "SilenceType");
	}
	
	return result;
}

XMLCh * Mp7JrsSilenceType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSilenceType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsSilenceType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SilenceType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_confidence_Exist)
	{
	// Class
	if (m_confidence != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_confidence->ToText();
		element->setAttributeNS(X(""), X("confidence"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_minDurationRef_Exist)
	{
	// String
	if(m_minDurationRef != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("minDurationRef"), m_minDurationRef);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsSilenceType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("confidence")))
	{
        // Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttributeNS(X(""), X("confidence")));
        this->Setconfidence(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("confidence")))
	{
		// Deserialize class type
        Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
        tmp->Parse(parent->getAttribute(X("confidence")));
        this->Setconfidence(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("minDurationRef")))
	{
        // Deserialize string type
        this->SetminDurationRef(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("minDurationRef"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("minDurationRef")))
	{
        // Deserialize string type
        this->SetminDurationRef(Mp7JrsConvert::TextToString(parent->getAttribute(X("minDurationRef"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsAudioDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSilenceType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSilenceType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtMethodImpl.h


