 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h"
#include "Mp7JrsMediaTimeType.h"
#include "Mp7JrsTextualSummaryComponentType_FreeText_CollectionType.h"
#include "Mp7JrsTextualSummaryComponentType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:FreeText

#include <assert.h>
IMp7JrsTextualSummaryComponentType::IMp7JrsTextualSummaryComponentType()
{

// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtPropInit.h

}

IMp7JrsTextualSummaryComponentType::~IMp7JrsTextualSummaryComponentType()
{
// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtPropCleanup.h

}

Mp7JrsTextualSummaryComponentType::Mp7JrsTextualSummaryComponentType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextualSummaryComponentType::~Mp7JrsTextualSummaryComponentType()
{
	Cleanup();
}

void Mp7JrsTextualSummaryComponentType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_AudioVisualSourceID = Mp7JrsUniqueIDPtr(); // Class
	m_AudioVisualSourceID_Exist = false;
	m_AudioVisualSourceLocator = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_AudioVisualSourceLocator_Exist = false;
	m_ComponentSourceTime = Mp7JrsMediaTimePtr(); // Class
	m_ComponentSourceTime_Exist = false;
	m_FreeText = Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr(); // Collection
	m_SyncTime = Mp7JrsMediaTimePtr(); // Class
	m_SyncTime_Exist = false;


// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtMyPropInit.h

}

void Mp7JrsTextualSummaryComponentType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_AudioVisualSourceID);
	// Mp7JrsFactory::DeleteObject(m_AudioVisualSourceLocator);
	// Mp7JrsFactory::DeleteObject(m_ComponentSourceTime);
	// Mp7JrsFactory::DeleteObject(m_FreeText);
	// Mp7JrsFactory::DeleteObject(m_SyncTime);
}

void Mp7JrsTextualSummaryComponentType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextualSummaryComponentTypePtr, since we
	// might need GetBase(), which isn't defined in ITextualSummaryComponentType
    const Mp7JrsPtr< Mp7JrsTextualSummaryComponentType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsTextualSummaryComponentType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidAudioVisualSourceID())
	{
		// Mp7JrsFactory::DeleteObject(m_AudioVisualSourceID);
		this->SetAudioVisualSourceID(Mp7JrsFactory::CloneObject(tmp->GetAudioVisualSourceID()));
	}
	else
	{
		InvalidateAudioVisualSourceID();
	}
	if (tmp->IsValidAudioVisualSourceLocator())
	{
		// Mp7JrsFactory::DeleteObject(m_AudioVisualSourceLocator);
		this->SetAudioVisualSourceLocator(Mp7JrsFactory::CloneObject(tmp->GetAudioVisualSourceLocator()));
	}
	else
	{
		InvalidateAudioVisualSourceLocator();
	}
	if (tmp->IsValidComponentSourceTime())
	{
		// Mp7JrsFactory::DeleteObject(m_ComponentSourceTime);
		this->SetComponentSourceTime(Mp7JrsFactory::CloneObject(tmp->GetComponentSourceTime()));
	}
	else
	{
		InvalidateComponentSourceTime();
	}
		// Mp7JrsFactory::DeleteObject(m_FreeText);
		this->SetFreeText(Mp7JrsFactory::CloneObject(tmp->GetFreeText()));
	if (tmp->IsValidSyncTime())
	{
		// Mp7JrsFactory::DeleteObject(m_SyncTime);
		this->SetSyncTime(Mp7JrsFactory::CloneObject(tmp->GetSyncTime()));
	}
	else
	{
		InvalidateSyncTime();
	}
}

Mp7JrsNodePtr Mp7JrsTextualSummaryComponentType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsTextualSummaryComponentType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsTextualSummaryComponentType::GetBase() const
{
	return m_Base;
}

Mp7JrsUniqueIDPtr Mp7JrsTextualSummaryComponentType::GetAudioVisualSourceID() const
{
	return m_AudioVisualSourceID;
}

// Element is optional
bool Mp7JrsTextualSummaryComponentType::IsValidAudioVisualSourceID() const
{
	return m_AudioVisualSourceID_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsTextualSummaryComponentType::GetAudioVisualSourceLocator() const
{
	return m_AudioVisualSourceLocator;
}

// Element is optional
bool Mp7JrsTextualSummaryComponentType::IsValidAudioVisualSourceLocator() const
{
	return m_AudioVisualSourceLocator_Exist;
}

Mp7JrsMediaTimePtr Mp7JrsTextualSummaryComponentType::GetComponentSourceTime() const
{
	return m_ComponentSourceTime;
}

// Element is optional
bool Mp7JrsTextualSummaryComponentType::IsValidComponentSourceTime() const
{
	return m_ComponentSourceTime_Exist;
}

Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr Mp7JrsTextualSummaryComponentType::GetFreeText() const
{
	return m_FreeText;
}

Mp7JrsMediaTimePtr Mp7JrsTextualSummaryComponentType::GetSyncTime() const
{
	return m_SyncTime;
}

// Element is optional
bool Mp7JrsTextualSummaryComponentType::IsValidSyncTime() const
{
	return m_SyncTime_Exist;
}

void Mp7JrsTextualSummaryComponentType::SetAudioVisualSourceID(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualSummaryComponentType::SetAudioVisualSourceID().");
	}
	if (m_AudioVisualSourceID != item || m_AudioVisualSourceID_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioVisualSourceID);
		m_AudioVisualSourceID = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioVisualSourceID) m_AudioVisualSourceID->SetParent(m_myself.getPointer());
		if (m_AudioVisualSourceID && m_AudioVisualSourceID->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_AudioVisualSourceID->UseTypeAttribute = true;
		}
		m_AudioVisualSourceID_Exist = true;
		if(m_AudioVisualSourceID != Mp7JrsUniqueIDPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioVisualSourceID->SetContentName(XMLString::transcode("AudioVisualSourceID"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualSummaryComponentType::InvalidateAudioVisualSourceID()
{
	m_AudioVisualSourceID_Exist = false;
}
void Mp7JrsTextualSummaryComponentType::SetAudioVisualSourceLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualSummaryComponentType::SetAudioVisualSourceLocator().");
	}
	if (m_AudioVisualSourceLocator != item || m_AudioVisualSourceLocator_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioVisualSourceLocator);
		m_AudioVisualSourceLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioVisualSourceLocator) m_AudioVisualSourceLocator->SetParent(m_myself.getPointer());
		if (m_AudioVisualSourceLocator && m_AudioVisualSourceLocator->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_AudioVisualSourceLocator->UseTypeAttribute = true;
		}
		m_AudioVisualSourceLocator_Exist = true;
		if(m_AudioVisualSourceLocator != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioVisualSourceLocator->SetContentName(XMLString::transcode("AudioVisualSourceLocator"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualSummaryComponentType::InvalidateAudioVisualSourceLocator()
{
	m_AudioVisualSourceLocator_Exist = false;
}
void Mp7JrsTextualSummaryComponentType::SetComponentSourceTime(const Mp7JrsMediaTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualSummaryComponentType::SetComponentSourceTime().");
	}
	if (m_ComponentSourceTime != item || m_ComponentSourceTime_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ComponentSourceTime);
		m_ComponentSourceTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ComponentSourceTime) m_ComponentSourceTime->SetParent(m_myself.getPointer());
		if (m_ComponentSourceTime && m_ComponentSourceTime->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ComponentSourceTime->UseTypeAttribute = true;
		}
		m_ComponentSourceTime_Exist = true;
		if(m_ComponentSourceTime != Mp7JrsMediaTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ComponentSourceTime->SetContentName(XMLString::transcode("ComponentSourceTime"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualSummaryComponentType::InvalidateComponentSourceTime()
{
	m_ComponentSourceTime_Exist = false;
}
void Mp7JrsTextualSummaryComponentType::SetFreeText(const Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualSummaryComponentType::SetFreeText().");
	}
	if (m_FreeText != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FreeText);
		m_FreeText = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FreeText) m_FreeText->SetParent(m_myself.getPointer());
		if(m_FreeText != Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FreeText->SetContentName(XMLString::transcode("FreeText"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualSummaryComponentType::SetSyncTime(const Mp7JrsMediaTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualSummaryComponentType::SetSyncTime().");
	}
	if (m_SyncTime != item || m_SyncTime_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SyncTime);
		m_SyncTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SyncTime) m_SyncTime->SetParent(m_myself.getPointer());
		if (m_SyncTime && m_SyncTime->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SyncTime->UseTypeAttribute = true;
		}
		m_SyncTime_Exist = true;
		if(m_SyncTime != Mp7JrsMediaTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SyncTime->SetContentName(XMLString::transcode("SyncTime"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualSummaryComponentType::InvalidateSyncTime()
{
	m_SyncTime_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsTextualSummaryComponentType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("AudioVisualSourceID")) == 0)
	{
		// AudioVisualSourceID is simple element UniqueIDType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetAudioVisualSourceID()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUniqueIDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetAudioVisualSourceID(p, client);
					if((p = GetAudioVisualSourceID()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AudioVisualSourceLocator")) == 0)
	{
		// AudioVisualSourceLocator is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetAudioVisualSourceLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetAudioVisualSourceLocator(p, client);
					if((p = GetAudioVisualSourceLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ComponentSourceTime")) == 0)
	{
		// ComponentSourceTime is simple element MediaTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetComponentSourceTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetComponentSourceTime(p, client);
					if((p = GetComponentSourceTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FreeText")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:FreeText is item of type TextualType
		// in element collection TextualSummaryComponentType_FreeText_CollectionType
		
		context->Found = true;
		Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr coll = GetFreeText();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateTextualSummaryComponentType_FreeText_CollectionType;
				SetFreeText(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SyncTime")) == 0)
	{
		// SyncTime is simple element MediaTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSyncTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSyncTime(p, client);
					if((p = GetSyncTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TextualSummaryComponentType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TextualSummaryComponentType");
	}		
	
	return result;
}

XMLCh * Mp7JrsTextualSummaryComponentType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsTextualSummaryComponentType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsTextualSummaryComponentType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TextualSummaryComponentType"));
	// Element serialization:
	// Class
	
	if (m_AudioVisualSourceID != Mp7JrsUniqueIDPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioVisualSourceID->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioVisualSourceID"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_AudioVisualSourceID->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_AudioVisualSourceLocator != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioVisualSourceLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioVisualSourceLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_AudioVisualSourceLocator->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ComponentSourceTime != Mp7JrsMediaTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ComponentSourceTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ComponentSourceTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ComponentSourceTime->Serialize(doc, element, &elem);
	}
	if (m_FreeText != Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_FreeText->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_SyncTime != Mp7JrsMediaTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SyncTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SyncTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SyncTime->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsTextualSummaryComponentType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioVisualSourceID"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioVisualSourceID")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateUniqueIDType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioVisualSourceID(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioVisualSourceLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioVisualSourceLocator")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioVisualSourceLocator(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ComponentSourceTime"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ComponentSourceTime")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaTimeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetComponentSourceTime(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("FreeText"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("FreeText")) == 0))
        {
            // Deserialize factory type
            Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr tmp = CreateTextualSummaryComponentType_FreeText_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetFreeText(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SyncTime"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SyncTime")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMediaTimeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSyncTime(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextualSummaryComponentType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("AudioVisualSourceID")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateUniqueIDType;
    }
    this->SetAudioVisualSourceID(child);
  }
  if (XMLString::compareString(elementname, X("AudioVisualSourceLocator")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetAudioVisualSourceLocator(child);
  }
  if (XMLString::compareString(elementname, X("ComponentSourceTime")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaTimeType;
    }
    this->SetComponentSourceTime(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("FreeText")) == 0))
  {
    Mp7JrsTextualSummaryComponentType_FreeText_CollectionPtr tmp = CreateTextualSummaryComponentType_FreeText_CollectionType;
    this->SetFreeText(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("SyncTime")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMediaTimeType;
    }
    this->SetSyncTime(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextualSummaryComponentType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetAudioVisualSourceID() != Mp7JrsNodePtr())
		result.Insert(GetAudioVisualSourceID());
	if (GetAudioVisualSourceLocator() != Mp7JrsNodePtr())
		result.Insert(GetAudioVisualSourceLocator());
	if (GetComponentSourceTime() != Mp7JrsNodePtr())
		result.Insert(GetComponentSourceTime());
	if (GetFreeText() != Mp7JrsNodePtr())
		result.Insert(GetFreeText());
	if (GetSyncTime() != Mp7JrsNodePtr())
		result.Insert(GetSyncTime());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextualSummaryComponentType_ExtMethodImpl.h


