 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsTextureBrowsingType_CollectionType.h"
#include "Mp7JrsTextureBrowsingType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsTextureBrowsingType::IMp7JrsTextureBrowsingType()
{

// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtPropInit.h

}

IMp7JrsTextureBrowsingType::~IMp7JrsTextureBrowsingType()
{
// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtPropCleanup.h

}

Mp7JrsTextureBrowsingType::Mp7JrsTextureBrowsingType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextureBrowsingType::~Mp7JrsTextureBrowsingType()
{
	Cleanup();
}

void Mp7JrsTextureBrowsingType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Regularity = Mp7JrsTextureBrowsingType_Regularity_LocalType::UninitializedEnumeration; // Enumeration
	m_TextureBrowsingType_LocalType = Mp7JrsTextureBrowsingType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtMyPropInit.h

}

void Mp7JrsTextureBrowsingType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_TextureBrowsingType_LocalType);
}

void Mp7JrsTextureBrowsingType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextureBrowsingTypePtr, since we
	// might need GetBase(), which isn't defined in ITextureBrowsingType
    const Mp7JrsPtr< Mp7JrsTextureBrowsingType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsTextureBrowsingType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		this->SetRegularity(tmp->GetRegularity());
		// Mp7JrsFactory::DeleteObject(m_TextureBrowsingType_LocalType);
		this->SetTextureBrowsingType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetTextureBrowsingType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsTextureBrowsingType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsTextureBrowsingType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsTextureBrowsingType::GetBase() const
{
	return m_Base;
}

Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration Mp7JrsTextureBrowsingType::GetRegularity() const
{
	return m_Regularity;
}

Mp7JrsTextureBrowsingType_CollectionPtr Mp7JrsTextureBrowsingType::GetTextureBrowsingType_LocalType() const
{
	return m_TextureBrowsingType_LocalType;
}

void Mp7JrsTextureBrowsingType::SetRegularity(Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextureBrowsingType::SetRegularity().");
	}
	if (m_Regularity != item)
	{
		// nothing to free here, hopefully
		m_Regularity = item;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextureBrowsingType::SetTextureBrowsingType_LocalType(const Mp7JrsTextureBrowsingType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextureBrowsingType::SetTextureBrowsingType_LocalType().");
	}
	if (m_TextureBrowsingType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextureBrowsingType_LocalType);
		m_TextureBrowsingType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextureBrowsingType_LocalType) m_TextureBrowsingType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsTextureBrowsingType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for TextureBrowsingType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "TextureBrowsingType");
	}
	
	return result;
}

XMLCh * Mp7JrsTextureBrowsingType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsTextureBrowsingType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsTextureBrowsingType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TextureBrowsingType"));
	// Element serialization:
	if(m_Regularity != Mp7JrsTextureBrowsingType_Regularity_LocalType::UninitializedEnumeration) // Enumeration
	{
		XMLCh * tmp = Mp7JrsTextureBrowsingType_Regularity_LocalType::ToText(m_Regularity);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("Regularity"), false);
		XMLString::release(&tmp);
	}
	if (m_TextureBrowsingType_LocalType != Mp7JrsTextureBrowsingType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_TextureBrowsingType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsTextureBrowsingType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize enumeration element
	if(Mp7JrsUtil::HasNodeName(parent, X("Regularity"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Regularity")) == 0))
        {
            Mp7JrsTextureBrowsingType_Regularity_LocalType::Enumeration tmp = Mp7JrsTextureBrowsingType_Regularity_LocalType::Parse(Mp7JrsUtil::GetElementText(parent));
            if(tmp == Mp7JrsTextureBrowsingType_Regularity_LocalType::UninitializedEnumeration)
            {
                    return false;
            }
            this->SetRegularity(tmp);
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Sequence Collection
        if((parent != NULL) && (
            (XMLString::compareString(parent->getNodeName()
            , X("urn:mpeg:mpeg7:schema:2004:Direction")) == 0)
                ||
            (XMLString::compareString(parent->getNodeName()
            , X("urn:mpeg:mpeg7:schema:2004:Scale")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsTextureBrowsingType_CollectionPtr tmp = CreateTextureBrowsingType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTextureBrowsingType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextureBrowsingType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
    // local type, so no need to check explicitly for a name (BAW 05 04 2004)
  {
    Mp7JrsTextureBrowsingType_CollectionPtr tmp = CreateTextureBrowsingType_CollectionType;
    this->SetTextureBrowsingType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextureBrowsingType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTextureBrowsingType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetTextureBrowsingType_LocalType());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextureBrowsingType_ExtMethodImpl.h


