 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTimeType_ExtImplInclude.h



#include "Mp7JrstimePointType.h"
#include "Mp7JrsRelTimePointType.h"
#include "Mp7JrsRelIncrTimePointType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsIncrDurationType.h"
#include "Mp7JrsTimeType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsTimeType::IMp7JrsTimeType()
{

// no includefile for extension defined 
// file Mp7JrsTimeType_ExtPropInit.h

}

IMp7JrsTimeType::~IMp7JrsTimeType()
{
// no includefile for extension defined 
// file Mp7JrsTimeType_ExtPropCleanup.h

}

Mp7JrsTimeType::Mp7JrsTimeType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTimeType::~Mp7JrsTimeType()
{
	Cleanup();
}

void Mp7JrsTimeType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_TimePoint = Mp7JrstimePointPtr(); // Pattern
	m_RelTimePoint = Mp7JrsRelTimePointPtr(); // Class
	m_RelIncrTimePoint = Mp7JrsRelIncrTimePointPtr(); // Class with content 
	m_Duration = Mp7JrsdurationPtr(); // Pattern
	m_IncrDuration = Mp7JrsIncrDurationPtr(); // Class with content 


// no includefile for extension defined 
// file Mp7JrsTimeType_ExtMyPropInit.h

}

void Mp7JrsTimeType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTimeType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_TimePoint);
	// Mp7JrsFactory::DeleteObject(m_RelTimePoint);
	// Mp7JrsFactory::DeleteObject(m_RelIncrTimePoint);
	// Mp7JrsFactory::DeleteObject(m_Duration);
	// Mp7JrsFactory::DeleteObject(m_IncrDuration);
}

void Mp7JrsTimeType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TimeTypePtr, since we
	// might need GetBase(), which isn't defined in ITimeType
    const Mp7JrsPtr< Mp7JrsTimeType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_TimePoint);
		this->SetTimePoint(Mp7JrsFactory::CloneObject(tmp->GetTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_RelTimePoint);
		this->SetRelTimePoint(Mp7JrsFactory::CloneObject(tmp->GetRelTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_RelIncrTimePoint);
		this->SetRelIncrTimePoint(Mp7JrsFactory::CloneObject(tmp->GetRelIncrTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_Duration);
		this->SetDuration(Mp7JrsFactory::CloneObject(tmp->GetDuration()));
		// Mp7JrsFactory::DeleteObject(m_IncrDuration);
		this->SetIncrDuration(Mp7JrsFactory::CloneObject(tmp->GetIncrDuration()));
}

Mp7JrsNodePtr Mp7JrsTimeType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTimeType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrstimePointPtr Mp7JrsTimeType::GetTimePoint() const
{
	return m_TimePoint;
}

Mp7JrsRelTimePointPtr Mp7JrsTimeType::GetRelTimePoint() const
{
	return m_RelTimePoint;
}

Mp7JrsRelIncrTimePointPtr Mp7JrsTimeType::GetRelIncrTimePoint() const
{
	return m_RelIncrTimePoint;
}

Mp7JrsdurationPtr Mp7JrsTimeType::GetDuration() const
{
	return m_Duration;
}

Mp7JrsIncrDurationPtr Mp7JrsTimeType::GetIncrDuration() const
{
	return m_IncrDuration;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsTimeType::SetTimePoint(const Mp7JrstimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTimeType::SetTimePoint().");
	}
	if (m_TimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TimePoint);
		m_TimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TimePoint) m_TimePoint->SetParent(m_myself.getPointer());
		if (m_TimePoint && m_TimePoint->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:timePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TimePoint->UseTypeAttribute = true;
		}
		if(m_TimePoint != Mp7JrstimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TimePoint->SetContentName(XMLString::transcode("TimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_RelTimePoint);
	m_RelTimePoint = Mp7JrsRelTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_RelIncrTimePoint);
	m_RelIncrTimePoint = Mp7JrsRelIncrTimePointPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTimeType::SetRelTimePoint(const Mp7JrsRelTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTimeType::SetRelTimePoint().");
	}
	if (m_RelTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RelTimePoint);
		m_RelTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RelTimePoint) m_RelTimePoint->SetParent(m_myself.getPointer());
		if (m_RelTimePoint && m_RelTimePoint->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_RelTimePoint->UseTypeAttribute = true;
		}
		if(m_RelTimePoint != Mp7JrsRelTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RelTimePoint->SetContentName(XMLString::transcode("RelTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_TimePoint);
	m_TimePoint = Mp7JrstimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_RelIncrTimePoint);
	m_RelIncrTimePoint = Mp7JrsRelIncrTimePointPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTimeType::SetRelIncrTimePoint(const Mp7JrsRelIncrTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTimeType::SetRelIncrTimePoint().");
	}
	if (m_RelIncrTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RelIncrTimePoint);
		m_RelIncrTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RelIncrTimePoint) m_RelIncrTimePoint->SetParent(m_myself.getPointer());
		if (m_RelIncrTimePoint && m_RelIncrTimePoint->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelIncrTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_RelIncrTimePoint->UseTypeAttribute = true;
		}
		if(m_RelIncrTimePoint != Mp7JrsRelIncrTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RelIncrTimePoint->SetContentName(XMLString::transcode("RelIncrTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_TimePoint);
	m_TimePoint = Mp7JrstimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_RelTimePoint);
	m_RelTimePoint = Mp7JrsRelTimePointPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsTimeType::SetDuration(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTimeType::SetDuration().");
	}
	if (m_Duration != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Duration);
		m_Duration = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Duration) m_Duration->SetParent(m_myself.getPointer());
		if (m_Duration && m_Duration->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:durationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Duration->UseTypeAttribute = true;
		}
		if(m_Duration != Mp7JrsdurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Duration->SetContentName(XMLString::transcode("Duration"));
		}
	// Mp7JrsFactory::DeleteObject(m_IncrDuration);
	m_IncrDuration = Mp7JrsIncrDurationPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTimeType::SetIncrDuration(const Mp7JrsIncrDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTimeType::SetIncrDuration().");
	}
	if (m_IncrDuration != item)
	{
		// Mp7JrsFactory::DeleteObject(m_IncrDuration);
		m_IncrDuration = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_IncrDuration) m_IncrDuration->SetParent(m_myself.getPointer());
		if (m_IncrDuration && m_IncrDuration->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:IncrDurationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_IncrDuration->UseTypeAttribute = true;
		}
		if(m_IncrDuration != Mp7JrsIncrDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_IncrDuration->SetContentName(XMLString::transcode("IncrDuration"));
		}
	// Mp7JrsFactory::DeleteObject(m_Duration);
	m_Duration = Mp7JrsdurationPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsTimeType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("TimePoint")) == 0)
	{
		// TimePoint is simple element timePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:timePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrstimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTimePoint(p, client);
					if((p = GetTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RelTimePoint")) == 0)
	{
		// RelTimePoint is simple element RelTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRelTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRelTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRelTimePoint(p, client);
					if((p = GetRelTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RelIncrTimePoint")) == 0)
	{
		// RelIncrTimePoint is simple element RelIncrTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRelIncrTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RelIncrTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRelIncrTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRelIncrTimePoint(p, client);
					if((p = GetRelIncrTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Duration")) == 0)
	{
		// Duration is simple element durationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDuration()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:durationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsdurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDuration(p, client);
					if((p = GetDuration()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("IncrDuration")) == 0)
	{
		// IncrDuration is simple element IncrDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetIncrDuration()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:IncrDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsIncrDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetIncrDuration(p, client);
					if((p = GetIncrDuration()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TimeType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TimeType");
	}		
	
	return result;
}

XMLCh * Mp7JrsTimeType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTimeType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsTimeType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TimeType"));
	// Element serialization:
	if(m_TimePoint != Mp7JrstimePointPtr()) // Pattern FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Pattern
		// CON contentname already set in element setter -- not needed here
//		XMLCh * contentname = XMLString::transcode("TimePoint");
//		m_TimePoint->SetContentName(contentname);
		m_TimePoint->UseTypeAttribute = this->UseTypeAttribute;
		m_TimePoint->Serialize(doc, element);
	}
	// Class
	
	if (m_RelTimePoint != Mp7JrsRelTimePointPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_RelTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("RelTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_RelTimePoint->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_RelIncrTimePoint != Mp7JrsRelIncrTimePointPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_RelIncrTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("RelIncrTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_RelIncrTimePoint->Serialize(doc, element, &elem);
	}
	if(m_Duration != Mp7JrsdurationPtr()) // Pattern FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Pattern
		// CON contentname already set in element setter -- not needed here
//		XMLCh * contentname = XMLString::transcode("Duration");
//		m_Duration->SetContentName(contentname);
		m_Duration->UseTypeAttribute = this->UseTypeAttribute;
		m_Duration->Serialize(doc, element);
	}
	// Class with content		
	
	if (m_IncrDuration != Mp7JrsIncrDurationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_IncrDuration->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("IncrDuration"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_IncrDuration->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsTimeType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
    {
        // Deserialize pattern element
	if(Mp7JrsUtil::HasNodeName(parent, X("TimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//	        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TimePoint")) == 0))
        {
            Mp7JrstimePointPtr tmp = CreatetimePointType;
            tmp->Deserialize(doc, parent, current);
            this->SetTimePoint(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("RelTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("RelTimePoint")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRelTimePointType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRelTimePoint(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("RelIncrTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("RelIncrTimePoint")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRelIncrTimePointType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRelIncrTimePoint(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
	do // Deserialize choice just one time!
    {
        // Deserialize pattern element
	if(Mp7JrsUtil::HasNodeName(parent, X("Duration"), X("urn:mpeg:mpeg7:schema:2004")))
//	        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Duration")) == 0))
        {
            Mp7JrsdurationPtr tmp = CreatedurationType;
            tmp->Deserialize(doc, parent, current);
            this->SetDuration(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("IncrDuration"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("IncrDuration")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateIncrDurationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetIncrDuration(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsTimeType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTimeType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("TimePoint")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatetimePointType;
    }
    this->SetTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("RelTimePoint")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRelTimePointType;
    }
    this->SetRelTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("RelIncrTimePoint")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRelIncrTimePointType;
    }
    this->SetRelIncrTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("Duration")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatedurationType;
    }
    this->SetDuration(child);
  }
  if (XMLString::compareString(elementname, X("IncrDuration")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateIncrDurationType;
    }
    this->SetIncrDuration(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTimeType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetTimePoint());
	if (GetRelTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetRelTimePoint());
	if (GetRelIncrTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetRelIncrTimePoint());
	if (GetDuration() != Mp7JrsNodePtr())
		result.Insert(GetDuration());
	if (GetIncrDuration() != Mp7JrsNodePtr())
		result.Insert(GetIncrDuration());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTimeType_ExtMethodImpl.h


