 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtImplInclude.h



#include "Mp7JrsVisualDSType.h"
#include "Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionType.h"
#include "Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionType.h"
#include "Mp7JrsMotionActivityType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsVideoSegmentFeatureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsVisualTimeSeriesType.h" // Element collection urn:mpeg:mpeg7:schema:2004:TemporalTransition
#include "Mp7JrsGopGofFeature.h" // Element collection urn:mpeg:mpeg7:schema:2004:RepresentativeFeature

#include <assert.h>
IMp7JrsVideoSegmentFeatureType::IMp7JrsVideoSegmentFeatureType()
{

// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtPropInit.h

}

IMp7JrsVideoSegmentFeatureType::~IMp7JrsVideoSegmentFeatureType()
{
// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtPropCleanup.h

}

Mp7JrsVideoSegmentFeatureType::Mp7JrsVideoSegmentFeatureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsVideoSegmentFeatureType::~Mp7JrsVideoSegmentFeatureType()
{
	Cleanup();
}

void Mp7JrsVideoSegmentFeatureType::Init()
{
	// Init base
	m_Base = CreateVisualDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_TemporalTransition = Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr(); // Collection
	m_RepresentativeFeature = Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr(); // Collection
	m_MotionActivity = Mp7JrsMotionActivityPtr(); // Class
	m_MotionActivity_Exist = false;


// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtMyPropInit.h

}

void Mp7JrsVideoSegmentFeatureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_TemporalTransition);
	// Mp7JrsFactory::DeleteObject(m_RepresentativeFeature);
	// Mp7JrsFactory::DeleteObject(m_MotionActivity);
}

void Mp7JrsVideoSegmentFeatureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use VideoSegmentFeatureTypePtr, since we
	// might need GetBase(), which isn't defined in IVideoSegmentFeatureType
    const Mp7JrsPtr< Mp7JrsVideoSegmentFeatureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsVideoSegmentFeatureType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_TemporalTransition);
		this->SetTemporalTransition(Mp7JrsFactory::CloneObject(tmp->GetTemporalTransition()));
		// Mp7JrsFactory::DeleteObject(m_RepresentativeFeature);
		this->SetRepresentativeFeature(Mp7JrsFactory::CloneObject(tmp->GetRepresentativeFeature()));
	if (tmp->IsValidMotionActivity())
	{
		// Mp7JrsFactory::DeleteObject(m_MotionActivity);
		this->SetMotionActivity(Mp7JrsFactory::CloneObject(tmp->GetMotionActivity()));
	}
	else
	{
		InvalidateMotionActivity();
	}
}

Mp7JrsNodePtr Mp7JrsVideoSegmentFeatureType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsVideoSegmentFeatureType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDSType > Mp7JrsVideoSegmentFeatureType::GetBase() const
{
	return m_Base;
}

Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr Mp7JrsVideoSegmentFeatureType::GetTemporalTransition() const
{
	return m_TemporalTransition;
}

Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr Mp7JrsVideoSegmentFeatureType::GetRepresentativeFeature() const
{
	return m_RepresentativeFeature;
}

Mp7JrsMotionActivityPtr Mp7JrsVideoSegmentFeatureType::GetMotionActivity() const
{
	return m_MotionActivity;
}

// Element is optional
bool Mp7JrsVideoSegmentFeatureType::IsValidMotionActivity() const
{
	return m_MotionActivity_Exist;
}

void Mp7JrsVideoSegmentFeatureType::SetTemporalTransition(const Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SetTemporalTransition().");
	}
	if (m_TemporalTransition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalTransition);
		m_TemporalTransition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TemporalTransition) m_TemporalTransition->SetParent(m_myself.getPointer());
		if(m_TemporalTransition != Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TemporalTransition->SetContentName(XMLString::transcode("TemporalTransition"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::SetRepresentativeFeature(const Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SetRepresentativeFeature().");
	}
	if (m_RepresentativeFeature != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RepresentativeFeature);
		m_RepresentativeFeature = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RepresentativeFeature) m_RepresentativeFeature->SetParent(m_myself.getPointer());
		if(m_RepresentativeFeature != Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RepresentativeFeature->SetContentName(XMLString::transcode("RepresentativeFeature"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::SetMotionActivity(const Mp7JrsMotionActivityPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SetMotionActivity().");
	}
	if (m_MotionActivity != item || m_MotionActivity_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_MotionActivity);
		m_MotionActivity = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MotionActivity) m_MotionActivity->SetParent(m_myself.getPointer());
		if (m_MotionActivity && m_MotionActivity->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MotionActivity->UseTypeAttribute = true;
		}
		m_MotionActivity_Exist = true;
		if(m_MotionActivity != Mp7JrsMotionActivityPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MotionActivity->SetContentName(XMLString::transcode("MotionActivity"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::InvalidateMotionActivity()
{
	m_MotionActivity_Exist = false;
}
XMLCh * Mp7JrsVideoSegmentFeatureType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsVideoSegmentFeatureType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsVideoSegmentFeatureType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsVideoSegmentFeatureType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsVideoSegmentFeatureType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsVideoSegmentFeatureType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsVideoSegmentFeatureType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsVideoSegmentFeatureType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsVideoSegmentFeatureType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsVideoSegmentFeatureType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsVideoSegmentFeatureType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsVideoSegmentFeatureType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsVideoSegmentFeatureType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsVideoSegmentFeatureType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsVideoSegmentFeatureType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVideoSegmentFeatureType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsVideoSegmentFeatureType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsVideoSegmentFeatureType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVideoSegmentFeatureType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsVideoSegmentFeatureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("TemporalTransition")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:TemporalTransition is item of abstract type VisualTimeSeriesType
		// in element collection VideoSegmentFeatureType_TemporalTransition_CollectionType
		
		context->Found = true;
		Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr coll = GetTemporalTransition();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateVideoSegmentFeatureType_TemporalTransition_CollectionType;
				SetTemporalTransition(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsVisualTimeSeriesPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RepresentativeFeature")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:RepresentativeFeature is item of type GopGofFeature
		// in element collection VideoSegmentFeatureType_RepresentativeFeature_CollectionType
		
		context->Found = true;
		Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr coll = GetRepresentativeFeature();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateVideoSegmentFeatureType_RepresentativeFeature_CollectionType;
				SetRepresentativeFeature(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GopGofFeature")))) != empty)
			{
				// Is type allowed
				Mp7JrsGopGofFeaturePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MotionActivity")) == 0)
	{
		// MotionActivity is simple element MotionActivityType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMotionActivity()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMotionActivityPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMotionActivity(p, client);
					if((p = GetMotionActivity()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for VideoSegmentFeatureType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "VideoSegmentFeatureType");
	}		
	
	return result;
}

XMLCh * Mp7JrsVideoSegmentFeatureType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsVideoSegmentFeatureType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsVideoSegmentFeatureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("VideoSegmentFeatureType"));
	// Element serialization:
	if (m_TemporalTransition != Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_TemporalTransition->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_RepresentativeFeature != Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_RepresentativeFeature->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_MotionActivity != Mp7JrsMotionActivityPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MotionActivity->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
		
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MotionActivity"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MotionActivity->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsVideoSegmentFeatureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("TemporalTransition"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("TemporalTransition")) == 0))
        {
            // Deserialize factory type
            Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr tmp = CreateVideoSegmentFeatureType_TemporalTransition_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTemporalTransition(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("RepresentativeFeature"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("RepresentativeFeature")) == 0))
        {
            // Deserialize factory type
            Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr tmp = CreateVideoSegmentFeatureType_RepresentativeFeature_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetRepresentativeFeature(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MotionActivity"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MotionActivity")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMotionActivityType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetMotionActivity(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsVideoSegmentFeatureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("TemporalTransition")) == 0))
  {
    Mp7JrsVideoSegmentFeatureType_TemporalTransition_CollectionPtr tmp = CreateVideoSegmentFeatureType_TemporalTransition_CollectionType;
    this->SetTemporalTransition(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("RepresentativeFeature")) == 0))
  {
    Mp7JrsVideoSegmentFeatureType_RepresentativeFeature_CollectionPtr tmp = CreateVideoSegmentFeatureType_RepresentativeFeature_CollectionType;
    this->SetRepresentativeFeature(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("MotionActivity")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMotionActivityType;
    }
    this->SetMotionActivity(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsVideoSegmentFeatureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTemporalTransition() != Mp7JrsNodePtr())
		result.Insert(GetTemporalTransition());
	if (GetRepresentativeFeature() != Mp7JrsNodePtr())
		result.Insert(GetRepresentativeFeature());
	if (GetMotionActivity() != Mp7JrsNodePtr())
		result.Insert(GetMotionActivity());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsVideoSegmentFeatureType_ExtMethodImpl.h


