 

// Core library class
// based on ValCollectionType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


#include <stdio.h>
#include "Mp7JrsbeatType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtImplInclude.h


Mp7JrsbeatType::Mp7JrsbeatType()
{
	m_Item = new Mp7JrsValueVectorOf <int>(0);
	
// no includefile for extension defined 
// file Mp7JrsbeatType_ExtPropInit.h


}

Mp7JrsbeatType::~Mp7JrsbeatType()
{
// no includefile for extension defined 
// file Mp7JrsbeatType_ExtPropCleanup.h


	delete m_Item;
}

void Mp7JrsbeatType::DeepCopy(const Mp7JrsNodePtr & original)
{
	const Mp7JrsbeatPtr tmp = original;
	if (tmp == NULL) return;  // EXIT: wrong argument type

	m_Item->removeAllElements();
	for(unsigned int i = 0; i < tmp->size(); i++)
	{
		m_Item->addElement(tmp->elementAt(i));
	}
}

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtMethodImpl.h


void Mp7JrsbeatType::Initialize(unsigned int maxElems)
{
	m_Item->ensureExtraCapacity(maxElems);
}

void Mp7JrsbeatType::addElement(const int & toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbeatType::addElement().");
	}
	m_Item->addElement(toAdd);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsbeatType::setElementAt(const int & toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbeatType::setElementAt().");
	}
	m_Item->setElementAt(toSet, setAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsbeatType::insertElementAt(const int & toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbeatType::insertElementAt().");
	}
	m_Item->insertElementAt(toInsert, insertAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsbeatType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbeatType::removeAllElements().");
	}
	m_Item->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsbeatType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbeatType::removeElementAt().");
	}
	m_Item->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsbeatType::containsElement(const int &toCheck)
{
	return m_Item->containsElement(toCheck);
}

unsigned int Mp7JrsbeatType::curCapacity() const
{
	return m_Item->curCapacity();
}

const int Mp7JrsbeatType::elementAt(unsigned int getAt) const
{
	return m_Item->elementAt(getAt);
}

int Mp7JrsbeatType::elementAt(unsigned int getAt)
{
	return m_Item->elementAt(getAt);
}

unsigned int Mp7JrsbeatType::size() const
{
	return m_Item->size();
}

void Mp7JrsbeatType::ensureExtraCapacity(unsigned int length)
{
	m_Item->ensureExtraCapacity(length);
}

const int * Mp7JrsbeatType::rawData() const
{
	return m_Item->rawData();
}

int Mp7JrsbeatType::elementIndexOf(const int &toCheck) const
{
	for(unsigned int i = 0; i < m_Item->size(); i++)
	{
		if(m_Item->elementAt(i) == toCheck)
		{
			return (int) i;
		}
	}
	return -1; // Not found
}


XMLCh * Mp7JrsbeatType::ToText() const
{
	XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
	XMLCh * delim = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(" ");
	for(unsigned int i = 0; i < m_Item->size(); i++)
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_Item->elementAt(i));
		Mp7JrsUtil::CatString(&buf, tmp);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		if(i < m_Item->size() - 1)
		{
			Mp7JrsUtil::CatString(&buf, delim);
		}
	}

	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&delim);
	// Note: You must release this buffer
	return buf;
}

bool Mp7JrsbeatType::Parse(const XMLCh * const txt)
{
	if(txt == NULL)
	{
		return false;
	}
	XERCES_CPP_NAMESPACE_QUALIFIER BaseRefVectorOf<XMLCh> * tmp = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::tokenizeString(txt);
	if(tmp != NULL)
	{
		for(unsigned int i = 0; i < tmp->size(); i++)
		{
			m_Item->addElement(Mp7JrsConvert::TextToInt(tmp->elementAt(i)));
		}
	}
	delete tmp; // FTT release?
	return m_Item->size() > 0;
}

bool Mp7JrsbeatType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsbeatType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsbeatType::ContentToString() const
{
	// Mp7JrsbeatType has no attributes so forward it to ToText()
	return ToText();
}

/**

 * Item type is Empty.
 */
void Mp7JrsbeatType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
	// The element we serialize into
	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
	if (!newElem) newElem = &localNewElem;

	if (*newElem) {
		element = *newElem;
	} else if(parent) {
		element = parent;
		if(this->GetContentName() != (XMLCh*)NULL)
		{
// OLD 			XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
      DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			element->appendChild(elem);
			*newElem = elem;
			element = elem;
		}
	} else {
		element = doc->getDocumentElement();
	}
	*newElem = element;

	if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
	{
		// FTT This has to be solved in all classes using  setAttribute(X("xsi:type")!
		element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:beatType")))));
	}

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtPreSerialize.h


	for(unsigned int i = 0; i < m_Item->size(); i++)
	{
		XMLCh * txt = Mp7JrsConvert::BinToText(m_Item->elementAt(i));
		// sat 2004-04-14: removed creation of elements for each item in value collection
		// This could be wrong as well, but at least there are no duplicated tags in Mp7JrsintegerVector any more.
//		if(this->GetContentName() != NULL)
//		{
//			XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
//			elem->appendChild(doc->createTextNode(txt));
//			element->appendChild(elem);
//		}
//		else
//		{
			element->appendChild(doc->createTextNode(txt));
//		}
		if(i < m_Item->size() - 1)
		{
			element->appendChild(doc->createTextNode(X(" ")));
		}
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&txt);
	}
}





bool Mp7JrsbeatType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
	if(parent == NULL)
	{
		return false;
	}
	* current = parent; // Init with first element - check this!

	// Deserialize value type collection
	XERCES_CPP_NAMESPACE_QUALIFIER BaseRefVectorOf<XMLCh> * tmp = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::tokenizeString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		for(unsigned int i = 0; i < tmp->size(); i++)
		{
			m_Item->addElement(Mp7JrsConvert::TextToInt(tmp->elementAt(i)));
		}
		// TODO Check do we need elementnames?
	}
	delete tmp; // FTT release?

// no includefile for extension defined 
// file Mp7JrsbeatType_ExtPostDeserialize.h


	return true;
}
