 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtImplInclude.h



#include "Mp7JrscharacterSetCode.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrscharacterSetCode::IMp7JrscharacterSetCode()
{

// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtPropInit.h

}

IMp7JrscharacterSetCode::~IMp7JrscharacterSetCode()
{
// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtPropCleanup.h

}

Mp7JrscharacterSetCode::Mp7JrscharacterSetCode()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrscharacterSetCode::~Mp7JrscharacterSetCode()
{
	Cleanup();
}

void Mp7JrscharacterSetCode::Init()
{
	// Init base
	m_Base = XMLString::transcode("");




// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtMyPropInit.h

}

void Mp7JrscharacterSetCode::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtMyPropCleanup.h


	XMLString::release(&m_Base);
}

void Mp7JrscharacterSetCode::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use characterSetCodePtr, since we
	// might need GetBase(), which isn't defined in IcharacterSetCode
    const Mp7JrsPtr< Mp7JrscharacterSetCode > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	XMLString::release(&m_Base);
	m_Base = XMLString::replicate(tmp->GetContent());
}

Mp7JrsNodePtr Mp7JrscharacterSetCode::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrscharacterSetCode::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrscharacterSetCode::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscharacterSetCode::SetContent().");
	}
    if (m_Base != item)
    {
		XMLString::release(&m_Base);
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrscharacterSetCode::GetContent() const
{
	return m_Base;
}

Mp7JrsNodeEnum Mp7JrscharacterSetCode::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for characterSetCode
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "characterSetCode");
	}
	
	return result;
}

XMLCh * Mp7JrscharacterSetCode::ToText() const
{
	return XMLString::replicate(m_Base);
}


bool Mp7JrscharacterSetCode::Parse(const XMLCh * const txt)
{
	XMLString::release(&m_Base);
	m_Base = XMLString::replicate(txt);
	return true;
}

bool Mp7JrscharacterSetCode::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrscharacterSetCode has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrscharacterSetCode::ContentToString() const
{
	// Mp7JrscharacterSetCode has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrscharacterSetCode::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


    element->appendChild(doc->createTextNode(m_Base));

	assert(element);
// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("characterSetCode"));
	// Element serialization:

}

bool Mp7JrscharacterSetCode::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	XMLCh * tmp = Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		XMLString::release(&m_Base);
		m_Base = tmp;
		found = true;
	}
// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrscharacterSetCode::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrscharacterSetCode::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrscharacterSetCode_ExtMethodImpl.h


