 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2009, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtImplInclude.h



#include "Mp7JrscurvatureType_CollectionType.h"
#include "Mp7JrscurvatureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrscurvatureType::IMp7JrscurvatureType()
{

// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtPropInit.h

}

IMp7JrscurvatureType::~IMp7JrscurvatureType()
{
// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtPropCleanup.h

}

Mp7JrscurvatureType::Mp7JrscurvatureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrscurvatureType::~Mp7JrscurvatureType()
{
	Cleanup();
}

void Mp7JrscurvatureType::Init()
{
	// Init base
	m_Base = CreatecurvatureType_CollectionType;
	m_Base->m_myself = m_myself.getPointer();
	




// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtMyPropInit.h

}

void Mp7JrscurvatureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrscurvatureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use curvatureTypePtr, since we
	// might need GetBase(), which isn't defined in IcurvatureType
    const Mp7JrsPtr< Mp7JrscurvatureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrscurvatureType_CollectionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrscurvatureType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
}

Mp7JrsNodePtr Mp7JrscurvatureType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrscurvatureType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrscurvatureType_CollectionType > Mp7JrscurvatureType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrscurvatureType_CollectionType > Mp7JrscurvatureType::GetBaseCollection() const
{
	return GetBase();
}

void Mp7JrscurvatureType::Initialize(unsigned int maxElems)
{
	GetBase()->Initialize(maxElems);
}

void Mp7JrscurvatureType::addElement(const Mp7JrsNodePtr &toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscurvatureType::addElement().");
	}
	GetBase()->addElement(toAdd);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_ADDED,
		client, toAdd);
}

void Mp7JrscurvatureType::setElementAt(const Mp7JrsNodePtr &toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscurvatureType::setElementAt().");
	}
	GetBase()->setElementAt(toSet, setAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, toSet, setAt);
}

void Mp7JrscurvatureType::insertElementAt(const Mp7JrsNodePtr &toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscurvatureType::insertElementAt().");
	}
	GetBase()->insertElementAt(toInsert, insertAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_INSERTED,
		client, toInsert, insertAt);
}

void Mp7JrscurvatureType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscurvatureType::removeAllElements().");
	}
	GetBase()->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrscurvatureType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrscurvatureType::removeElementAt().");
	}
	GetBase()->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrscurvatureType::containsElement(const Mp7JrsNodePtr &toCheck)
{
	return GetBase()->containsElement(toCheck);
}

unsigned int Mp7JrscurvatureType::curCapacity() const
{
	return GetBase()->curCapacity();
}

const Mp7JrsNodePtr Mp7JrscurvatureType::elementAt(unsigned int getAt) const
{
	return GetBase()->elementAt(getAt);
}

Mp7JrsNodePtr Mp7JrscurvatureType::elementAt(unsigned int getAt)
{
	return GetBase()->elementAt(getAt);
}

Mp7JrsNodeEnum Mp7JrscurvatureType::GetElementsOfType(unsigned int id) const
{
	return GetBase()->GetElementsOfType(id);
}

Mp7JrsNodeEnum Mp7JrscurvatureType::GetElements() const
{
	return GetBase()->GetElements();
}


unsigned int Mp7JrscurvatureType::size() const
{
	return GetBase()->size();
}

void Mp7JrscurvatureType::ensureExtraCapacity(unsigned int length)
{
	GetBase()->ensureExtraCapacity(length);
}

int Mp7JrscurvatureType::elementIndexOf(const Mp7JrsNodePtr &toCheck) const
{
	return GetBase()->elementIndexOf(toCheck);
}


Mp7JrsNodeEnum Mp7JrscurvatureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for curvatureType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "curvatureType");
	}
	
	return result;
}

XMLCh * Mp7JrscurvatureType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrscurvatureType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrscurvatureType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrscurvatureType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrscurvatureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("curvatureType"));
	// Element serialization:

}

bool Mp7JrscurvatureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

  // Extensionbase is Mp7JrscurvatureType_CollectionType
  // Deserialize cce factory type
  if(m_Base->Deserialize(doc, parent, current))
  {
    found = true;
    parent = * current;
    parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
  }
// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrscurvatureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrscurvatureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrscurvatureType_ExtMethodImpl.h


